package eu.convergence.middleware.engine.ConetTe;

import eu.convergence.middleware.engine.ConetTe.exception.ConetIOException;
import eu.convergence.middleware.engine.ConetTe.services.*;

public interface ConetTE {
		
		public int NAMED_DATA_RTYPE = 1;
		public int NAMED_SAP_RTYPE = 2;
		
		public int NAMED_DATA_AS_BYTE_ARRAY = 1;
		public int NAMED_DATA_AS_STRING = 2;
		
		
		/**
		 * Stores a resource in the CoNet with a given NID (Network Identifier), using the 
		 * local node as a CoNet repository (thus assuming that the local
		 * node is a CoNet node that acts as a permanent storage of the resource).<BR>
		 * It also requests the CoNet to perform CoNet advertising operations in order 
		 * to associate the NID with the resource.
		 * @param nid is the CoNet identifier (name) of the resource to be stored
		 * @param rType (NOT USED) is for future extensions and can represent the resource type
		 * @param fileName is the local file name of the resource
		 * @return >=0 OK -1 generic error 
		 */
		public int storeAndAdvertise (NID nid, int rType, String fileName) ;

		
		/**
		 * Stores a resource in the CoNet with a given NID (Network Identifier), using the 
		 * given hostingNid node as a CoNet repository (thus assuming that hostingNid
		 * is a CoNet node that acts as a permanent storage of the resource).<BR>
		 * It also requests the CoNet to perform CoNet advertising operations in order 
		 * to associate the NID with the resource.
		 * @param nid is the CoNet identifier (name) of the resource
		 * @param rType (NOT USED) is for future extensions and can represent the resource type
		 * @param fileName is the local file name of the resource to be stored
		 * @param hostingNid the remote CoNet repository node
		 * @return >=0 OK; -1 generic error
		 */
		public int storeAndAdvertise (NID nid, int rType, String fileName, NID hostingNid) ;

		
		/**
		 * Revokes (flags as un-reachable) a specific resource, identified by its NID, in the CoNet.<BR>
		 * The resource is revoked from the local repository of the local CoNet node.
		 * @param nid is the CoNet identifier (name) of the resource to be revoked
		 * @param rType (NOT USED) is for future extensions and can represent the resource type
		 * @return >=0 OK; -1 generic error
		 */
		public int revoke (NID nid, int rType) ;
		
		/**
		 * Revokes (flags as un-reachable) a specific resource, identified by its NID, in the CoNet.<BR>
		 * The resource is revoked from the CoNet repository in the node specified by hostingNid.
		 * @param nid is the CoNet identifier (name) of the resource to be revoked
		 * @param rType (NOT USED) is for future extensions and can represent the resource type
		 * @param hostingNid is the remote CoNet repository node
		 * @return >=0 OK; -1 generic error
		 */
		public int revoke (NID nid, int rType, NID hostingNid) ;
		
		
		/**
		 * Retrieves the resource from the CoNet which corresponds to the given NID<BR>
		 * The retrieved named-resource is stored in a local file.<BR>
		 * This method is blocking, i.e. the method call does not exit until the resource is
		 * retrieved or an exception is thrown.
		 * @param nid is the CoNet identifier (name) of the resource to be fetched
		 * @param localFileName is a local file name where to save the fetched resource
		 * @return >=0 OK -1 generic error 
		 */
		public int get (NID nid, String localFileName) throws ConetIOException;
		
		/**
		 * Advertise a SAP (Service Access Point) for receiving unNamed data, under the given NID.
		 * A callBack to be invoked when unNamed data is received must be provided.<BR>
		 * By default, the received data will be delivered as byte array. This is equivalent to invoking
		 * the complete advertiseSap(nid, callBack, localDeliveryType) method
		 * with a localDeliveryType=NAMED_DATA_AS_BYTE_ARRAY value.
		 * @param nid is the CoNet identifier (name) of the named-sap
		 * @param callBack is a callBack method to be invoked upon receive
		 * @return always returns zero
		 */
		public int advertiseSap (NID nid, SAPListener callBack);

		/**
		 * Advertise a SAP (Service Access Point) for receiving unNamed data, under the given NID.
		 * A callBack to be invoked when unNamed data is received must be provided.<BR>
		 * The localDelivery type parameter allows to specify how the received
		 * unNamed data can be handled (e.g. as a byte array or a string).
		 * @param nid is the CoNet identifier (name) of the named-sap
		 * @param callBack is a callBack method to be invoked upon receive
		 * @param localDeliveryType possible values are NAMED_DATA_AS_STRING and NAMED_DATA_AS_BYTE_ARRAY
		 * @return always returns zero
		 */
		public int advertiseSap (NID nid, SAPListener callBack, int localDeliveryType) ;
		
		/**
		 * Sends unNamed data towards a NID, which most likely represents a SAP.<BR>
		 * The unnamed data is expressed as a byte array.
		 * @param nid is the CoNet identifier (name) of the named-sap
		 * @param unNamedData is the message to be sent
		 * @return >= 0 OK -1 generic error
		 */
		public int sendToName (NID nid, byte [] unNamedData) ;
		
		/**
		 * Sends unNamed data towards a NID, which most likely represents a SAP.<BR>
		 * The unnamed data is expressed as a String, which means that the 
		 * receiving entity shall be ready to receive the data as String.
		 * @param nid is the CoNet identifier (name) of the named-sap
		 * @param unNamedDataAsString is the message to be sent
		 * @return >= 0 OK -1 generic error
		 */
		public int sendToName (NID nid, String unNamedDataAsString) ;

		/**
		 * Sends unNamed data towards a LID.<BR>
		 * The unnamed data is expressed as a byte array<BR>
		 * NB FOR THE TIME BEING IT WILL NOT BE IMPLEMENTED
		 * @param lid
		 * @param unNamedData
		 * @return 0
		 */
		public int sendToLocation (LID lid, byte [] unNamedData) ;

		/**
		 * Sends unNamed data towards a LID.<BR>
		 * The unnamed data is expressed as a String this means that the 
		 * receiving entity is ready to receive the data as String<BR>
		 * NB FOR THE TIME BEING IT WILL NOT BE IMPLEMENTED
		 * @param lid
		 * @param unNamedDataAsString
		 * @return 0
		 */
		public int sendToLocation (LID lid, String unNamedDataAsString) ;			

	
}
