package eu.convergence.middleware.cdsTE;

import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.KeywordAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexNotFoundException;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.PrefixQuery;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.RAMDirectory;
import org.apache.lucene.util.Version;

import com.hp.hpl.jena.rdf.model.Statement;

public class EntityExplorer implements eu.convergence.middleware.engine.cdsTE.EntityExplorer
{
	private final String WORDNET_LABEL_PROPERTY = "http://www.w3.org/2006/03/wn/wn20/schema/senseLabel";
	private final String COREST_LABEL_PROPERTY = "http://www.ict-convergence.eu/CCO/hasCorestLabel";
	private final String WORDNET_DEFINITION_PROPERTY = "http://www.w3.org/2006/03/wn/wn20/schema/gloss";
	private final String RDFS_LABEL_PROPERTY = "http://www.w3.org/2000/01/rdf-schema#label";
	
	private final String RDF_TYPE = "http://www.w3.org/1999/02/22-rdf-syntax-ns#type";
	private final String RDF_CLASS = "http://www.w3.org/2000/01/rdf-schema#Class";
	private final String OWL_CLASS = "http://www.w3.org/2002/07/owl#Class";
	
	private static final String ENTITY_FIELD = "ENTITY_URI";
	private static final String ENTITY_LABEL_FIELD = "ENTITY_LABEL";
	
	private RepositoryManager repositoryManager;
	
	private Analyzer keywordAnalyzer;
	
	private HashMap<String,Directory> ontology2dir;
	
	public EntityExplorer(RepositoryManager repositoryManager)
	{
		this.repositoryManager = repositoryManager;

		ontology2dir = new HashMap<String,Directory>();
		keywordAnalyzer = new KeywordAnalyzer();
	}
	
	public void initEntityExplorer()
	{
		try 
		{
			initLabelIndex("wordnet");
			initLabelIndex("corest");
			initLabelIndex("static_vocabulary");
			initLabelIndex("escom_vocabulary");
			initLabelIndex("arc_root");
			initLabelIndex("pcia_root");
			initLabelIndex("alia_root");
		} 
		catch (CorruptIndexException e) 
		{
			e.printStackTrace();
		} 
		catch (IOException e)
		{
			e.printStackTrace();
		}
	}
	
	// label2uri
	public synchronized List<Map<String,List<String>>> requestOntologyEntity(String label, String ontologyEntityType, String ontologyURI)
	{
		List<Map<String,List<String>>> resultList = new ArrayList<Map<String,List<String>>>();
		
		Map<String,List<String>> label2uris = new HashMap<String,List<String>>();
		Map<String,List<String>> uri2glossary = new HashMap<String,List<String>>();
		Map<String,List<String>> uri2synonyms = new HashMap<String,List<String>>();
		
		if (ontology2dir.get(ontologyURI)!=null)
		{
			try
			{
				Directory ontologyEntityDir = ontology2dir.get(ontologyURI); // TODO use ontologyURI parameter	
				IndexSearcher entitySearcher = new IndexSearcher(ontologyEntityDir,true);
				
				Term term = new Term(ENTITY_LABEL_FIELD, label); // TODO maybe first an Exact Keyword Query
				PrefixQuery termQuery = new PrefixQuery(term);
				
				// TODO check the correct entityType
				
				ScoreDoc[] docIdHits = entitySearcher.search(termQuery,100).scoreDocs; // TODO parameter of search 100
				
				for (ScoreDoc docId : docIdHits)
				{
					Document doc = entitySearcher.doc(docId.doc);
					
					String entity = doc.getValues(ENTITY_FIELD)[0];
					String entityLabel = doc.getValues(ENTITY_LABEL_FIELD)[0];
					
					if (ontologyURI.equals("wordnet"))
					{
						if (uri2synonyms.get(entity)==null)
						{
							uri2synonyms.put(entity, new ArrayList<String>());
							List<Statement> statements = repositoryManager.getStatementsWithPattern(entity, WORDNET_LABEL_PROPERTY, null, ontologyURI);
							
							for (Statement statement : statements)
							{
								uri2synonyms.get(entity).add(statement.getLiteral().getString());
							}
						}
						
						if (uri2glossary.get(entity)==null)
						{
							uri2glossary.put(entity, new ArrayList<String>());
							List<Statement> glossaryStatements = repositoryManager.getStatementsWithPattern(entity, WORDNET_DEFINITION_PROPERTY, null, ontologyURI);
							
							String glossary = "";
							if (glossaryStatements.size()>0)
							{
								glossary = glossaryStatements.get(0).getLiteral().getString();
							}
						
							uri2glossary.get(entity).add(glossary);
						}
					}
					else
					{
						if (ontologyEntityType!=null && !ontologyEntityType.equals("")) // psaxno sigekrimeno tipo (kai klassi)
						{
							if (!repositoryManager.getStatementsWithPattern(entity, RDF_TYPE, ontologyEntityType, ontologyURI).isEmpty())
							{
								if (label2uris.get(entityLabel)==null)
								{
									label2uris.put(entityLabel, new ArrayList<String>());
								}
								
								label2uris.get(entityLabel).add(entity);
							}
						}
						else // psaxno individual
						{
							if (repositoryManager.getStatementsWithPattern(entity, RDF_TYPE, RDF_CLASS, ontologyURI).isEmpty() &&
								repositoryManager.getStatementsWithPattern(entity, RDF_TYPE, OWL_CLASS, ontologyURI).isEmpty())
							{
								if (label2uris.get(entityLabel)==null)
								{
									label2uris.put(entityLabel, new ArrayList<String>());
								}
								
								label2uris.get(entityLabel).add(entity);
							}
						}
					}
				}
				
				entitySearcher.close();
			}
			catch (IndexNotFoundException ex)
			{
				ex.printStackTrace();
			}
			catch (IOException ex)
			{
				ex.printStackTrace();
			}
		}
		else
		{
			System.err.println("Ontology " + ontologyURI + " isn't loaded. Cannot return results");
		}
		
		resultList.add(label2uris);
		resultList.add(uri2glossary);
		resultList.add(uri2synonyms);
		
		return resultList;
	}
	
	private void initLabelIndex(String ontologyURI) throws CorruptIndexException, IOException
	{
		ontology2dir.put(ontologyURI, new RAMDirectory());
		
		IndexWriter labelWriter = new IndexWriter(ontology2dir.get(ontologyURI), 
								  new IndexWriterConfig(Version.LUCENE_33, keywordAnalyzer));
		
		List<Statement> statements = new ArrayList<Statement>();
		if (ontologyURI.equals("wordnet"))
		{
			statements = repositoryManager.getStatementsWithPattern(null, WORDNET_LABEL_PROPERTY, null, ontologyURI);
		}
		else if (ontologyURI.equals("corest"))
		{
			statements = repositoryManager.getStatementsWithPattern(null, COREST_LABEL_PROPERTY, null, ontologyURI);
		}
		else if (ontologyURI.equals("static_vocabulary"))
		{
			statements = repositoryManager.getStatementsWithPattern(null, RDFS_LABEL_PROPERTY, null, ontologyURI);
		}
		else if (ontologyURI.equals("escom_vocabulary"))
		{
			statements = repositoryManager.getStatementsWithPattern(null, RDFS_LABEL_PROPERTY, null, ontologyURI);
		}
		else if (ontologyURI.equals("pcia_root"))
		{
			statements = repositoryManager.getStatementsWithPattern(null, RDFS_LABEL_PROPERTY, null, ontologyURI);
		}
		else if (ontologyURI.equals("arc_root"))
		{
			statements = repositoryManager.getStatementsWithPattern(null, RDFS_LABEL_PROPERTY, null, ontologyURI);
		}
		else if (ontologyURI.equals("alia_root"))
		{
			statements = repositoryManager.getStatementsWithPattern(null, RDFS_LABEL_PROPERTY, null, ontologyURI);
		}
		
		int count = 0;
		for(Statement statement : statements) // TODO exo polla doc gia to idio entity
		{
			String entity = statement.getSubject().getURI();
			String entityLabel = statement.getLiteral().getString();
			
//			if ("en".equals(statement.getLiteral().getLanguage())) // TODO FMSH specific
//			{
				count++;
				Document doc = new Document();
				
				doc.add(new Field(ENTITY_FIELD, entity, Field.Store.YES, Field.Index.NO));
				doc.add(new Field(ENTITY_LABEL_FIELD, entityLabel.toLowerCase().trim(), Field.Store.YES, Field.Index.NOT_ANALYZED)); // TODO mipos na ginetai tokenize ?
				
				labelWriter.addDocument(doc);
//			}
		}
		
		labelWriter.optimize();
		labelWriter.close();
		
		System.out.println("Entitiy label index for " + ontologyURI + " with " + count + " entities created successfully !");
	}
	
}
