/*
SMILE: a Simple Middleware Independent LayEr 
Author: Giovanni Bartolomeo

Copyright (C) 2007  University of Rome Tor Vergata - Dipartimento di Ingegneria Elettronica


This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

package it.uniroma2.smile.sipbinding;

import java.util.Hashtable;
import java.util.Stack;
import java.util.Vector;
import org.json.me.JSONException;
import org.json.me.JSONObject;
import org.istsms.util.Javabean2JSON;
import org.zoolu.sip.address.NameAddress;
import org.zoolu.sip.address.SipURL;
import org.zoolu.sip.provider.SipProvider;

import it.uniroma2.smile.core.Broker;
import it.uniroma2.smile.core.Fault;
import it.uniroma2.smile.core.SendingMessageException;
import it.uniroma2.smile.core.Message;
import it.uniroma2.smile.core.Process;
import it.uniroma2.smile.core.ProcessID;
import it.uniroma2.smile.sipbinding.fragmentation.*;
import it.uniroma2.smile.sipbinding.sipmessage.MessageEnvelope;
import it.uniroma2.smile.sipbinding.util.StringTokenizer;
import java.util.Enumeration;

import local.ua.UserAgentProfile;

/**
 * 
 * @author Giovanni Bartolomeo
 */

public abstract class SipBaseProcess implements Process, DefragmenterListener {

	private final boolean DEBUG=true;
	private Dispatcher dispatcher;
	protected ProcessID pID;
	private int seqNo=0; //used to progressively mark messages	
	private Hashtable<Integer, Reply> replyTable=new Hashtable<Integer, Reply>();
	private Stack<String> contacts=new Stack<String>();
	private final int MAX_CONTACTS_SIZE=200;	

	private class Reply {
		boolean waitingState=false;
		int expectedReplySeqNo;
		Message reply;

	}

	private Hashtable<String, Defragmenter> defragTable = new Hashtable<String, Defragmenter>();

	private void deleteDefragmenter(NameAddress remote_user) {
		synchronized (defragTable) {
			if (defragTable.remove(remote_user.toString()) != null) {
//				log("Defragmenter for " + remote_user + " deleted.");
			} else {
//				log("Warning: Defragmenter for " + remote_user + " does not exist and cannot be deleted.");
			}
		}
	}

	//timeout for each request, could be changed by specifying pTimeout=XXX (millis)
	public int REQ_TIME_OUT=30000;


	public SipBaseProcess(String configFileName) {      
		create(configFileName, (String) null,REQ_TIME_OUT); 
		log("created process from file "+configFileName);    	
	}	

	public SipBaseProcess(String configFileName,int req_timeout) {    	  
		create(configFileName, (String) null, req_timeout);  
		log("created process from file "+configFileName+" application req timeout is "+req_timeout);

	}

	
	public SipBaseProcess(String configFileName,String params,int req_timeout) {    	
		create(configFileName, params, req_timeout);
		log("created process from file "+configFileName+" and parameters "+params);    	
	}

	public SipBaseProcess(String configFileName, SipProcessID sipPID, int req_timeout) {    	
		create(configFileName, sipPID, req_timeout);
		
		log("created process from file "+configFileName+" and sipPID "+sipPID.toString());    	
	}

	

	/**
	 * it is called in the constructor of SipBaseProcess<br>
	 * it creates a user agent profile<br>
	 * then it instantiates or gets the dispatcher associated to the baseURI<br>
	 * when a dispatcher is instantiated the SIP provider is instantiated as well<br>
	 * then it register the specific URI (composed adding the params to the baseURI)
	 * to the dispatcher 
	 * 
	 * @param configFileName
	 * @param params
	 * @param req_timeout
	 */
	private void create(String configFileName,String params,int req_timeout) {
		create (configFileName,params, null, req_timeout);
	}

	private void create(String configFileName, SipProcessID sipPID, int req_timeout) {
		create (configFileName, null, sipPID, req_timeout);
	}

	/**
	 * either params or sipPID MUST be null and the other MUST be not null<br>
	 * if params is not null, the base SIP address is taken from the configuration file<br>
	 * if sipPID is not null, it defines the complete address, including both the base SIP address
	 * and the parameters pname and ptype
	 * @param configFileName
	 * @param params
	 * @param sipPID
	 * @param req_timeout
	 */
	private void create(String configFileName, String params, SipProcessID sipPID, int req_timeout) {
		if (configFileName==null) {
			log("ERROR: config file name is null.");
			return;
		}
		UserAgentProfile uap=new UserAgentProfile(configFileName);	
		if (uap==null) {
			log("ERROR: no user agent profile can be created. Please check "+configFileName);
			return;
		}

//		SipProvider sp=new SipProvider(configFileName);			
//		if (sp==null) {
//			log("ERROR: no sip provider can be created. Please check "+configFileName);
//			return;
//		}		

		REQ_TIME_OUT=req_timeout;  
		
		if (sipPID == null) {
			dispatcher=Dispatcher.getInstance(uap, configFileName, null);  

			SipURL sipUrl=new SipURL(dispatcher.getMyNameAddress().getAddress().getUserName(),
					dispatcher.getMyNameAddress().getAddress().getHost(), 
					dispatcher.getMyNameAddress().getAddress().getPort());

			if (params!=null && !params.equals("")) { 
				StringTokenizer stringTokenizer=new StringTokenizer();
				String[] temp=stringTokenizer.getTokensS(params,";");
				for(int i=0; i < temp.length; i++) {
					if (temp[i].length()>3){
						String[] paramsArray=stringTokenizer.getTokensS(temp[i],"=");
						if (paramsArray.length>1) {
							sipUrl.addParameter(paramsArray[0],paramsArray[1]);
						}
					}
				}
			}
//			System.out.println("creating nameAddress...");	        
			pID=new SipProcessID();

			NameAddress nameAddress=new NameAddress(dispatcher.getMyNameAddress().getDisplayName(), sipUrl);                
			((SipProcessID)pID).setNameAddress(nameAddress);        
			
		} else {
			//if sipPID is not null, either we set the base sip address in the uap
			//taking it from sipPID, or we keep the uap with the default sip base address taken
			//from file and use a new getInstance method that includes the sipPID
			pID = sipPID;
			dispatcher=Dispatcher.getInstance(uap, configFileName, (SipProcessID) pID); 
		}
		
		dispatcher.registerToDispatcher(this, pID);
		log("Registered SMILE process with SIP address: "+((SipProcessID)pID).getNameAddress().getAddress().toString());
		
	}

	
	public void deregister() {
		dispatcher.degisterFromDispatcher(pID);
	}              

	public void onMessageReceived(NameAddress sender,String content, String callID){    	
		//log("***received msg from: "+sender.getAddress().toString()+" content: "+content);
//		MessageEnvelope mInfo;
		MessageEnvelope mInfo=new MessageEnvelope();
//		try {
//			mInfo = MessageEnvelope.fromJSONObject(new JSONObject(content));
			mInfo.fromString(content);
			//check whether this message has been already received	    	      
			if (contacts.contains(callID)) {
				log("out of order message: callID=" + callID + " was already received");
				return;
			}
			contacts.push(callID);
			if (contacts.size() > MAX_CONTACTS_SIZE) {
				contacts.removeElementAt(0);
			}			

			Defragmenter defrag = defragTable.get(sender.toString());
			if (defrag == null) {
				defrag = new Defragmenter(sender, this);
				synchronized (defragTable) {
					defragTable.put(sender.toString(), defrag);
				}
//				log("Created defragmenter for " + sender + ".");
			}
			defrag.messageReceived(mInfo);


//		} catch (JSONException e) {
//			log("Parsing problem, discarding message: " + content);
			// TODO Auto-generated catch block
//			e.printStackTrace();
//		}			
	}			

	public void onDefragmentedMessage(Defragmenter defragmenter, NameAddress sender, MessageEnvelope mInfo) {
		if (defragmenter.isEmpty()) deleteDefragmenter(sender);
//		else log("Defragmenter for " + sender + " has incoming messages: not deleted.");
		SipProcessID s = new SipProcessID();
		s.setNameAddress(sender);
		Message m=null;
		try {
			m = (Message)Javabean2JSON.fromJSONObject(new JSONObject(mInfo.getFrag()));
		} catch (JSONException e1) {
			// TODO Auto-generated catch block
			log("impossible to deserialize message "+mInfo.getFrag());
			return;
		}

		if (mInfo.getRefS()>-1) {
			Reply expectedReply=replyTable.get(mInfo.getRefS());
			if (expectedReply==null) {
				log("out of order reply: refSerial="+mInfo.getRefS()+" nothing was expected");		
				return;
			}
			if (expectedReply.waitingState) {
				synchronized (expectedReply) {        	
					if (mInfo.getRefS()==expectedReply.expectedReplySeqNo) {         	
						expectedReply.reply=m;
						expectedReply.waitingState=false;						
						expectedReply.notify();
					} else {
						//do nothing, discard the response message silently (out of order response)
						log("out of order reply: refSerial="+mInfo.getRefS()+" was expected: "+expectedReply.expectedReplySeqNo);
					}
					return;
				}
			} else {
				log("out of order reply: refSerial="+mInfo.getRefS()+", this process was not expecting any reply");        			
			}
		} else {
			if (mInfo.isReq())
				try {
					MessageEnvelope serMess=toJsonMessage(onRequestReceived(s, m));
					serMess.setRefS(mInfo.getSNo());
					send(serMess,s);												
				} catch (SendingMessageException e) {
					// TODO Auto-generated catch block
					e.printStackTrace();
				}
				else onReceived(s, m, null);
		}
	}

	public void onDefragmentationTimeout(Defragmenter defragmenter, NameAddress remote_user) {
		deleteDefragmenter(remote_user);
	}	

	public ProcessID getProcessID() {
		return pID;
	}    

	public void send(Message message, ProcessID[] receivers)
	throws SendingMessageException {
		Vector<ProcessID> invalid = null;
		for (int i = 0; i < receivers.length; i++) {
			try {
				send(message, receivers[i]);
			} catch (SendingMessageException e) {				
				if (invalid == null) invalid = new Vector<ProcessID>();				
				invalid.add(e.getAddress()[0]);
			}
		}
		if (invalid != null) {
			ProcessID[] invs = new ProcessID[invalid.size()];
			for (int i = 0; i < invalid.size(); i++) {
				invs[i] = invalid.elementAt(i);
			}
			throw new SendingMessageException(invs);
		}
	}     

	public void send(Message message, ProcessID receiver) throws SendingMessageException {				
	SipProcessID receiverSippID = (SipProcessID) receiver;
		
		if (receiverSippID.equals(pID) ) {
			onReceived(pID, message, null);
			return;
		} else if (receiverSippID.isLocal()) {
//			if (Broker.processTable.containsKey(sippID.toString())) {
				Process myProcess = (Process)(Broker.processTable.get(receiverSippID.toString()));
				Broker.getDefaultBroker().onReceived(myProcess, pID, message);
				return;
//			}			
		}
		
		MessageEnvelope serMess=toJsonMessage(message);
		send(serMess,receiver);		
	}

	public Message doRequest(Message request, ProcessID receiver) throws Fault,
	SendingMessageException {
		try {
			SipProcessID receiverSippID = (SipProcessID) receiver;
			
			
			if (receiverSippID.equals(pID) ) {
				Message response=onRequestReceived(pID, request);
				if (response.isFault()) throw new Fault(response);
				return response;
			} else if (receiverSippID.isLocal()) {
//				if (Broker.processTable.containsKey(sippID.toString())) {
					Process myProcess = (Process)(Broker.processTable.get(receiverSippID.toString()));
					Message response= Broker.getDefaultBroker().onRequestReceived(myProcess, pID, request);
					return response;
//				}
			}
			
			
			MessageEnvelope req=toJsonMessage(request);
			Reply expectedReply=new Reply();
			synchronized(expectedReply) {				
				req.setReq(true);
				expectedReply.expectedReplySeqNo=req.getSNo();				
				expectedReply.waitingState=true;	
				replyTable.put(expectedReply.expectedReplySeqNo, expectedReply);
				send(req, receiver);			   
				try {			    	
					expectedReply.wait(REQ_TIME_OUT);
				} catch (InterruptedException e) {									
					e.printStackTrace();					
				}
				replyTable.remove(expectedReply.expectedReplySeqNo);
				if (expectedReply.waitingState) {
					log("***timeout expired!");
					expectedReply.waitingState=false;			    	
					throw new SendingMessageException(new ProcessID[] { receiver });
				}
			}										
			if (expectedReply.reply.isFault()) throw new Fault(expectedReply.reply);
			return expectedReply.reply;							
		} catch (ClassCastException e) {
			//e.printStackTrace();			
			//no registered process can be found matching the recipient PID
			//when performing a doRequest, throw an InvalidReceiverAddressException
			throw new SendingMessageException(new ProcessID[] { receiver });
		}
	}	

	private void send(MessageEnvelope jsonMessage, ProcessID receiver) throws SendingMessageException {		
		//String serMess=jsonMessage.toString();		
		try {
			Fragmenter fragmenter = new Fragmenter();
			fragmenter.send(dispatcher, ((SipProcessID)receiver).getNameAddress(), ((SipProcessID)pID).getNameAddress(), jsonMessage);
			//log("sending "+jsonMessage.toString()+" to "+remote_user);
			//dispatcher.send(((SipProcessID)receiver).getNameAddress(), ((SipProcessID)pID).getNameAddress(), serMess);		
		} catch (ClassCastException e) {
			//throw an InvalidReceiverAddressException if and only if
			//the address is invalid (it is not a SipProcessID)	
			e.printStackTrace();
			throw new SendingMessageException(new ProcessID[] {receiver});
		}			
	}

	private MessageEnvelope toJsonMessage(Message m) {
		//TODO: implement the conversion
		MessageEnvelope jm=new MessageEnvelope();
		jm.setFrag(Javabean2JSON.toJSONObject(m).toString());
		jm.setSNo(seqNo++);
		return jm;
	}

	protected void log(String string) {
		System.out.println("["+pID+"] "+string);		
	}    	
}
