/*
 * MOVE application (Mobile Open & Very Easy)
 *
 * Department of Electronic Engineering - University of Rome "Tor Vergata"
 * Simple Mobile Services project (http://www.ist-sms.org/)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read LICENSE in the root of this distribution.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU General Public License
 * along with MjSip; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * As per the GPL, removal of this notice is prohibited.
 */

package it.uniroma2.smile.sipbinding.fragmentation;

import it.uniroma2.smile.sipbinding.sipmessage.MessageEnvelope;
import java.util.Hashtable;
import java.util.Timer;
import java.util.TimerTask;
import org.zoolu.sip.address.NameAddress;


public class Defragmenter implements SlidingWindowListener {

	private static final int TIMEOUT = 10000;  // milliseconds

	private Hashtable messages;
	private Hashtable slidingWindows;
	private Hashtable timeoutTasks;
	private NameAddress remote_user;

	private Timer timer;

	private DefragmenterListener listener;

	private String id;  // used to track this defragmenter in log messages

	private int GC_TIMES=0;


	public Defragmenter(NameAddress remote) {
		remote_user = remote;
		timer = new Timer();
		id = this.toString().substring(this.toString().indexOf("@")+1);
		messages = new Hashtable();
		slidingWindows = new Hashtable();
		timeoutTasks = new Hashtable();
	}

	public Defragmenter(NameAddress remote, DefragmenterListener l) {
		this(remote);
		setListener(l);
	}

	public synchronized void messageReceived(MessageEnvelope message) {

		Integer seq = new Integer(message.getSNo());

		// update timeout timer...
		TimerTask task = (TimerTask) timeoutTasks.get(seq);
		if (task != null)
			task.cancel();

		task = new DefragmentationTimeout(this, seq);
		timeoutTasks.put(seq, task);
		timer.schedule(task, TIMEOUT);

		// get the sliding window for this sequence number...
		SlidingWindow slidingWindow = (SlidingWindow) slidingWindows.get(seq);

		if (slidingWindow == null) {
			slidingWindow = new SlidingWindow();
			slidingWindow.setListener(this);
			slidingWindows.put(seq, slidingWindow);
			log("Created sliding windows for message " + seq + ".");
		}

		slidingWindow.fragmentReceived(message);
	}

	public synchronized void onSlidingWindowMessageReceived(MessageEnvelope fragment) {

		// extract fragmentation infos...
		int fragNo = fragment.getFNo();
		int fragTot = fragment.getFTot();
		Integer seqNo = new Integer(fragment.getSNo());

		MessageEnvelope message;

		if (fragNo == 0) {  // first fragment received
			message = fragment; // in order to have a consistent naming
			messages.put(seqNo, message);
			log("Message reconstructed so far: " + message.getFrag());
		} else {
			message = (MessageEnvelope) messages.get(seqNo);
			message.setFrag(message.getFrag() + fragment.getFrag());
			messages.put(seqNo, message);
			log("Message reconstructed so far: " + message.getFrag());
		}

		if (fragNo == fragTot) {
			// remove timeout timer
			TimerTask task = (TimerTask) timeoutTasks.remove(seqNo);
			if (task != null) {
				task.cancel();
				timer.purge();
			}
			// remove fully received message from hashtable
			messages.remove(seqNo);
			slidingWindows.remove(seqNo);//GB 23.06.08
			log("[DEFRAG] ***  Full message received: " + message.getFrag());
			GC_TIMES++;			
			printMEM();			
			if (listener != null) {				
				new InternalDispatchingThread(this, remote_user, message,listener).start();
				//listener.onDefragmentedMessage(this, remote_user, message);
			}
		}
	}

	private class InternalDispatchingThread extends Thread {
		Defragmenter _defragmenter;
		NameAddress _remote_user;
		MessageEnvelope _message;
		DefragmenterListener _listener;

		public InternalDispatchingThread(Defragmenter _defragmenter, NameAddress _remote_user, MessageEnvelope _message,DefragmenterListener _listener) {
			this._defragmenter=_defragmenter;
			this._message=_message;
			this._remote_user=_remote_user;
			this._listener=_listener;
		}

		public void run() {
			_listener.onDefragmentedMessage(_defragmenter,_remote_user,_message);
		}
	}    

	public void onSlidingWindowTransmissionFailed() {
	}

	public boolean isEmpty() {
		return messages.isEmpty();
	}

	public DefragmenterListener getListener() {
		return listener;
	}

	public void setListener(DefragmenterListener l) {
		listener = l;
	}



	private class DefragmentationTimeout extends TimerTask {

		private Defragmenter fragmenter;
		private Integer sequence;  // identifies which message has to be deleted

		DefragmentationTimeout(Defragmenter f, Integer seq) {
			fragmenter = f;
			sequence = seq;
		}

		public void run() {
			log("Warning: Defragmentation timeout for message " + sequence + ".");
			messages.remove(sequence);			
			slidingWindows.remove(sequence);//GB 23.06.08
			timeoutTasks.remove(sequence);//GB 23.06.08		
			GC_TIMES++;
			System.out.println("Defrag Timeout");
			printMEM();
			// if there are no messages left, we can delete the fragmenter
			if (messages.isEmpty())
				if (listener != null)
					listener.onDefragmentationTimeout(fragmenter, remote_user);
		}       
	}

	private void log(String message) {
		if (Fragmenter.DEBUG)  System.out.println("[Defragmenter][" + id + "] " + message);
	}
	
	private void callGC() {
		if (GC_TIMES>40) {
			System.gc();
			GC_TIMES=0;
			printMEM();
		}
	}
	
	void printMEM() {
//				System.out.println(" *** Memory:\n used "+ (Runtime.getRuntime().totalMemory()- Runtime.getRuntime().freeMemory())+
//				"\n avail: "+Runtime.getRuntime().freeMemory()+
//				"\n total: "+Runtime.getRuntime().totalMemory());		
	}
}