/*
 * MOVE application (Mobile Open & Very Easy)
 *
 * Department of Electronic Engineering - University of Rome "Tor Vergata"
 * Simple Mobile Services project (http://www.ist-sms.org/)
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version. For
 * more details read LICENSE in the root of this distribution.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
 *
 * You should have received a copy of the GNU General Public License
 * along with MjSip; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * As per the GPL, removal of this notice is prohibited.
 */

package it.uniroma2.smile.sipbinding.fragmentation;


import java.util.Hashtable;
import it.uniroma2.smile.sipbinding.sipmessage.MessageEnvelope;
import org.zoolu.sip.message.Message;
import org.zoolu.sip.transaction.TransactionClient;
import org.zoolu.sip.transaction.TransactionClientListener;


public class SlidingWindow implements TransactionClientListener {
    
    private static final short WINDOW_SIZE = 4;
    
    private short maxSequence;
    private short windowSize;
    
    // sender window
    private short ackExpected;
    private short nextFrameToSend;
    
    // receiver window
    private short frameExpected;
    private short tooFar = WINDOW_SIZE;
    
    private MessageEnvelope[] receiveBuffer;
    private short nbuffered;
    private boolean[] arrived;
    private boolean[] sent;
    private Hashtable callIDs;  // used to store sent messages call-ids, needed for ack management
    
    private SlidingWindowListener listener;
    
    
    private String id;  // used to track this sliding window in log messages
    

    public SlidingWindow() {
        this(WINDOW_SIZE);
    }

    public SlidingWindow(short size) {
    	callIDs=new Hashtable();
        setWindowSize(size);
        id = this.toString().substring(this.toString().indexOf("@")+1);
    }
    
    public short getWindowSize() {
        return windowSize;
    }
    
    public void setWindowSize(short s) {
        windowSize = s;
        maxSequence = (short) ((windowSize*2)-1);
        receiveBuffer = new MessageEnvelope[windowSize];
        arrived = new boolean[windowSize];
        sent = new boolean[windowSize];
        for (int i=0; i < windowSize; i++) {
            arrived[i] = false;
            sent[i] = false;
        }
    }
    
    public void setListener(SlidingWindowListener l) {
        listener = l;
    }
    
    public SlidingWindowListener getListener() {
        return listener;
    }

    public synchronized void addFragment(Message message) {
        
        if (isFull()) {
            log("Buffer full: waiting...");
            try {
                wait();
            } catch (InterruptedException e) {
            }
        }
        
        nbuffered++;
        callIDs.put(message.getCallIdHeader().getCallId(), new Integer(nextFrameToSend % (maxSequence+1)));
        nextFrameToSend = increment(nextFrameToSend);
        log("Message sent: " + message.getBody() + " with Call-ID: " + message.getCallIdHeader().getCallId());
        
    }

    public synchronized void fragmentReceived(MessageEnvelope message) {

        short seq = (short) (message.getFNo() % (maxSequence+1));
        
        log("Message received with sequence number: " + seq + ", expected: " + frameExpected);

        if (isBetween(frameExpected, seq, tooFar) && arrived[seq % windowSize] == false) {
            arrived[seq % windowSize] = true;
            receiveBuffer[seq % windowSize] = message;
            while (arrived[frameExpected % windowSize]) {  // send messages to the upper level, in order
                MessageEnvelope mm = receiveBuffer[frameExpected % windowSize];
                arrived[frameExpected % windowSize] = false;
                frameExpected = increment(frameExpected);
                tooFar = increment(tooFar);
                log("Message received: " + mm);
                if (listener != null)
                    listener.onSlidingWindowMessageReceived(mm);
            }
        } else {
            log("Warning: Message received with an unexpected sequence number: discarded. (seq=" + seq + ")");
        }

    }

    public synchronized void ackReceived(Message message) {
        
        String callid = message.getCallIdHeader().getCallId();
        short seq = getSequenceNumber(callid);
        
        if (seq == -1) {
            log ("Warning: ACK received with an invalid call-id.");
            return;
        }
        
        if (isBetween(ackExpected, seq, nextFrameToSend)) {
            log("ACK received for message with sequence number " + seq);
            sent[seq % windowSize] = true;
            while (sent[ackExpected % windowSize]) {
                sent[ackExpected % windowSize] = false;
                ackExpected = increment(ackExpected);
                nbuffered--;
                notify();
            }
        } else {
            log("Warning: ACK received with an unexpected sequence number: discarded. (seq: " + seq + ")");
        }
        
    }
    
    public synchronized void ackNotReceived(TransactionClient tc, String reason) {
        
        String callid = tc.getRequestMessage().getCallIdHeader().getCallId();
        short seq = getSequenceNumber(callid);
        
        if (seq == -1)
            return;
        
        notify();
        log("Warning: ACK for message " + seq + " not received. " + reason);
        
        if (listener != null)
            listener.onSlidingWindowTransmissionFailed();
        
    }
    
    public void onTransProvisionalResponse(TransactionClient tc, Message response) {
        log("Provisional response.");
    }

    public void onTransSuccessResponse(TransactionClient tc, Message response) {
        ackReceived(response);
    }

    public void onTransFailureResponse(TransactionClient tc, Message response) {
        ackNotReceived(tc, "Transmission failure: " + response.getFirstLine());
    }

    public void onTransTimeout(TransactionClient tc) {
        ackNotReceived(tc, "Timeout.");
    }

    private boolean isBetween(short a, short b, short c) {
        return ((a<=b) && (b<c)) || ((c<a) && (a<=b)) || ((b<c) && (c<a));
    }
    
    private boolean isFull() {
        return nbuffered < windowSize ? false : true;
    }
    
    private short increment(short n) {
        if (n < maxSequence)
            return ++n;
        return 0;
    }
    
    private short getSequenceNumber(String callid) {
        
        Integer s = (Integer) callIDs.get(callid);
        if (s == null)
            return -1;
        else
            return s.shortValue();
        
    }

    private void log(String message) {
    	if (Fragmenter.DEBUG) System.out.println("[Sliding-Window][" + id + "] " + message);
    }
    
}