package local.ua;


import org.zoolu.net.SocketAddress;
import org.zoolu.sip.address.*;
import org.zoolu.sip.provider.SipStack;
import org.zoolu.sip.provider.SipProvider;
import org.zoolu.tools.Configure;
import org.zoolu.tools.Parser;

import it.uniroma2.smile.sipbinding.SipProcessID;

import java.util.Vector;


/** UserAgentProfile maintains the user configuration.
 */
public class UserAgentProfile extends Configure {
    /** The default configuration file */
    private static String config_file="mjsip.cfg";
    
    
    // ********************** user configurations *********************
    
    /** Display name for the user.
     * It is used in the user's AOR registered with the registrar server
     * and used as From URL. */
    public String display_name=null;
    
    /** User's name.
     * It is used to build the user's AOR (used as the From URL) and Contact URL in case
     * they are not explicitely defined respectively by means of the <i>from_url</i>
     * and <i>contact_url</i> parameters. */
    public String user=null;
    
    /** Fully qualified domain name (or address) of the proxy server.
     * It is part of the user's AOR registered with the registrar server
     * and used as From URL.
     * <p/>
     * If <i>proxy</i> is not defined, the <i>registrar</i> value is used in its place.
     * <p/>
     * If <i>registrar</i> is not defined, the <i>proxy</i> value is used in its place. */
    public String proxy=null;
    
    /** These is a string containing a ;-separated list of parameters used to build the SIP URI, e.g. francesco@iptel.uniroma2.it:5060;p1=forward;p2=none */
    public String params;
    
    /** Fully qualified domain name (or address) of the registrar server.
     * It is used as recipient for REGISTER requests.
     * <p/>
     * If <i>registrar</i> is not defined, the <i>proxy</i> value is used in its place.
     * <p/>
     * If <i>proxy</i> is not defined, the <i>registrar</i> value is used in its place. */
    public String registrar=null;
    
    /** Contact URL.
     * If not defined (default), it is formed as sip:<i>user</i>@<i>via_address</i>:<i>host_port</i>.
     * <p/>
     * Normally you can leave it undefined. */
    public NameAddress contact_url=null;
    
    /** User's name used for server authentication. */
    public String auth_user=null;
    /** User's realm used for server authentication. */
    public String auth_realm=null;
    /** User's passwd used for server authentication. */
    public String auth_passwd=null;
    
    /** Path for the 'ua.jar' lib, used to retrive various UA media (gif, wav, etc.).
     * Use value 'NONE' for getting resources from external folders.
     * By default, it is "lib/ua.jar". */
    public static String ua_jar="lib/ua.jar";
    
    /** Path for the media resources (gif, wav, etc.), used by the UA.
     * By default, it is the "media/local/ua/" folder. */
    public static String media_path="media/local/ua/";
    
    /** Path for the 'contacts.lst' file where save and load the VisualUA contacts
     * By default, it is used the "config/contacts.lst" folder */
    public static String contacts_file="contacts.lst";
    
    /** Whether registering with the registrar server */
    public boolean do_register=false;
    /** Whether unregistering the contact address */
    public boolean do_unregister=false;
    /** Whether unregistering all contacts beafore registering the contact address */
    public boolean do_unregister_all=false;
    /** Expires time (in seconds). */
    public int expires=3600;
    
    /** Rate of keep-alive tokens (datagrams) sent toward the outbound proxy
     * (if present) or toward the registrar server.
     * Its value specifies the delta-time (in millesconds) between two
     * keep-alive tokens. <br/>
     * Set keepalive_time=0 for not sending keep-alive datagrams. */
    public long keepalive_time=0;
    
    /** Automatic call a remote user secified by the 'call_to' value.
     * Use value 'NONE' for manual calls (or let it undefined).  */
    public NameAddress call_to=null;
    
    /** Response time in seconds; it is the maximum time the user can wait before responding to an incoming call; after such time the call is automatically declined (refused). */
    public int refuse_time=20;
    /** Automatic answer time in seconds; time<0 corresponds to manual answer mode. */
    public int accept_time=-1;
    /** Automatic hangup time (call duartion) in seconds; time<=0 corresponds to manual hangup mode. */
    public int hangup_time=-1;
    /** Automatic call transfer time in seconds; time<0 corresponds to no auto transfer mode. */
    public int transfer_time=-1;
    /** Automatic re-inviting time in seconds; time<0 corresponds to no auto re-invite mode.  */
    public int re_invite_time=-1;
    
    /** Redirect incoming call to the secified url.
     * Use value 'NONE' for not redirecting incoming calls (or let it undefined). */
    public NameAddress redirect_to=null;
    
    /** Transfer calls to the secified url.
     * Use value 'NONE' for not transferring calls (or let it undefined). */
    public NameAddress transfer_to=null;
    
    /** No offer in the invite */
    public boolean no_offer=false;
    /** Do not use prompt */
    public boolean no_prompt=false;
    
    /** Whether using audio */
    public boolean audio=false;
    
    /** Whether using video */
    public boolean video=false;
    
    /** Whether loopbacking toward the sender the incoming media streams. */
    public boolean loopback=false;
    /** Whether playing in receive only mode */
    public boolean recv_only=false;
    /** Whether playing in send only mode */
    public boolean send_only=false;
    /** Whether playing a test tone in send only mode */
    public boolean send_tone=false;
    /** Audio file to be played */
    public String send_file=null;
    /** Audio file to be recorded */
    public String recv_file=null;
    
    /** Media address (use it if you want to specify a media address different from the via address) */
    public String media_addr=null;
    
    /** Audio port */
    public int audio_port=21000;
    
    /** Audio types (as Vector of Strings) */
    private Vector audio_type_list=new Vector();
    /** Audio types (all in one String) */
    private String audio_types=null;
    
    /** Audio avp */
    public int[] audio_avp={ 0 };
    /** Audio codec */
    public String[] audio_codec={ "PCMU" };
    /** Audio sample rate [samples/sec] */
    public int[] audio_sample_rate={ 8000 };
    /** Audio packet size [bytes] */
    public int[] audio_packet_size={ 160 };
    
    /** Video port */
    public int video_port=21070;
    
    /** Video types (as Vector of Strings) */
    private Vector video_type_list=new Vector();
    /** Video types (all in one String) */
    private String video_types=null;
    /** Video avp */
    public int[] video_avp={ 17 };
    
    /** Whether using JMF for audio/video streaming */
    public boolean use_jmf=false;
    /** Whether using RAT (Robust Audio Tool) as audio sender/receiver */
    public boolean use_rat=false;
    /** Whether using VIC (Video Conferencing Tool) as video sender/receiver */
    public boolean use_vic=false;
    /** RAT command-line executable */
    public String bin_rat="rat";
    /** VIC command-line executable */
    public String bin_vic="vic";
    
    // ******************** undocumented parametes ********************
    
    /** Whether running the UAS (User Agent Server), or acting just as UAC (User Agent Client). In the latter case only outgoing calls are supported. */
    public boolean ua_server=true;
    /** Whether running an Options Server, that automatically responds to OPTIONS requests. */
    public boolean options_server=true;
    /** Whether running an Null Server, that automatically responds to not-implemented requests. */
    public boolean null_server=true;
    
    /** Alternative javax-sound-based audio launcher (currently just for tests) */
    public String javax_sound_launcher=null;
    
    /** Whether using explicit external converter (i.e. direct access to an external conversion provider) instead of that provided by javax.sound.sampled.spi. It applies only when javax sound is used, that is when no other audio apps (such as jmf or rat) are used.  */
    public boolean javax_sound_direct_convertion=false;
    
    /** Sender synchronization adjustment, that is the time (in milliseconds) that a frame
     * should be sent in advance by the RTP sender, before the nominal time.
     * A value less that 0 means no re-synchronization explicitely performed by the RTP sender.
     * <p/>
     * Note that when using audio capturing, synchronization with the sample rate
     * is implicitely performed by the audio capture device and frames are read at constat bit rate.
     * However, a value of this parameter >=0 (explicit re-synchronization) is suggested
     * in order to let the read() method non-blocking (in the other case
     * the UA audio performances seem to decrease). */
    public int javax_sound_sync_adj=2;
    
    /** Fixed audio multicast socket address; if defined, it forces the use of this maddr+port for audio session */
    public SocketAddress audio_mcast_soaddr=null;
    /** Fixed video multicast socket address; if defined, it forces the use of this maddr+port for video session */
    public SocketAddress video_mcast_soaddr=null;
    
    public SocketAddress outbound_proxy=null;
    
    
    // ************************** costructors *************************
    
    /** Costructs a void UserAgentProfile */
    public UserAgentProfile() {
        init();
    }
    
    /** Costructs a new UserAgentProfile */
    public UserAgentProfile(String file) {  // load configuration
        loadFile(file);
        // post-load manipulation
        init();
    }
    
    /** Inits the UserAgentProfile. */
    private void init() {
        // initialization and post-load manipulation
        
    	if(outbound_proxy!=null)
    		expires=120;
    	
        if (proxy!=null && proxy.equalsIgnoreCase(Configure.NONE)) proxy=null;
        if (registrar!=null && registrar.equalsIgnoreCase(Configure.NONE)) registrar=null;
        if (display_name!=null && display_name.equalsIgnoreCase(Configure.NONE)) display_name=null;
        if (user!=null && user.equalsIgnoreCase(Configure.NONE)) user=null;
        if (auth_realm!=null && auth_realm.equalsIgnoreCase(Configure.NONE)) auth_realm=null;
        if (send_file!=null && send_file.equalsIgnoreCase(Configure.NONE)) send_file=null;
        if (recv_file!=null && recv_file.equalsIgnoreCase(Configure.NONE)) recv_file=null;
        
        if (audio_types!=null) {
            Parser par=new Parser(audio_types);
            if (par.indexOf('{')>=0) par.goTo('{').skipChar();
            while (par.hasMore()) {
                char[] delim={ ',' , ';' , '}' };
                String audio_type=par.getWord(delim).trim();
                if (audio_type.length()>0) audio_type_list.addElement(audio_type);
                par.skipChar();
            }
        }
        if (audio_type_list.size()>0) {
            int n=audio_type_list.size();
            audio_avp=new int[n];
            audio_codec=new String[n];
            audio_sample_rate=new int[n];
            audio_packet_size=new int[n];
            for (int i=0; i<n; i++) {
                Parser par=new Parser((String)audio_type_list.elementAt(i));
                audio_avp[i]=par.getInt();
                audio_codec[i]=par.getString();
                audio_sample_rate[i]=par.getInt();
                audio_packet_size[i]=par.getInt();
            }
        }
        if (video_types!=null) {
            Parser par=new Parser(video_types);
            if (par.indexOf('{')>=0) par.goTo('{').skipChar();
            while (par.hasMore()) {
                char[] delim={ ',' , ';' , '}' };
                String video_type=par.getWord(delim).trim();
                if (video_type.length()>0) video_type_list.addElement(video_type);
            }
        }
        if (video_type_list.size()>0) {
            int n=video_type_list.size();
            video_avp=new int[n];
            for (int i=0; i<n; i++) {
                Parser par=new Parser((String)video_type_list.elementAt(i));
                video_avp[i]=par.getInt();
            }
        }
        if (ua_jar!=null && ua_jar.equalsIgnoreCase(Configure.NONE)) ua_jar=null;
        
        if (registrar==null && proxy!=null) registrar=proxy;
        if (proxy==null && registrar!=null) proxy=registrar;
        
        if (auth_realm==null && proxy!=null) auth_realm=proxy;
        if (auth_realm==null && registrar!=null) auth_realm=registrar;
        if (auth_user==null && user!=null) auth_user=user;
        
       
    }
    
    
    // ************************ public methods ************************
    
    public void setParams(String params){
        this.params=params;
    }
    
    public String getParams(){
        return this.params;
    }
    
    /** Gets the user's AOR (Address Of Record) registered with the registrar server
     * and used as From URL.
     * <p/>
     * In case of <i>proxy</i> and <i>user</i> parameters have been defined
     * it is formed as "<i>display_name</i>"&lt;sip:<i>user</i>@<i>proxy</i>&rt,
     * otherwhise the <i>contact_url</i> is returned. */
    public NameAddress getUserURI() {
        if (proxy!=null && user!=null) return new NameAddress(display_name,new SipURL(user,proxy,-1));
        else return new NameAddress(contact_url);
    }
    
    /** Sets the user's AOR (Address Of Record) registered with the registrar server
     * and used as From URL, regardless if the uap has null parameters !!!
     * <p/>
     * It actually sets the <i>display_name</i>, <i>user</i>, and <i>proxy</i> parameters.
     * <p/>
     * <i>registrar</i> is set to the <i>proxy</i> value */
    public void setUserURI(NameAddress naddr) {
        SipURL url=naddr.getAddress();
        display_name=naddr.getDisplayName();
        user=url.getUserName();
        proxy=(url.hasPort())? url.getHost()+":"+url.getPort() : url.getHost();
        registrar=proxy;
    }

    /** Sets the user's AOR (Address Of Record) registered with the registrar server
     * and used as From URL, ONLY if the uap has null parameters !!!
     * <p/>
     * It actually sets the <i>display_name</i>, <i>user</i>, and <i>proxy</i> parameters.
     * <p/>
     * If <i>registrar</i> is not defined, the <i>proxy</i> value is used in its place. */
    public void setIfNullUserURI(NameAddress naddr) {
        SipURL url=naddr.getAddress();
        if (display_name==null) display_name=naddr.getDisplayName();
        if (user==null) user=url.getUserName();
        if (proxy==null) proxy=(url.hasPort())? url.getHost()+":"+url.getPort() : url.getHost();
        if (registrar==null) registrar=proxy;
    }

    
    /** Sets contact_url with transport information, taking this information from the SIP provider
     * <p/>
     * This method actually sets contact_url only if it hasn't still been initialized
     * when creating the userAgent from the configuration file
     */
    public void initContactAddress(SipProvider sip_provider) {  // contact_url
        if (contact_url==null) {
            SipURL url=(sip_provider.getPort()!=SipStack.default_port)? new SipURL(user,sip_provider.getViaAddress(),sip_provider.getPort()) : new SipURL(user,sip_provider.getViaAddress());
            if (!sip_provider.getDefaultTransport().equals(SipProvider.PROTO_UDP)) url.addTransport(sip_provider.getDefaultTransport());
            contact_url=new NameAddress(url);
        }
    }
    

//    /** Sets contact_url from a SipPID
//     * <p/>
//     * This method actually sets contact_url regardless if it has already been initialized
//     */
//    public void setContactAddressFromSipPID(SipProcessID sipPID) {  // contact_url
//        	contact_url = sipPID.getNameAddress();
//    }


    /** Sets contact_url from a NameAddress
     * <p/>
     * This method actually sets contact_url regardless if it has already been initialized
     */
    public void setContactAddress(NameAddress nameAddr) {  // contact_url
        	contact_url = nameAddr;
    }

    
    // *********************** protected methods **********************
    
    /** Parses a single line (loaded from the config file) */
    protected void parseLine(String line) {
        String attribute;
        Parser par;
        int index=line.indexOf("=");
        if (index>0) {  attribute=line.substring(0,index).trim(); par=new Parser(line,index+1);  } else {  attribute=line; par=new Parser("");  }
        
        if (attribute.equals("display_name"))   {  display_name=par.getRemainingString().trim();  return;  }
        if (attribute.equals("user"))           {  user=par.getString();  return;  }
        if (attribute.equals("proxy"))          {  proxy=par.getString();  return;  }
        if (attribute.equals("registrar"))      {  registrar=par.getString();  return;  }
        
        if (attribute.equals("contact_url")) {
            String naddr=par.getRemainingString().trim();
            if (naddr==null || naddr.length()==0 || naddr.equalsIgnoreCase(Configure.NONE)) contact_url=null;
            else contact_url=new NameAddress(naddr);
            return;
        }
        if (attribute.equals("auth_user"))      {  auth_user=par.getString();  return;  }
        if (attribute.equals("auth_realm"))     {  auth_realm=par.getRemainingString().trim();  return;  }
        if (attribute.equals("auth_passwd"))    {  auth_passwd=par.getRemainingString().trim();  return;  }
        
        if (attribute.equals("ua_jar"))         {  ua_jar=par.getStringUnquoted();  return;  }
        if (attribute.equals("media_path"))     {  media_path=par.getStringUnquoted();  return;  }
        if (attribute.equals("contacts_file"))  {  contacts_file=par.getStringUnquoted();  return;  }
        
        if (attribute.equals("do_register"))    {  do_register=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("do_unregister"))  {  do_unregister=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("do_unregister_all")) {  do_unregister_all=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("expires"))        {  expires=par.getInt();  return;  }
        if (attribute.equals("keepalive_time")) {  keepalive_time=par.getInt();  return;  }
        
        if (attribute.equals("call_to")) {
            String naddr=par.getRemainingString().trim();
            if (naddr==null || naddr.length()==0 || naddr.equalsIgnoreCase(Configure.NONE)) call_to=null;
            else call_to=new NameAddress(naddr);
            return;
        }
        if (attribute.equals("redirect_to")) {
            String naddr=par.getRemainingString().trim();
            if (naddr==null || naddr.length()==0 || naddr.equalsIgnoreCase(Configure.NONE)) redirect_to=null;
            else redirect_to=new NameAddress(naddr);
            return;
        }
        if (attribute.equals("transfer_to")) {
            String naddr=par.getRemainingString().trim();
            if (naddr==null || naddr.length()==0 || naddr.equalsIgnoreCase(Configure.NONE)) transfer_to=null;
            else transfer_to=new NameAddress(naddr);
            return;
        }
        if (attribute.equals("refuse_time"))    {  refuse_time=par.getInt();  return;  }
        if (attribute.equals("accept_time"))    {  accept_time=par.getInt();  return;  }
        if (attribute.equals("hangup_time"))    {  hangup_time=par.getInt();  return;  }
        if (attribute.equals("transfer_time"))  {  transfer_time=par.getInt();  return;  }
        if (attribute.equals("re_invite_time")) {  re_invite_time=par.getInt();  return;  }
        if (attribute.equals("no_offer"))       {  no_offer=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("no_prompt"))      {  no_prompt=(par.getString().toLowerCase().startsWith("y"));  return;  }
        
        if (attribute.equals("audio"))          {  audio=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("video"))          {  video=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("loopback"))       {  loopback=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("recv_only"))      {  recv_only=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("send_only"))      {  send_only=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("send_tone"))      {  send_tone=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("send_file"))      {  send_file=par.getRemainingString().trim();  return;  }
        if (attribute.equals("recv_file"))      {  recv_file=par.getRemainingString().trim();  return;  }
        
        if (attribute.equals("media_addr"))     {  media_addr=par.getString();  return;  }
        if (attribute.equals("audio_port"))     {  audio_port=par.getInt();  return;  }
        if (attribute.equals("video_port"))     {  video_port=par.getInt();  return;  }
        
        if (attribute.equals("audio_type"))     {  audio_type_list.addElement(par.getRemainingString().trim());  return;  }
        if (attribute.equals("video_type"))     {  video_type_list.addElement(par.getRemainingString().trim());  return;  }
        if (attribute.equals("audio_types"))    {  audio_types=par.getRemainingString().trim();  return;  }
        if (attribute.equals("video_types"))    {  video_types=par.getRemainingString().trim();  return;  }
        
        if (attribute.equals("use_jmf"))        {  use_jmf=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("use_rat"))        {  use_rat=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("bin_rat"))        {  bin_rat=par.getStringUnquoted();  return;  }
        if (attribute.equals("use_vic"))        {  use_vic=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("bin_vic"))        {  bin_vic=par.getStringUnquoted();  return;  }
        
        if (attribute.equals("ua_server")) {  ua_server=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("options_server")) {  options_server=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("null_server")) {  null_server=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("javax_sound_launcher")) {  javax_sound_launcher=par.getString();  return;  }
        if (attribute.equals("javax_sound_direct_convertion")) {  javax_sound_direct_convertion=(par.getString().toLowerCase().startsWith("y"));  return;  }
        if (attribute.equals("javax_sound_sync_adj")) {  javax_sound_sync_adj=par.getInt();  return;  }
        if (attribute.equals("audio_mcast_soaddr")) {  audio_mcast_soaddr=new SocketAddress(par.getString());  return;  }
        if (attribute.equals("video_mcast_soaddr")) {  video_mcast_soaddr=new SocketAddress(par.getString());  return;  }
        
        // for backward compatibily
        
        if (attribute.equals("from_url"))         {  setIfNullUserURI(new NameAddress(par.getRemainingString().trim()));  return;  }
        if (attribute.equals("contact_user"))     {  user=par.getString();  return;  }
        if (attribute.equals("auto_accept"))      {  accept_time=((par.getString().toLowerCase().startsWith("y")))? 0 : -1;  return;  }
        
        if (attribute.equals("audio_avp"))        {  audio_avp[0]=par.getInt();  return;  }
        if (attribute.equals("audio_codec"))      {  audio_codec[0]=par.getString();  return;  }
        if (attribute.equals("audio_sample_rate")){  audio_sample_rate[0]=par.getInt();  return;  }
        if (attribute.equals("audio_packet_size")){  audio_packet_size[0]=par.getInt();  return;  }
        if (attribute.equals("video_avp"))        {  video_avp[0]=par.getInt();  return;  }
        
        if (attribute.equals("audio_frame_size")) {  audio_packet_size[0]=par.getInt();  return;  }
        
        
        if (attribute.equals("outbound_proxy"))
        {  String soaddr=par.getString();
           if (soaddr==null || soaddr.length()==0 || soaddr.equalsIgnoreCase(Configure.NONE) || soaddr.equalsIgnoreCase("NO-OUTBOUND")) outbound_proxy=null;
           else outbound_proxy=new SocketAddress(soaddr);
           return;
        }
    }
    
    
    /** Converts the entire object into lines (to be saved into the config file) */
    protected String toLines() {  // currently not implemented..
        return contact_url.toString();
    }
}
