package eu.convergence.middleware.matchTE.repository;

import java.io.File;
import java.io.IOException;
import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import org.apache.lucene.analysis.Analyzer;
import org.apache.lucene.analysis.KeywordAnalyzer;
import org.apache.lucene.document.Document;
import org.apache.lucene.document.Field;
import org.apache.lucene.document.Fieldable;
import org.apache.lucene.document.NumericField;
import org.apache.lucene.index.CorruptIndexException;
import org.apache.lucene.index.IndexNotFoundException;
import org.apache.lucene.index.IndexWriter;
import org.apache.lucene.index.IndexWriterConfig;
import org.apache.lucene.index.Term;
import org.apache.lucene.search.BooleanClause;
import org.apache.lucene.search.BooleanQuery;
import org.apache.lucene.search.IndexSearcher;
import org.apache.lucene.search.NumericRangeQuery;
import org.apache.lucene.search.ScoreDoc;
import org.apache.lucene.search.TermQuery;
import org.apache.lucene.search.TermRangeQuery;
import org.apache.lucene.store.Directory;
import org.apache.lucene.store.FSDirectory;
import org.apache.lucene.store.LockObtainFailedException;
import org.apache.lucene.util.Version;

import eu.convergence.middleware.matchTE.RepositoryManager;
import eu.convergence.middleware.matchTE.metadata.FieldValueMetadata;
import eu.convergence.middleware.matchTE.query.FieldValueQuery;

public class FieldValueRepository
{
	private static final String SUBS_DIR = "SUBS";
	private static final String PUBS_DIR = "PUBS";
	private static final String VDI_ID_FIELD = "VDI_ID";
	
	private static final int NUMBER_DIGITS = 10;
	
	private Analyzer keywordAnalyzer;
	
	private HashMap<String,Directory> coRestPubDir;
	private HashMap<String,Directory> coRestSubDir;
	
	public FieldValueRepository()
	{
		coRestPubDir = new HashMap<String,Directory>();
		coRestSubDir = new HashMap<String,Directory>();
		
		keywordAnalyzer = new KeywordAnalyzer();
	}
	
	public void addServingCoReST(String coReST) throws IOException
	{
		File subsDir = new File(RepositoryManager.MATCH_TE_DIR + coReST + File.separator + SUBS_DIR);
		subsDir.mkdirs();
		File pubsDir = new File(RepositoryManager.MATCH_TE_DIR + coReST + File.separator + PUBS_DIR);
		pubsDir.mkdirs();
		
		Directory pubDirectory = FSDirectory.open(pubsDir);
		Directory subDirectory = FSDirectory.open(subsDir);
		
		coRestPubDir.put(coReST, pubDirectory);
		coRestSubDir.put(coReST, subDirectory);
	}
	
	private IndexWriter getIndexWriter(String coReST, boolean isPub) throws CorruptIndexException, LockObtainFailedException, IOException
	{
		if (isPub)
		{
			return new IndexWriter(coRestPubDir.get(coReST), new IndexWriterConfig(Version.LUCENE_33, keywordAnalyzer));
		}
		else
		{
			return new IndexWriter(coRestSubDir.get(coReST), new IndexWriterConfig(Version.LUCENE_33, keywordAnalyzer));
		}
	}
	
	public synchronized void storePublication(String CoRest, String PVDI_ID, FieldValueMetadata metadata) throws CorruptIndexException, LockObtainFailedException, IOException
	{
		if (coRestPubDir.get(CoRest)==null)
		{
			addServingCoReST(CoRest);
		}
		
		IndexWriter coRestWriter = getIndexWriter(CoRest,true);
		
		Document doc = new Document();
		
		doc.add(new Field(VDI_ID_FIELD, PVDI_ID, Field.Store.YES, Field.Index.NO));
		
		Map<String, List<Object>> fieldValue = metadata.getEqualMap();

		for (String field : fieldValue.keySet()) // Store publications with numeric field.
		{
			for (Object value : fieldValue.get(field)) // TODO no metadata ...
			{
				if (value instanceof Long)
				{
					NumericField numericfield = new NumericField(field);
					numericfield.setLongValue((Long)value);

					doc.add(numericfield);
				}
				else if (value instanceof Integer)
				{
					NumericField numericfield = new NumericField(field);
					numericfield.setIntValue((Integer)value);

					doc.add(numericfield);
				}
				else if (value instanceof String)
				{
					doc.add(new Field(field, value.toString(), Field.Store.YES, Field.Index.NOT_ANALYZED));
				}

				// TODO date
			}
		}
		
		coRestWriter.addDocument(doc); // TODO add optimize
		coRestWriter.close();
		
		System.out.println("Stored Pub " + PVDI_ID + " in CoReST " + CoRest + " !");
	}
	
	public synchronized void storeSubscription(String CoRest, String SVDI_ID, FieldValueQuery query) throws CorruptIndexException, LockObtainFailedException, IOException
	{
		if (coRestSubDir.get(CoRest)==null)
		{
			addServingCoReST(CoRest);
		}
		
		IndexWriter coRestWriter = getIndexWriter(CoRest,false);
		
		Document doc = new Document();
		
		doc.add(new Field(VDI_ID_FIELD, SVDI_ID, Field.Store.YES, Field.Index.NO));
		
		// Store subscriptions with operator
		Map<String, List<Object>> fieldValue = query.getEqualMap();

		for (String field : fieldValue.keySet())  
		{
			for (Object value : fieldValue.get(field))
			{
				if (value instanceof String)
				{
					doc.add(new Field(field, createRepositoryValue(value), Field.Store.YES, Field.Index.NOT_ANALYZED));
				}
				else
				{
					doc.add(new Field(field, "=" + createRepositoryValue(value), Field.Store.YES, Field.Index.NOT_ANALYZED));
				}
			}
		}

		Map<String, Object> fieldValueInequal = query.getLessThanMap();

		for (String field : fieldValueInequal.keySet())
		{
			doc.add(new Field(field, "<" + createRepositoryValue(fieldValueInequal.get(field)), Field.Store.YES, Field.Index.NOT_ANALYZED));
		}

		fieldValueInequal = query.getGreaterThanMap();

		for (String field : fieldValueInequal.keySet())
		{
			doc.add(new Field(field, ">" + createRepositoryValue(fieldValueInequal.get(field)), Field.Store.YES, Field.Index.NOT_ANALYZED));
		}
		
		coRestWriter.addDocument(doc);
		coRestWriter.close();
		
		System.out.println("Stored Sub " + SVDI_ID + " in CoReST " + CoRest + " !");
	}
	
	public List<String> matchSubToPubsWith(String CoRest, FieldValueQuery query) throws CorruptIndexException, IOException
	{
		List<String> results = new ArrayList<String>();
		
		try
		{
			Directory corestDir = coRestPubDir.get(CoRest);	
			IndexSearcher coRestSearcher = new IndexSearcher(corestDir,true);
			
			BooleanQuery searchQuery = new BooleanQuery();
			
			Map<String, List<Object>> equalMap = query.getEqualMap();

			for (String field : equalMap.keySet())
			{
				for (Object keyword : equalMap.get(field))
				{
					Term term = new Term(field, keyword.toString());
					System.out.println("************SUBSCRIPTION************************ " + field + " = " + keyword.toString());
					TermQuery termQuery = new TermQuery(term);

					searchQuery.add(termQuery, BooleanClause.Occur.MUST);
				}
			}

			Map<String, Object> inequalityMap = query.getLessThanMap();

			for (String field : inequalityMap.keySet())
			{
				Object value =  inequalityMap.get(field);

				if (value instanceof Long)
				{
					NumericRangeQuery rangeQuery = NumericRangeQuery.newLongRange(field, null, (Long)inequalityMap.get(field), true, true);

					searchQuery.add(rangeQuery, BooleanClause.Occur.MUST);
				}
				else if (value instanceof Integer)
				{
					NumericRangeQuery rangeQuery = NumericRangeQuery.newIntRange(field, null, (Integer)inequalityMap.get(field), true, true);

					searchQuery.add(rangeQuery, BooleanClause.Occur.MUST);
				}

				// TODO date
			}

			inequalityMap = query.getGreaterThanMap();

			for (String field : inequalityMap.keySet())
			{
				Object value =  inequalityMap.get(field);

				if (value instanceof Long)
				{
					NumericRangeQuery rangeQuery = NumericRangeQuery.newLongRange(field, (Long)inequalityMap.get(field), null, true, true);

					searchQuery.add(rangeQuery, BooleanClause.Occur.MUST);
				}
				else if (value instanceof Integer)
				{
					NumericRangeQuery rangeQuery = NumericRangeQuery.newIntRange(field, (Integer)inequalityMap.get(field), null, true, true);

					searchQuery.add(rangeQuery, BooleanClause.Occur.MUST);
				}

				// TODO date
			}
			
			ScoreDoc[] docIdHits = coRestSearcher.search(searchQuery,1000).scoreDocs; // TODO parameter of search 1000
			
			for (ScoreDoc docId : docIdHits)
			{
				Document doc = coRestSearcher.doc(docId.doc);
				
				results.add(doc.getValues(VDI_ID_FIELD)[0]);
			}
			
			coRestSearcher.close();
		}
		catch(IndexNotFoundException ex) // TODO make it a MatchTE exception
		{
			System.out.println("No publications.");
		}
		
		return results;
	}
	
	public List<String> matchPubToSubsWith(String CoRest, FieldValueMetadata metadata) throws CorruptIndexException, IOException
	{
		List<String> results = new ArrayList<String>();
		
		try
		{
			Directory corestDir = coRestSubDir.get(CoRest);	
			IndexSearcher coRestSearcher = new IndexSearcher(corestDir,true);
			
			BooleanQuery query = new BooleanQuery();

			Map<String, List<Object>> fieldValue = metadata.getEqualMap();

			for (String field : fieldValue.keySet())
			{
				for (Object value : fieldValue.get(field))
				{
					if (value instanceof String)
					{
						Term term = new Term(field, (String) value);
						TermQuery termQuery = new TermQuery(term);

						query.add(termQuery, BooleanClause.Occur.SHOULD);
					}
					else
					{
						Term term = new Term(field, "=" + createRepositoryValue(value));
						TermQuery termQuery = new TermQuery(term);

						query.add(termQuery, BooleanClause.Occur.SHOULD);

						TermRangeQuery maxRangeQuery = new TermRangeQuery(field, ">" + createRepositoryValue(0L), ">" +createRepositoryValue(value), true, true);

						query.add(maxRangeQuery, BooleanClause.Occur.SHOULD);

						TermRangeQuery minRangeQuery = new TermRangeQuery(field, "<" +createRepositoryValue(value), "<" + (Math.pow(10, NUMBER_DIGITS)-1), true, true);
						query.add(minRangeQuery, BooleanClause.Occur.SHOULD);
					}
				}
			}

			ScoreDoc[] docIdHits = coRestSearcher.search(query,1000).scoreDocs; // TODO parameter of search 1000

			for (ScoreDoc docId : docIdHits)
			{
				Document doc = coRestSearcher.doc(docId.doc);

				// to pub prepei na exei ola ta fields kai se kathe field prepei na exei oles tis times
				// se periptosi string paizo opos epeza, se peiptosi > na metatrepseis to string se Long kai na sigrineis ?

				boolean isOk = true;

				for (Fieldable fieldable : doc.getFields())
				{
					String field = fieldable.name();

					if(!field.equals(VDI_ID_FIELD))
					{
						for (String subValue : doc.getValues(field))
						{
							if (fieldValue.get(field)!=null)
							{
								List<Object> pubValues = fieldValue.get(field);

								if (subValue.startsWith("=") || subValue.startsWith(">") || subValue.startsWith("<"))
								{
									Long subLong = Long.parseLong(subValue.substring(1));

									Object pubValue = pubValues.get(0);

									if (pubValue instanceof Long)
									{
										Long pubLong = (Long) pubValue;

										if (subValue.startsWith("=") && !pubLong.equals(subLong))
										{
											isOk = false;
										}

										if (subValue.startsWith(">") && subLong>pubLong)
										{
											isOk = false;
										}

										if (subValue.startsWith("<") && subLong<pubLong)
										{
											isOk = false;
										}
									}
									else if (pubValue instanceof Integer)
									{
										Integer pubLong = (Integer) pubValue;

										if (subValue.startsWith("=") && !pubLong.equals(subLong))
										{
											isOk = false;
										}

										if (subValue.startsWith(">") && subLong>pubLong)
										{
											isOk = false;
										}

										if (subValue.startsWith("<") && subLong<pubLong)
										{
											isOk = false;
										}
									}
									else
									{
										isOk = false;
									}
									// TODO date
								}
								else
								{
									if (!pubValues.contains(subValue))
									{
										isOk = false;
									}
								}
							}
							else
							{
								isOk = false;
							}

							if (!isOk)
							{
								break;
							}
						}
					}

					if (!isOk)
					{
						break;
					}
				}

				if (isOk)
				{
					results.add(doc.getValues(VDI_ID_FIELD)[0]);
				}
			}
			
			coRestSearcher.close();
		}
		catch(IndexNotFoundException ex)
		{
			System.out.println("No subscriptions.");
		}
		
		return results;
	}
	
	private String createRepositoryValue(Object object)
	{
		if (object instanceof Long)
		{
			String toReturn = ((Long)object).toString();
			
			while (toReturn.length()<NUMBER_DIGITS)
			{
				toReturn = "0" + toReturn;
			}
			
			return toReturn;
		}
		else if (object instanceof Integer)
		{
			String toReturn = ((Integer)object).toString();
			
			while (toReturn.length()<NUMBER_DIGITS)
			{
				toReturn = "0" + toReturn;
			}
			
			return toReturn;
		}
		else if (object instanceof String)
		{
			return (String) object;
		}
		
		// TODO date
		
		return "";
	}
}
