package eu.convergence.middleware.matchTE;

import java.io.File;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;

import junit.framework.Assert;
import junit.framework.TestCase;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;

import eu.convergence.middleware.core.ConvergenceEngineName;
import eu.convergence.middleware.engine.matchTE.exception.MatchEngineException;
import eu.convergence.middleware.matchTE.metadata.FieldValueMetadata;
import eu.convergence.middleware.matchTE.query.FieldValueQuery;

public class MatchEngineResultsValidityTest extends TestCase
{
	private final String MIX_CD_COREST = "TEST MIX CD"; // Need 2 corests because MatchTE is static and the deletion of the corest doesn't
	private final String MIX_TAPE_COREST = "TEST MIX TAPE"; // delete the directories from memory.
	private String[] PUB_FIRST_TEST_RESULT;
	private String[] SUB_FIRST_TEST_RESULT;
	private List<FieldValueMetadata> pubs;
	private List<FieldValueQuery> subs;
	
	protected void setUp()
	{
		RepositoryCleaner repositoryCleaner = new RepositoryCleaner(); 
		repositoryCleaner.cleanFolder(MIX_CD_COREST);
		repositoryCleaner.cleanFolder(MIX_TAPE_COREST);
		
		Object[][] keywordsArray = {{"HOUSE"},
									{"TECHNO"},
									{"HOUSE", "DISCO HOUSE"},
									{"HOUSE", "TECHNO"},
									{"TECHNO", "DEEP HOUSE"},
									{"HOUSE", "DISCO HOUSE", "DEEP HOUSE"},
									{"ELECTRO", "DISCO HOUSE", "DEEP HOUSE"},
									{"ELECTRO", "TECHNO", "DEEP HOUSE"},
									{"ELECTRO", "TECHNO", "DRUM AND BASS"}};
		
		pubs = new ArrayList<FieldValueMetadata>();
		subs = new ArrayList<FieldValueQuery>();
		
		for (int i=0; i<keywordsArray.length; i++)
		{
			FieldValueMetadata pubMeta = new FieldValueMetadata();
			FieldValueQuery subQuery = new FieldValueQuery();
			
			List<Object> keywords = new ArrayList<Object>();
			for (int j=0; j<keywordsArray[i].length; j++)
			{
				keywords.add(keywordsArray[i][j]);
			}
			
			pubMeta.getEqualMap().put("KEYWORD", keywords);
			subQuery.getEqualMap().put("KEYWORD", keywords);
			
			pubs.add(pubMeta);
			subs.add(subQuery);
		}
		
		PUB_FIRST_TEST_RESULT = new String[] {"[P1, P3, P4, P6]", "[P2, P4, P5, P8, P9]", "[P3, P6]",
											  "[P4]", "[P5, P8]", "[P6]", "[P7]", "[P8]", "[P9]"};
		SUB_FIRST_TEST_RESULT = new String[] {"[S1]", "[S2]", "[S1, S3]", "[S1, S2, S4]", "[S2, S5]",
											  "[S1, S3, S6]", "[S7]", "[S2, S5, S8]", "[S2, S9]"};
	}
	
	protected void tearDown()
	{
		RepositoryCleaner repositoryCleaner = new RepositoryCleaner(); 
		repositoryCleaner.cleanFolder(MIX_CD_COREST);
		repositoryCleaner.cleanFolder(MIX_TAPE_COREST);
	}
	
	public void testPublicationsOverSubscription() throws MXMConfigurationException, MXMEngineException
	{
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();
		
		MatchTE matchTE = (MatchTE)mxm.getDefaultEngine(ConvergenceEngineName.MatchTE.value());
		
		List<String> coRest = new ArrayList<String>();
		coRest.add(MIX_CD_COREST);
		
		for (int i=0; i<pubs.size(); i++)
		{
			pubs.get(i).setCorests(coRest);
			
			matchTE.getPublicationManager().matchSubsWith("P" + (i+1), new MXMAdapter(pubs.get(i)), new Date());
		}
		
		String[] pub_first_results = new String[pubs.size()];
		
		for (int i=0; i<subs.size(); i++)
		{
			subs.get(i).setCorests(coRest);
			
			List<String> results = matchTE.getSubscriptionManager().matchPubsWith("S" + (i+1), new MXMAdapter(subs.get(i)), new Date());
			
			if (results.size()>0)
			{
				Collections.sort(results);
				System.out.println("Found matches for S" +(i+1) + " : " + results);
				pub_first_results[i] = results.toString();
			}
		}
		
		for (int i=0; i<pubs.size(); i++)
		{
			Assert.assertEquals(PUB_FIRST_TEST_RESULT[i], pub_first_results[i]);
		}
	}
		
	public void testSubscriptionsOverPublications() throws MXMConfigurationException, MXMEngineException
	{
		File f = new File(getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		MXM mxm = MXM.getInstance(f);
		mxm.listEnginesAndProperties();
		
		MatchTE matchTE = (MatchTE)mxm.getDefaultEngine(ConvergenceEngineName.MatchTE.value());
		
		List<String> coRest = new ArrayList<String>();
		coRest.add(MIX_TAPE_COREST);
		
		for (int i=0; i<subs.size(); i++)
		{
			subs.get(i).setCorests(coRest);
			
			matchTE.getSubscriptionManager().matchPubsWith("S" + (i+1), new MXMAdapter(subs.get(i)), new Date());
		}
		
		String[] sub_first_results = new String[subs.size()];
		
		for (int i=0; i<pubs.size(); i++)
		{
			pubs.get(i).setCorests(coRest);
			
			List<String> results = matchTE.getPublicationManager().matchSubsWith("P" + (i+1), new MXMAdapter(pubs.get(i)), new Date());
			
			if (results.size()>0)
			{
				Collections.sort(results);
				System.out.println("Found matches for P" +(i+1) + " : " + results);
				sub_first_results[i] = results.toString();
			}
		}
		
		for (int i=0; i<subs.size(); i++)
		{
			Assert.assertEquals(SUB_FIRST_TEST_RESULT[i], sub_first_results[i]);
		}
	}
	
}
