package eu.convergence.middleware.orchestratorTE.handle;

import eu.convergence.middleware.conetTe.ConetTE;
import eu.convergence.middleware.engine.ConetTe.exception.ConetIOException;
import eu.convergence.middleware.engine.metadataTE.ConvergenceMetadataEngine;
import eu.convergence.middleware.engine.metadataTE.schemahandler.ConvergenceMetadata;
import eu.convergence.middleware.engine.metadataTE.technologyhandler.MetadataKinds;
import eu.convergence.middleware.engine.metadataTE.technologyhandler.MetadataParser;
import eu.convergence.middleware.engine.vdiTE.VdiEngine;
import eu.convergence.middleware.event.ERArrivalEvent;
import eu.convergence.middleware.event.MatchingPublicationReceivedEvent;
import eu.convergence.middleware.metadataTE.ConvergenceMetadataTE;
import eu.convergence.middleware.metadataTE.technologyhandler.ConvergenceMetadataParser;
import eu.convergence.middleware.orchestratorTE.build.ConetEngineOrchestrator;
import eu.convergence.middleware.orchestratorTE.build.VdiOrchestrator;
import eu.convergence.middleware.vdiTE.VDITE;
import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.core.event.management.MXMEvent;
import org.iso.mpeg.mxm.core.event.management.MXMEventListener;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;

import java.io.*;
import java.util.Date;
import java.util.ArrayList;
import java.util.Iterator;
import java.util.LinkedHashMap;
import java.util.List;
import java.util.Set;

/**
 * Implementation of the ReceiverOrchestrator that must be notified by the ERROrchestrator that a match publication
 * VDI has been found and the PVDI identifier must be given to it,
 * so that the ReceiverOrchestrator can fetch the resource VDI associated with the PVDI.
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro"> Bogdan Ardeleanu</a>
 * Date: 8/31/11
 * Time: 4:11 PM
 * <p/>
 * UTI Systems, TIC
 */
public class ReceiverOrchestrator implements MXMEventListener
{
    private static ReceiverOrchestrator RECEIVER_ORCHESTRATOR = null;
  	private Logger logger = Logger.getLogger(ReceiverOrchestrator.class);
  	private VdiEngine vdiEngine = null;
  	private ConvergenceMetadataEngine convMetaEngine = null;
  	private ConetEngineOrchestrator conetOrchestrator = null;

    private ReceiverOrchestrator(VdiEngine d, ConvergenceMetadataEngine c, ConetEngineOrchestrator conetEngineOrchestrator)
    {
        this.vdiEngine = d;
        this.convMetaEngine = c;
    	this.conetOrchestrator = conetEngineOrchestrator;
    }

    public static ReceiverOrchestrator getInstance(VdiEngine d, ConvergenceMetadataEngine c, ConetEngineOrchestrator m)
    {
        if (null == RECEIVER_ORCHESTRATOR)
        {
            RECEIVER_ORCHESTRATOR = new ReceiverOrchestrator(d, c, m);
        }

        return RECEIVER_ORCHESTRATOR;
    }
    
    public void bootReceiver()
    {
    	 try
         {
             // register for the ER event
             MXM.getInstance().getMXMEventHandler().registerMXMEventListener(ERArrivalEvent.class.getName(), this);
         } 
         catch (MXMConfigurationException e)
         {
             e.printStackTrace();
         }
    }
    
    /**
     * This method/workflow receives a VDI ID of a Publication VDI and does the following:
     * <p/>
     * - fetch the corresponding PVDI from CoNet using the PVDI ID
     * <p/>
     * - look inside the PVDI to get the ID of the associated RVDI
     * <p/>
     * - fetch the RVDI from CoNet using the RVDI ID
     * <p/>
     * - extract the actual resource pointer (the url of the picture) from the RVDI
     * <p/>
     * - pass the pointer of the resource to the Application by invoking a callback on it??
     * @param pvdi_id
     * @return
     * @throws MXMException 
     */
    public boolean receivePVDI(String pvdi_id, String svdi_id) throws MXMException
    {
    	// fetch the PVDI from PVDI ID
    	String pvdiFileName = null;
    	try
    	{
    		pvdiFileName = conetOrchestrator.fetchResourceFromConet(pvdi_id);
    	}
    	catch (ConetIOException e)
    	{
    		logger.error("Cannot fetch the PVDI " + pvdi_id +" from CoNet. Receive workflow failed", e);
    		return false;
    	}

    	//get RVDI ID from inside the PVDI
    	DIDL didl = vdiEngine.getVDIEngineSchemaHandler().getDIDL();
    	didl.parse(new MXMAdapter(new File(pvdiFileName)));
    	
    	StructuredData convMetadata = didl.getMPEGMItem().getMetadata();
    	
    	String rvdiId = null;
    	if (convMetadata!=null)
    	{
	    	MetadataParser metadataParser = convMetaEngine.getConvergenceMetadataTechnologyHandler().getMetadataParser(convMetadata);
	    	
	    	if(metadataParser.getMetadataKind() == MetadataKinds.CONVERGENCE) 
	    	{
	    		rvdiId = ((ConvergenceMetadataParser)metadataParser).getConvergenceMetadata().getPublicationMetadata().getRVDIIdentifier();
				logger.info("PVDI points to this RVDI: " + rvdiId);
	    	}
    	}

//    	// fetch the PVDI from RVDI ID
//    	String rvdi_file_name = null;
//    	try {
//    		rvdi_file_name = didOrchestrator.fetchAndSaveVDI(rvdi_id);
//    	}
//    	catch (ConetIOException e) {
//    		logger.error("Cannot fetch the RVDI " + rvdi_id +" from CoNet. Receive workflow failed", e);
//    		return MXMEngineResponse.FAILURE;
//    	}
//  		
//  		String resource_url = null;
//  		//String mime_type = null;
//  		LinkedHashMap<String, String> resourceUrlAndMimeT = new LinkedHashMap<String, String>();
//  		
//  		ArrayList<String> annotations = new ArrayList<String>();
//  		
//  		//get resource url from inside the RVDI
//  		try {
//  				diParser = didEngine.getDIParser();
//			
//				response = diParser.parseDI(new FileInputStream(rvdi_file_name));
//			} catch (MXMEngineException e) {
//  			logger.error("Cannot parse the RVDI file " + rvdi_file_name +" . Receive workflow failed", e);
//  			return MXMEngineResponse.FAILURE;
//			} catch (FileNotFoundException e){
//				e.printStackTrace();
//			}
//  		if(response.equals(MXMEngineResponse.SUCCESS)) {
//  			//resource_url = diParser.getRef();
//  			//mime_type = diParser.getMimeType();
//  			
//  			List<String> contentElementIDs = diParser.getContentElementIDList();
//			for(String contentElementID : contentElementIDs){
//				resourceUrlAndMimeT.put(diParser.getRef(contentElementID), diParser.getMimeType(contentElementID));
//			}
//			
//  			
//  			ArrayList<String> keywords = null;
//  			
//  			//scan metadata from RVDI iterating the different kinds of metadata blocks
//  			ArrayList<MXMObject> rvdiMetadataContainers = diParser.getContentMetadataList();
//  			if(rvdiMetadataContainers != null) {
//  				for (MXMObject rvdiMetadataContainer : rvdiMetadataContainers) {
//  					if(rvdiMetadataContainer.hasContent()) {
//  						MetadataParserInterface metaParser;
//						try {
//							metaParser = convMetaEngine.getMetadataParser(rvdiMetadataContainer);
//						} catch (MetadataEngineException e) {
//				  			logger.error("Cannot get the MetadataParser to read the resource annotations. Receive workflow failed", e);
//				  			return MXMEngineResponse.FAILURE;
//						}
//  						if(metaParser.getMetadataKind() == MetadataKinds.CONVERGENCE) {
//  							Date expiryDate = ((ConvergenceMetadataParser)metaParser).getConvergenceMetadata().getExpiryDate();
//  							if(expiryDate != null)
//  								logger.info("RVDI has expiry date: " + expiryDate.toLocaleString());
//  							else
//  								logger.info("RVDI has no expiry date.");
//  						}
//  						else if(metaParser.getMetadataKind() == MetadataKinds.MPEG7) {
//  							keywords = ((Mpeg7MetadataParser)metaParser).extractKeywords();
//  							if(keywords != null) {
//  								annotations.addAll(keywords);
//  							}
//  						}
//  					}
//  				}
//  			}
//  			
//  			Set resourceList = resourceUrlAndMimeT.keySet();
//  			Iterator iter = resourceList.iterator();
//  			while( iter.hasNext()){
//  				resource_url = (String) iter.next();
//  				logger.info("RVDI contains resource: " + resource_url);
//  			}
//  		}
  		
  		// TODO notify Application via callback ???
		//MatchingPublicationReceivedEvent matchEvent = new MatchingPublicationReceivedEvent(this,resource_url, annotations,rvdi_id, svdi_id, mime_type);
  		//MatchingPublicationReceivedEvent matchEvent = new MatchingPublicationReceivedEvent(this,resourceUrlAndMimeT, annotations,rvdi_id, svdi_id);
    	MatchingPublicationReceivedEvent matchEvent = new MatchingPublicationReceivedEvent(this,null, null,rvdiId, svdi_id);  // TODO why annotations etc. ???
    	try
  		{
  			MXM.getInstance().getMXMEventHandler().fireMXMEvent(matchEvent);
  		} 
  		catch (MXMConfigurationException e)
  		{
  			logger.info("Exception in getting MXM instance to fire a MatchingPublicationReceivedEvent. Receive workflow failed", e);
  		}
    	
		return false;
    }

    @Override
    public void mxmEventOccurred(MXMEvent event)
    {
        // ReceiverOrchestrator handles the MXMERArrivalEvent
        if (event instanceof ERArrivalEvent)
        {
            ERArrivalEvent evt = (ERArrivalEvent) event;
            String publicationVdiId = evt.getPublicationVdiId();
            String subscriptionVdiId = evt.getSubscriptionVdiId();

            logger.info("MXMERArrivalEvent occured: a match publication VDI has been found and the P/SVDI ids are " 
            																+ publicationVdiId + " , " + subscriptionVdiId);
            try
            {
            	receivePVDI(publicationVdiId, subscriptionVdiId);
            } 
            catch (Exception e)
            {
            	 e.printStackTrace();
            }
        }
    }

}
