/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2007
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s): Edoardo Radica
 *
 * ***** END LICENSE BLOCK ***** */

package eu.convergence.middleware.relTE.authorisation;

import java.util.List;

import javax.xml.bind.JAXBElement;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.relTE.DiReference;
import org.iso.mpeg.mxm.engine.relTE.DigitalResource;
import org.iso.mpeg.mxm.engine.relTE.ForAll;
import org.iso.mpeg.mxm.engine.relTE.Grant;
import org.iso.mpeg.mxm.engine.relTE.GrantGroup;
import org.iso.mpeg.mxm.engine.relTE.Issuer;
import org.iso.mpeg.mxm.engine.relTE.License;
import org.iso.mpeg.mxm.engine.relTE.Principal;
import org.iso.mpeg.mxm.engine.relTE.PropertyPossessor;
import org.iso.mpeg.mxm.engine.relTE.ProtectedResource;
import org.iso.mpeg.mxm.engine.relTE.Right;
import org.iso.mpeg.mxm.engine.relTE.ServiceReference;
import org.iso.mpeg.mxm.engine.relTE.authorisation.AuthorisationResult;
import org.iso.mpeg.mxm.engine.relTE.exception.RELAuthorisationException;
import org.iso.mpeg.mxm.engine.relTE.exception.RELEngineException;
import org.iso.mpeg.mxm.engine.relTE.exception.RELParsingException;

import org.iso.mpeg.mxm.engine.securityTE.SecurityEngine;

import org.iso.mpeg.mxm.engine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.engine.relTE.schema.relr.Condition;
import org.iso.mpeg.mxm.engine.relTE.schema.relr.EncryptedContent;
import org.iso.mpeg.mxm.engine.relTE.schema.relsx.PropertyUri;


public class AuthorisationManager implements org.iso.mpeg.mxm.engine.relTE.authorisation.AuthorisationManager {
	
	private Logger logger = Logger.getLogger(AuthorisationManager.class);
	
	protected SecurityEngine securityEngine = null;
	
	protected AuthorisationResponse authorisationResponse = null;
	
	public AuthorisationManager() {
	}
	
	/**
	 * This method receives a License and a Query (the latter shall contain one Grant only)
	 * and returns an object of type ValidationResult containing the result of the validation
	 * @throws RELEngineException 
	 * @throws SecurityEngineException 
	 */	
	public AuthorisationResult authorise(License license, License query, SecurityEngine securityEngine) 
	throws RELEngineException, SecurityEngineException {
		this.securityEngine = securityEngine;

		authorisationResponse = new AuthorisationResponse();
		authorisationResponse.setLicenseID(license.getLicenseID());
		authorisationResponse.setIssuerToBeVerified(license.getIssuer());
		
		int queryGrantSize = query.getGrants().size();
		if (queryGrantSize != 1)
			throw new RELAuthorisationException("Only queries with 1 Grant are currently supported, " +
					"while the supplied query has " + queryGrantSize + " grants.");
			
		Grant queryGrant = query.getGrants().get(0);
		
		for (Grant licenseGrant : license.getGrants()) {
			EvaluatedGrant evalGrant = this.verifyGrant(licenseGrant, queryGrant);
			authorisationResponse.addEvaluatedGrant(evalGrant);		
		}
		return authorisationResponse.getResult();
	}
						
	
	public AuthorisationResult authorise(License license, MXMObject issuer,
			MXMObject user, Right right, MXMObject resource,
			SecurityEngine securityEngine) throws RELEngineException, SecurityEngineException {

		this.securityEngine = securityEngine;

		authorisationResponse = new AuthorisationResponse();
		authorisationResponse.setLicenseID(license.getLicenseID());
		authorisationResponse.setIssuerToBeVerified(license.getIssuer());

		for (Grant grant : license.getGrants()) {
			if (grant.getEncryptedContent(EncryptedContent.class.getCanonicalName()).getContent() == null) {
				
				EvaluatedGrant evalGrant = this.verifyGrant(grant, user, right, resource);
				authorisationResponse.addEvaluatedGrant(evalGrant);
				
			}
		}
		return authorisationResponse.getResult();

	}
	
	public AuthorisationResult authoriseIssue(License license, MXMObject issuer,
			MXMObject user, Right right, MXMObject resource,MXMObject principalToIssue,
			SecurityEngine securityEngine) throws RELEngineException, SecurityEngineException {

		this.securityEngine = securityEngine;

		authorisationResponse = new AuthorisationResponse();
		authorisationResponse.setLicenseID(license.getLicenseID());
		authorisationResponse.setIssuerToBeVerified(license.getIssuer());

		for (Grant grant : license.getGrants()) {
			if (grant.getEncryptedContent(EncryptedContent.class.getCanonicalName()).getContent() == null) {
				
				EvaluatedGrant evalGrant = this.verifyGrant(grant, user, right, resource, principalToIssue);
				authorisationResponse.addEvaluatedGrant(evalGrant);
				
			}
		}
		return authorisationResponse.getResult();

	}
	
	private EvaluatedGrant verifyGrant(Grant licenseGrant, MXMObject principal,
			Right right, MXMObject resource)
			throws RELEngineException, SecurityEngineException {

		Right queryRight = right;
		String reasonOfFailure = null;

		// prepare the result object
		EvaluatedGrant evalGrant = new EvaluatedGrant(licenseGrant);

		// VERIFY THE RESOURCE
		if (!this.verifyResource(licenseGrant, resource)) {
			reasonOfFailure = new String("The Grant in the supplied License does not grant Rights over the Resource specified in the Query.");
			evalGrant.setDisplayString(reasonOfFailure);
			evalGrant.setAuthorisationResult(AuthorisationResult.RESOURCE_IS_NOT_MET);
			return evalGrant;
		}

		// VERIFY THE PRINCIPAL
		if (!this.verifyPrincipal(licenseGrant.getPrincipal(), principal)) {
			reasonOfFailure = new String("The Grant in the supplied License does not grant Rights to the Principal specified in the Query.");
			evalGrant.setDisplayString(reasonOfFailure);
			evalGrant.setAuthorisationResult(AuthorisationResult.PRINCIPAL_IS_NOT_MET);
			return evalGrant;
		}

		// VERIFY THE RIGHT
		if (!this.verifyRight(licenseGrant, right)) {
			reasonOfFailure = new String("The Grant in the supplied License does not grant the Right: "
							+ queryRight.toString()
							+ ", which would be required to perform the requested action.");
			evalGrant.setDisplayString(reasonOfFailure);
			evalGrant.setAuthorisationResult(AuthorisationResult.RIGHT_IS_NOT_MET);
			return evalGrant;
		}

		// VERIFY THE CONDITIONS
		ConditionManager conditionInterpreter = new ConditionManager();
		List<MXMObject> conditions = licenseGrant.getConditions(JAXBElement.class.getCanonicalName());

//FIXME-security		
//		SecureRepositoryManager secureRepositoryManager = securityEngine.getSecureRepositoryManager();
//		String secureRepositoryPath = securityEngine.getPropertyValue(SecurityEngineKeys.SECURE_INFO_STORAGE_PATH);
//		QName currentCountry = secureRepositoryManager.getCountry(new MXMAdapter(secureRepositoryPath));
//		QName currentRegion = secureRepositoryManager.getRegion(new MXMAdapter(secureRepositoryPath));
//		boolean isCommercial = secureRepositoryManager.isCommercial(new MXMAdapter(secureRepositoryPath));
//
//		ConditionResult conditionResult = conditionInterpreter.verifyConditions(conditions, currentCountry, currentRegion,isCommercial);
//		evalGrant.setConditionEvaluationResult(conditionResult);
//		if (conditionResult.getResult() == ConditionResultType.SUCCESSFULLY_MET) {
//			evalGrant.setDisplayString("The Principal is authorised to exercise the right over the Resource.");
//			evalGrant.setAuthorisationResult(AuthorisationResult.AUTHORISATION_SUCCEEDED);
//			return evalGrant;
//		} else if (conditionResult.getResult() == ConditionResultType.COPYRIGHT_NOTICE) {
//			evalGrant.setDisplayString("The Principal may exercise the right over the Resource subject to one or more ORC Conditions.");
//			evalGrant.setAuthorisationResult(AuthorisationResult.ORC_CONDITIONS_FOUND);
//			return evalGrant;
//		} else if (conditionResult.getResult() == ConditionResultType.PAYMENT) {
//			evalGrant.setDisplayString("The Principal may exercise the right over the Resource upon making a transaction.");
//			evalGrant.setAuthorisationResult(AuthorisationResult.PAYMENT_CONDITIONS_FOUND);
//			return evalGrant;
//		} else {
//			evalGrant.setDisplayString("The Principal may not exercise the right over the Resource because a number of Conditions were not met.");
//			evalGrant.setAuthorisationResult(AuthorisationResult.CONDITIONS_ARE_NOT_MET);
//			return evalGrant;
//		}
//DUMMY: next line
		return evalGrant;
	}
	
	private EvaluatedGrant verifyGrant(Grant licenseGrant, MXMObject principal,Right right,
											MXMObject resource, MXMObject principalToIssue) 
	throws RELEngineException, SecurityEngineException {

		Right queryRight = right;
		String reasonOfFailure = null;
		
		//prepare the result object
		EvaluatedGrant evalGrant = new EvaluatedGrant(licenseGrant);
		
		// VERIFY THE RESOURCE
		if(!licenseGrant.getRight().equals(Right.issue)){
			if (! this.verifyResource(licenseGrant, resource)) {
				reasonOfFailure = new String("The Grant in the supplied License does not grant Rights over the Resource specified in the Query.");
				evalGrant.setDisplayString(reasonOfFailure);
				evalGrant.setAuthorisationResult(AuthorisationResult.RESOURCE_IS_NOT_MET);
				return evalGrant;
			}
		}
		// VERIFY THE PRINCIPAL TO ISSUE
		else {
			if(licenseGrant.isGrantResource()){
				Grant grant = licenseGrant.getGrantResource();
				if(!this.verifyPrincipal(grant.getPrincipal(), principalToIssue)){
					evalGrant.setAuthorisationResult(AuthorisationResult.PRINCIPAL_DOMAIN_IS_NOT_MET);
					return evalGrant;
				}
			} 
			else if(licenseGrant.isGrantGroupResource()){
				GrantGroup gg = licenseGrant.getGrantGroupResource();
				for (MXMObject grantOrGG : gg.getGrantOrGrantGroupResource()) {
					if(grantOrGG.getContent() instanceof Grant){
						Grant grant = (Grant) grantOrGG.getContent();
						if (grant.getEncryptedContent(EncryptedContent.class.getCanonicalName()).getContent() == null) {
							if(!this.verifyPrincipal(grant.getPrincipal(), principalToIssue)){
								evalGrant.setAuthorisationResult(AuthorisationResult.PRINCIPAL_DOMAIN_IS_NOT_MET);
								return evalGrant;
							}
						}
					}
				}
			} 
		}
		
		// VERIFY THE PRINCIPAL
		if (! this.verifyPrincipal(licenseGrant.getPrincipal(), principal) ){
			reasonOfFailure = new String("The Grant in the supplied License does not grant Rights to the Principal specified in the Query.");
			evalGrant.setDisplayString(reasonOfFailure);
			evalGrant.setAuthorisationResult(AuthorisationResult.PRINCIPAL_IS_NOT_MET);
			return evalGrant;
		}

		// VERIFY THE RIGHT
		if(! this.verifyRight(licenseGrant, right)) {
			reasonOfFailure = new String("The Grant in the supplied License does not grant the Right: " + 
							queryRight.toString() + ", which would be required to perform the requested action.");
			evalGrant.setDisplayString(reasonOfFailure);
			evalGrant.setAuthorisationResult(AuthorisationResult.RIGHT_IS_NOT_MET);
			return evalGrant;
		}					

		//VERIFY THE CONDITIONS
		ConditionManager conditionInterpreter = new ConditionManager();
		List<MXMObject> conditions = licenseGrant.getConditions(JAXBElement.class.getCanonicalName());
		
//FIXME-security		
//		SecureRepositoryManager secureRepositoryManager = securityEngine.getSecureRepositoryManager();
//		String secureRepositoryPath = securityEngine.getPropertyValue(SecurityEngineKeys.SECURE_INFO_STORAGE_PATH);
//		QName currentCountry = secureRepositoryManager.getCountry(new MXMAdapter(secureRepositoryPath));
//		QName currentRegion = secureRepositoryManager.getRegion(new MXMAdapter(secureRepositoryPath));
//		boolean isCommercial = secureRepositoryManager.isCommercial(new MXMAdapter(secureRepositoryPath));
//		
//		ConditionResult conditionResult = conditionInterpreter.verifyConditions(conditions, currentCountry, currentRegion, isCommercial);
//		evalGrant.setConditionEvaluationResult(conditionResult);
//		if (conditionResult.getResult() == ConditionResultType.SUCCESSFULLY_MET) {
//			evalGrant.setDisplayString("The Principal is authorised to exercise the right over the Resource.");
//			evalGrant.setAuthorisationResult(AuthorisationResult.AUTHORISATION_SUCCEEDED);
//			return evalGrant;			
//		}
//		else if (conditionResult.getResult() == ConditionResultType.COPYRIGHT_NOTICE) {
//			evalGrant.setDisplayString("The Principal may exercise the right over the Resource subject to one or more ORC Conditions.");
//			evalGrant.setAuthorisationResult(AuthorisationResult.ORC_CONDITIONS_FOUND);
//			return evalGrant;			
//		}
//		else if (conditionResult.getResult() == ConditionResultType.PAYMENT) {
//			evalGrant.setDisplayString("The Principal may exercise the right over the Resource upon making a transaction.");
//			evalGrant.setAuthorisationResult(AuthorisationResult.PAYMENT_CONDITIONS_FOUND);
//			return evalGrant;			
//		}
//		else {
//			evalGrant.setDisplayString("The Principal may not exercise the right over the Resource because a number of Conditions were not met.");
//			evalGrant.setAuthorisationResult(AuthorisationResult.CONDITIONS_ARE_NOT_MET);
//			return evalGrant;			
//		}
		//DUMMY next line
		return evalGrant;
	}
	
	private boolean verifyResource (Grant licenseGrant, MXMObject resource) throws RELEngineException, SecurityEngineException {
		if(licenseGrant.getResource().getContentType().equals(resource.getContentType())){
			if(licenseGrant.isDigitalResource()){
				String licenseResourceURI = licenseGrant.getDigitalResource().getNonSecureReferenceURI();
				String queryResourceURI = ((DigitalResource)resource.getContent()).getNonSecureReferenceURI();
				return (licenseResourceURI.equals(queryResourceURI));
			} 
			else if (licenseGrant.isProtectedResource()) {
				String licenseResourceURI = licenseGrant.getProtectedResource().getDigitalResource().getNonSecureReferenceURI();
				String queryResourceURI = ((ProtectedResource)resource.getContent()).getDigitalResource().getNonSecureReferenceURI();
				return (licenseResourceURI.equals(queryResourceURI));
			}
			else if (licenseGrant.isDiReference()) {
				String licenseDiReferenceURI = licenseGrant.getDiReferenceResource().getIdentifier();
				String queryDiReferenceURI = ((DiReference)resource.getContent()).getIdentifier();
				return (licenseDiReferenceURI.equals(queryDiReferenceURI));
			}
			else if (licenseGrant.isServiceReferenceResource()) {
				String licenseServiceLocation = licenseGrant.getServiceReferenceResource().getServiceLocation();
				String queryServiceLocation = ((ServiceReference)resource.getContent()).getServiceLocation();
				return (licenseServiceLocation.equals(queryServiceLocation));
			}
			else if (licenseGrant.isPropertyUri()) {
				String licensePropertyUri = ((PropertyUri)licenseGrant.getPropertyUri().getContent()).getDefinition();
				String queryPropertyUri = ((PropertyUri)resource.getContent()).getDefinition();
				return (licensePropertyUri.equals(queryPropertyUri));
			}
		}
		return false;
	}
	
	private boolean verifyRight (Grant licenseGrant, Right right) throws RELParsingException {
		Right licenseRightElement = licenseGrant.getRight();
		return licenseRightElement.equals(right);
	}
	////////////////////////

	private EvaluatedGrant verifyGrant(Grant licenseGrant, Grant queryGrant) 
	throws RELEngineException, SecurityEngineException {

		return verifyGrant(licenseGrant, queryGrant.getPrincipal(), queryGrant.getRight(), 
											queryGrant.getResource(), null);
	}
	
//	private boolean verifyResource (Grant licenseGrant, Grant queryGrant) throws RELEngineException, SecurityEngineException {
//		return verifyResource(licenseGrant, queryGrant.getResource());
//	}
	
	private EvaluatedGrantGroup verifyGrantGroup(GrantGroup licenseGrantGroup,GrantGroup queryGrantGroup) 
	throws RELEngineException, SecurityEngineException{
		String reasonOfFailure = null;
		
		//prepare the result object
		EvaluatedGrantGroup evalGrantGroup = new EvaluatedGrantGroup(licenseGrantGroup);
		
		// VERIFY THE PRINCIPAL
		if (! this.verifyPrincipal(licenseGrantGroup.getPrincipal(), queryGrantGroup.getPrincipal()) ){
			reasonOfFailure = new String("The Grant in the supplied License does not grant Rights to the Principal specified in the Query.");
			evalGrantGroup.setDisplayString(reasonOfFailure);
			evalGrantGroup.setAuthorisationResult(AuthorisationResult.PRINCIPAL_IS_NOT_MET);
			return evalGrantGroup;
		}
		
		// VERIFY THE RESOURCE (recursively)
		if (! this.verifyResource(licenseGrantGroup, queryGrantGroup)) {
			reasonOfFailure = new String("The Grant in the supplied License does not grant Rights over the Resource specified in the Query.");
			evalGrantGroup.setDisplayString(reasonOfFailure);
			evalGrantGroup.setAuthorisationResult(AuthorisationResult.RESOURCE_IS_NOT_MET);
			return evalGrantGroup;
		}					

		//VERIFY THE CONDITIONS
		ConditionManager conditionInterpreter = new ConditionManager();
		List<MXMObject> conditions = licenseGrantGroup.getConditions(JAXBElement.class.getCanonicalName());
		
//FIXME-security	
//		SecureRepositoryManager secureRepositoryManager = securityEngine.getSecureRepositoryManager();
//		String secureRepositoryPath = securityEngine.getPropertyValue(SecurityEngineKeys.SECURE_INFO_STORAGE_PATH);
//		QName currentCountry = secureRepositoryManager.getCountry(new MXMAdapter(secureRepositoryPath));
//		QName currentRegion = secureRepositoryManager.getRegion(new MXMAdapter(secureRepositoryPath));
//		boolean isCommercial = secureRepositoryManager.isCommercial(new MXMAdapter(secureRepositoryPath));
//		
//		ConditionResult conditionResult = conditionInterpreter.verifyConditions(conditions, currentCountry, currentRegion, isCommercial);
//		evalGrantGroup.setConditionEvaluationResult(conditionResult);
//		if (conditionResult.getResult() == ConditionResultType.SUCCESSFULLY_MET) {
//			evalGrantGroup.setDisplayString("The Principal is authorised to exercise the right over the Resource.");
//			evalGrantGroup.setAuthorisationResult(AuthorisationResult.AUTHORISATION_SUCCEEDED);
//			return evalGrantGroup;			
//		}
//		else if (conditionResult.getResult() == ConditionResultType.COPYRIGHT_NOTICE) {
//			evalGrantGroup.setDisplayString("The Principal may exercise the right over the Resource subject to one or more ORC Conditions.");
//			evalGrantGroup.setAuthorisationResult(AuthorisationResult.ORC_CONDITIONS_FOUND);
//			return evalGrantGroup;			
//		}
//		else if (conditionResult.getResult() == ConditionResultType.PAYMENT) {
//			evalGrantGroup.setDisplayString("The Principal may exercise the right over the Resource upon making a transaction.");
//			evalGrantGroup.setAuthorisationResult(AuthorisationResult.PAYMENT_CONDITIONS_FOUND);
//			return evalGrantGroup;			
//		}
//		else {
//			evalGrantGroup.setDisplayString("The Principal may not exercise the right over the Resource because a number of Conditions were not met.");
//			evalGrantGroup.setAuthorisationResult(AuthorisationResult.CONDITIONS_ARE_NOT_MET);
//			return evalGrantGroup;			
//		}
		//DUMMY next line
		return evalGrantGroup;
	}
	
	private boolean verifyResource(GrantGroup licenseGrantGroup, GrantGroup queryGrantGroup) throws RELEngineException, SecurityEngineException {
		boolean grantOrGrantGroupFounded = false;
		if (licenseGrantGroup.grantOrgrantGroupExist() && queryGrantGroup.grantOrgrantGroupExist()){
			List<MXMObject> listLicense = licenseGrantGroup.getGrantOrGrantGroupResource();
			List<MXMObject> listQuery = queryGrantGroup.getGrantOrGrantGroupResource();
			for (MXMObject mxmObjectLicense : listLicense) {
				grantOrGrantGroupFounded = false;
				for (MXMObject mxmObjectQuery : listQuery) {
					if(mxmObjectLicense.getContentType().equals(org.iso.mpeg.mxm.engine.relTE.schema.Grant.class.getCanonicalName()) &&
							mxmObjectQuery.getContentType().equals(org.iso.mpeg.mxm.engine.relTE.schema.Grant.class.getCanonicalName())){
						AuthorisationResult result = 
							verifyGrant((Grant)mxmObjectLicense.getContent(), (Grant)mxmObjectQuery.getContent()).getAuthorisationResult();
						if((result.equals(AuthorisationResult.AUTHORISATION_SUCCEEDED))){
							grantOrGrantGroupFounded = true;
							break;
						}
						continue;
						
					}
					else if (mxmObjectLicense.getContentType().equals(org.iso.mpeg.mxm.engine.relTE.schema.GrantGroup.class.getCanonicalName()) &&
							mxmObjectQuery.getContentType().equals(org.iso.mpeg.mxm.engine.relTE.schema.GrantGroup.class.getCanonicalName())) {
						AuthorisationResult result = 
							verifyGrantGroup((GrantGroup)mxmObjectLicense.getContent(), (GrantGroup)mxmObjectQuery.getContent()).getAuthorisationResult();
						if((result.equals(AuthorisationResult.AUTHORISATION_SUCCEEDED))){
							grantOrGrantGroupFounded = true;
							break;
						}
						continue;
					}
					else
						return false;
				}
				if(!grantOrGrantGroupFounded)
					return false;
			}
		}
		return grantOrGrantGroupFounded;
	}

//	private boolean verifyRight (Grant licenseGrant, Grant queryGrant) throws RELParsingException {
//		Right queryRightElement = queryGrant.getRight();
//		Right licenseRightElement = licenseGrant.getRight();
//		return licenseRightElement.equals(queryRightElement);
//	}

	private boolean verifyPrincipal (MXMObject licensePrincipalObject, MXMObject queryPrincipalObject) throws RELEngineException, SecurityEngineException {
		// modified by Edoardo
		//MXMObject licensePrincipalObject = licenseGrant.getPrincipal();
		if(licensePrincipalObject.getContent() instanceof Principal &&
				queryPrincipalObject.getContent() instanceof Principal){
			
//FIXME-security			
//			Principal licensePrincipal = (Principal)licensePrincipalObject.getContent();
//			if (licensePrincipal == null) 
//				return true;
//			else
//				licensePrincipal.setSecurityInfoParser(securityEngine.getSecurityInfoParser());
//			
//			Principal queryPrincipal = (Principal) queryPrincipalObject.getContent();
//			queryPrincipal.setSecurityInfoParser(securityEngine.getSecurityInfoParser());
//			
//			try {
//				return (licensePrincipal.equals(queryPrincipal).equals(MXMEngineResponse.SUCCESS) ? true : false);
//			} catch (MXMEngineException e) {
//				throw (SecurityEngineException)e;
//			}
		}
		else if (licensePrincipalObject.getContent() instanceof ForAll &&
					queryPrincipalObject.getContent() instanceof PropertyPossessor ) {
			
			ForAll forAll = (ForAll)licensePrincipalObject.getContent();
			
//FIXME-security			
//			PropertyPossessor queryPrincipal = (PropertyPossessor) queryPrincipalObject.getContent();
//			queryPrincipal.setSecurityInfoParser(securityEngine.getSecurityInfoParser());
//			
//			List<PropertyPossessor> pps = forAll.getPropertyPossessor();
//			for (PropertyPossessor propertyPossessor : pps) {
//				propertyPossessor.setSecurityInfoParser(securityEngine.getSecurityInfoParser());
//				try {
//					if(propertyPossessor.equals(queryPrincipal).equals(MXMEngineResponse.SUCCESS))
//						return true;
//				} catch (MXMEngineException e) {
//					// TODO Auto-generated catch block
//					e.printStackTrace();
//					throw new RELEngineException("Unable to compare propertyPossessor");
//				}
//			}
		}
		return false;
	}

	public Issuer getIssuersToVerify() {
		if (this.authorisationResponse == null)
			return null;
		else
			return authorisationResponse.getIssuerToBeVerified();
	}

	public Grant getSuccessfulGrant() {
		if (this.authorisationResponse == null)
			return null;
		else
			return authorisationResponse.getSuccessfulGrant().getGrant();
	}
		
}