package eu.convergence.middleware.tool.er;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ER;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.EventTriggerType;

/**
 * This tool allows a user to create and parse a ER (Event Report) and/or ERR (Event Report Request).
 * The user may use the Event Report Tool to create the ERR and after that will associate the ERR with
 * the VDI (Resource, Subscription or Publication)
 */
public class EventReportTool {
    private static EventReportTool INSTANCE;

    private EventReportEngine erEngine;

    private EventReportTool() {
        try {
            erEngine = (EventReportEngine) MXM.getInstance().getDefaultEngine(MXMEngineName.EventReportTE.toString());
        } catch (MXMEngineNotFoundException e) {
            e.printStackTrace();
        } catch (MXMConfigurationException e) {
            e.printStackTrace();
        }
    }

    public static synchronized EventReportTool getInstance() {
        if (null == INSTANCE) {
            INSTANCE = new EventReportTool();
        }

        return INSTANCE;
    }

    /**
     * The user provides the verb as the event trigger type.<br/>
     * The user may use the Event Report Tool to create the ERR and after that will associate the ERR with the VDI
     * (Resource, Subscription or Publication).<br/>
     * The ERR contains the peer service access point name<br/><br/>
     * <p/>
     * In order to create an ERR, the tool will do the following operation:
     * <p/>
     * 1. create an ERRobject<br/>
     * 2. set the identifier (internal generated)<br/>
     * 3. set the verb (provided as method parameter)<br/>
     * 4. set the SAP (get it from the EREngine)
     *
     * @param verb A string representing the event trigger type.
     * @return The event report request object.
     */
    public ERR createERR(EventTriggerType verb) {
        try {
            ERR err = erEngine.getEventReportEngineSchemaHandler().getERR();
            err.setOperation(verb.getOperation());
            err.setSAP(erEngine.getEventReportEngineTechHandler().getEREngineParams().getSAP());

            return err;
        } catch (Exception e) {
            e.printStackTrace();
        }

        return null;
    }

    /**
     * Parse an ERR and return the ERR identifier.
     *
     * @param eventReportRequest The event request report object.
     * @return The event report request identifier
     * @deprecated Use err.getERRIdentifier() instead
     */
    @Deprecated
    public String getERRIdentifier(ERR eventReportRequest) {
        /*try {
            ERRParser errParser = erEngine.getERRParser();
            MXMEngineResponse response = errParser.parseERR(new MXMAdapter(eventReportRequest));

            if (MXMEngineResponse.SUCCESS == response) {
                return errParser.getERRIdentifier();
            }
        } catch (EventReportException e) {
            e.printStackTrace();
        }
        return null;*/

        return eventReportRequest.getERRIdentifier();
    }

    /**
     * Parse an ER and return the ERR identifier.
     *
     * @param eventReport The event report object
     * @return A string representing the event report request identifier
     * @deprecated User er.getERRIdentifier() instead
     */
    @Deprecated
    public String getERRIdentifier(ER eventReport) {
        /*try {
            ERParser erParser = erEngine.getERParser();
            MXMEngineResponse response = erParser.parseER(new MXMAdapter(eventReport));

            if (MXMEngineResponse.SUCCESS == response) {
                return erParser.getERRIdentifier();
            }
        } catch (EventReportException e) {
            e.printStackTrace();
        }
        return null;*/
        return eventReport.getERRIdentifier();
    }

    /**
     * Parse an ER and return the DIDL identifier
     *
     * @param eventReport The event report object
     * @return A string representing the DIDL identifier
     * @deprecated Use er.getDIIId() instead
     */
    @Deprecated
    public String getDIDLIId(ER eventReport) {
        /*try {
            ERParser erParser = erEngine.getERParser();
            MXMEngineResponse response = erParser.parseER(new MXMAdapter(eventReport));

            if (MXMEngineResponse.SUCCESS == response) {
                return erParser.getPublicationVdiId();
            }
        } catch (EventReportException e) {
            e.printStackTrace();
        }
        return null;*/

        return eventReport.getDIIId();
    }


//    public String getVerb(ER eventReport) {
//        // ??? - maybe parse an ERR and return the verb = operation (see the method below)
//        return null;
//    }

    /**
     * Parse an ER and return the verb (match, notify etc)
     *
     * @param eventReportRequest The event request report object
     * @return A string representing the event trigger type
     * @deprecated  Use err.getOperation() instead
     */
    @Deprecated
    public String getVerb(ERR eventReportRequest) {
        /*try {
            ERRParser errParser = erEngine.getERRParser();
            MXMEngineResponse response = errParser.parseERR(new MXMAdapter(eventReportRequest));

            if (MXMEngineResponse.SUCCESS == response) {
                return errParser.getEventTriggerType();
            }
        } catch (EventReportException e) {
            e.printStackTrace();
        }
        return null;*/

        return eventReportRequest.getOperation();
    }
}