package eu.convergence.middleware.tool.security;


import java.security.Key;
import java.security.KeyPair;
import java.security.PublicKey;

import javax.crypto.SecretKey;
import javax.security.cert.CertificateException;
import javax.security.cert.X509Certificate;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMInvalidConfigurationException;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey;
import org.w3c.dom.Document;


/**
 * Provides general confidentiality methods along with methods for signature handling. In
 * the MXMConfiguration.xml you can choose whether you like to use the CONVERGENCE smart
 * card as crypto module or a less secure off-card solution. General crypto module
 * settings can also be found in the MXMConfiguration.xml.
 * 
 * @author Andreas Kohlos
 */
public class SecurityTool
{
    private static SecurityTool SECURITY_TOOL = null;

    /**
     * @return
     */
    public static synchronized SecurityTool getInstance()
    {
        if (SECURITY_TOOL == null)
        {
            SECURITY_TOOL = new SecurityTool();
        }
        return SECURITY_TOOL;
    }

    private SecurityEngine securityTE = null;

    /**
     * 
     */
    private SecurityTool()
    {
        try
        {
            this.securityTE = (SecurityEngine)MXM.getInstance().getDefaultEngine(
                MXMEngineName.SecurityTE.toString());
        }
        catch (final MXMEngineNotFoundException e)
        {
            e.printStackTrace();
        }
        catch (final MXMConfigurationException e)
        {
            e.printStackTrace();
        }
    }

    /**
     * Generates a digital signature of the input DOM object using the private key of the
     * crypto module. Does currently only support RSA keys and uses the
     * 'SHA256withRSA/PSSandMGF1' algorithm to sign.
     * 
     * @param doc The input W3C DOM object.
     * @return An MXM-Signature object which does also include the related public key.
     */
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature generateSignature(
        final Document doc)
    {
        try
        {
            return this.securityTE.getSecurityEngineTechnologyHandler()
                .getSignatureManager()
                .generateSignature(doc, this.securityTE.getPropertyValue("appAlias"));
        }
        catch (final MXMInvalidConfigurationException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
        catch (final SecurityEngineException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
        catch (final MXMEngineException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
    }

    /**
     * Generates a random AES key.
     * 
     * @param keyLength The length of the AES key.
     * @return The AES key as Java SecretKey object.
     */
    public SecretKey generateSymmetricKey(final int keyLength)
    {
        try
        {
            return this.securityTE.getSecurityEngineTechnologyHandler().getKeyManager()
                .generateRandomKey("AES", keyLength);
        }
        catch (final SecurityEngineException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
        catch (final MXMEngineException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
    }

    /**
     * Decrypts the provided input data by using the wrapped key from the hard disk. After
     * unwrapping by using the private key of the locally active crypto module, the
     * symmetric key is used for decrypting the intut data.
     * 
     * @param data The cryptogram which is to be decrypted.
     * @return The decrypted plaintext.
     */
    public byte[] hybridDecrypt(final byte[] data)
    {
        // TODO get multiple userIDs involved along with user certificates
        // TODO retrieve wrapped keys "from the cloud"
        byte[] plaintext = null;
        SecretKey symmetricKey = null;
        EncryptedKey wrappedKey = null;
        try
        {
            // Load wrapped key from local file system.
            wrappedKey = (EncryptedKey)this.securityTE
                .getSecurityEngineTechnologyHandler().getFileAndObjectHandler()
                .readObjectFromFile("./wrappedKey.bin");
            // Unwrap symmetric key by using the private key from the crypto module.
            symmetricKey = this.securityTE
                .getSecurityEngineTechnologyHandler()
                .getKeyManager()
                .unwrap(wrappedKey, this.securityTE.getPropertyValue("appAlias"),
                    this.securityTE.getPropertyValue("confPIN"));
            // Decipher date by using the symmetric key.
            plaintext = this.securityTE.getSecurityEngineTechnologyHandler()
                .getKeyManager().decrypt(data, symmetricKey, "AES/CBC/PKCS7Padding");
        }
        catch (final SecurityEngineException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
        catch (final MXMEngineException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
        return plaintext;
    }

    /**
     * Rolls a symmetric key which is used to encrypt the input data. Afterwards, this
     * symmetric key is asymmetrically wrapped by using the public key of the locally
     * active crypto module.
     * 
     * @param data The plaintext which is to be encrypted
     * @return Currently the method returns only the cryptogram of the symmetric
     *         encryption and writes the wrapped key to harddisk.
     */
    public byte[] hybridEncrypt(final byte[] data)
    {
        // TODO get multiple userIDs involved along with user certificates
        // TODO store wrapped keys "to the cloud"
        byte[] cryptogram = null;
        SecretKey symmetricKey = null;
        EncryptedKey wrappedKey = null;
        try
        {
            // Retrieve the public key for the current application from the crypto module.
            final X509Certificate x509Cert = X509Certificate.getInstance(this.securityTE
                .getSecurityEngineTechnologyHandler()
                .getKeyManager()
                .getCertificate(this.securityTE.getPropertyValue("appAlias"),
                    this.securityTE.getPropertyValue("gakp")).getX509Certificate());
            final PublicKey publicKey = x509Cert.getPublicKey();
            // Generate a random symmetric key.
            symmetricKey = this.securityTE.getSecurityEngineTechnologyHandler()
                .getKeyManager().generateRandomKey("AES", 128);
            // Encrypt data with symmetric key.
            cryptogram = this.securityTE.getSecurityEngineTechnologyHandler()
                .getKeyManager().encrypt(data, symmetricKey, "AES").getCipherData()
                .getCipherValue();
            // Wrap the symmetric key.
            wrappedKey = this.securityTE.getSecurityEngineTechnologyHandler()
                .getKeyManager().wrap(symmetricKey, "RSA", publicKey);
            // Store the wrapped key on local file system.
            this.securityTE.getSecurityEngineTechnologyHandler()
                .getFileAndObjectHandler()
                .writeObjectToFile("./wrappedKey.bin", wrappedKey);
        }
        catch (final SecurityEngineException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
        catch (final MXMEngineException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
        catch (final MXMInvalidConfigurationException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
        catch (final CertificateException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
        return cryptogram;
    }

    /**
     * Initializes the crypto module in order to sequentially decrypt multiple cryptograms
     * by using the StreamHybridDecrypt () method afterwards.
     * 
     * @return true if initialization was successful.
     * @throws Exception
     */
    public boolean initStreamHybridDecrypt() throws Exception
    {
        return this.securityTE
            .getSecurityEngineTechnologyHandler()
            .getKeyManager()
            .initStreamUnwrap(null, this.securityTE.getPropertyValue("appAlias"),
                this.securityTE.getPropertyValue("confPIN"));
    }

    /**
     * Decrypts the provided input cryptogram by using the wrapped key from the harddisk.
     * If the initStreamHybridDecrypt() has been successfully executed before, this method
     * can repeatedly be executed until the crypto module is reset.
     * 
     * @param data The cryptogram which is to be decrypted.
     * @return
     * @throws Exception
     */
    public byte[] StreamHybridDecrypt(final byte[] data) throws Exception
    {
        // TODO get multiple userIDs involved along with user certificates
        // TODO retrieve wrapped keys "from the cloud"
        byte[] plaintext = null;
        SecretKey symmetricKey = null;
        EncryptedKey wrappedKey = null;
        try
        {
            // Load wrapped key from local file system.
            wrappedKey = (EncryptedKey)this.securityTE
                .getSecurityEngineTechnologyHandler().getFileAndObjectHandler()
                .readObjectFromFile("./wrappedKey.bin");
            // Unwrap symmetric key by using the private key from the crypto module.
            symmetricKey = this.securityTE.getSecurityEngineTechnologyHandler()
                .getKeyManager().streamUnwrapAfterInit(wrappedKey);
            // Decipher date by using the symmetric key.
            plaintext = this.securityTE.getSecurityEngineTechnologyHandler()
                .getKeyManager().decrypt(data, symmetricKey, "AES/CBC/PKCS7Padding");
        }
        catch (final SecurityEngineException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
        catch (final MXMEngineException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
        return plaintext;
    }

    /**
     * Symmetrically decrypts the input data by using the provided symmetric key.
     * Currently only uses 'AES/CBC/PKCS7Padding' algorithm.
     * 
     * @param The cryptogram which is to be decrypted.
     * @param symmetricKey The key which is used for decryption.
     * @return The decrypted plaintext as byte array.
     */
    public byte[] symmetricDecrypt(final byte[] data, final SecretKey symmetricKey)
    {
        try
        {
            return this.securityTE.getSecurityEngineTechnologyHandler().getKeyManager()
                .decrypt(data, symmetricKey, "AES/CBC/PKCS7Padding");
        }
        catch (final SecurityEngineException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
        catch (final MXMEngineException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
    }

    /**
     * Symmetrically encrypts the input data by using the provided symmetric key.
     * Currently only supports 'AES' algorithm. The implementation uses the zero vector as
     * IV.
     * 
     * @param data The data to be encrypted.
     * @param symmetricKey the key to be used for encypting the data.
     * @return The encrypted data as byte array.
     */
    public byte[] symmetricEncrypt(final byte[] data, final SecretKey symmetricKey)
    {
        try
        {
            return this.securityTE.getSecurityEngineTechnologyHandler().getKeyManager()
                .encrypt(data, symmetricKey, "AES/CBC/PKCS7Padding").getCipherData()
                .getCipherValue();
        }
        catch (final SecurityEngineException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
        catch (final MXMEngineException e)
        {
            System.err.println(e.getMessage());
            return null;
        }
    }

    /**
     * Verifies a signature on the input DOM document by using the provided Signature.
     * Currently the underlying algorithm is restricted to 'SHA256withRSA/PSS'.
     * 
     * @param doc The input W3C DOM object.
     * @param signature The signature for the provided DOM object which is to be verified.
     * @return true if successful.
     */
    public boolean verifySignature(final Document doc,
        final org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature signature)
    {
        try
        {
            return this.securityTE.getSecurityEngineTechnologyHandler()
                .getSignatureManager().verifySignature(doc, signature);
        }
        catch (final SecurityEngineException e)
        {
            System.err.println(e.getMessage());
            return false;
        }
        catch (final MXMEngineException e)
        {
            System.err.println(e.getMessage());
            return false;
        }
    }

    /**
     * Generates a digital signature of the input data using the private key of the crypto
     * module. Does currently only support RSA keys and uses the
     * 'SHA256withRSA/PSSandMGF1' algorithm to sign.
     * 
     * @param message The input data to be signed.
     * @return An MXM-Signature object which does also include the related public key.
     * @throws Exception
     */
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature generateSignature(
        final byte[] message) throws Exception
    {
        try
        {
            return this.securityTE.getSecurityEngineTechnologyHandler()
                .getSignatureManager()
                .generateSignature(message, this.securityTE.getPropertyValue("appAlias"));
        }
        catch (final SecurityEngineException e)
        {
            System.err.println(e.getMessage());
        }
        catch (final MXMEngineException e)
        {
            System.err.println(e.getMessage());
        }
        return null;
    }

    // public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature
    // generateSignature(
    // final String message) throws Exception
    // {
    // throw new Exception("METHOD_NOT_IMPLEMENTED");
    // }
    //
    // public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data
    // getUserCertificate(
    // final String userID) throws Exception
    // {
    // throw new Exception("METHOD_NOT_IMPLEMENTED");
    // }
    /**
     * Verifies a signature on the input data by using the provided Signature. Currently
     * the underlying algorithm is restricted to 'SHA256withRSA/PSS'.
     * 
     * @param message The input data.
     * @param signature The signature for the provided data which is to be verified.
     * @return true if successful.
     * @throws Exception
     */
    public boolean verifySignature(final byte[] message,
        final org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature signature)
        throws Exception
    {
        try
        {
            return this.securityTE.getSecurityEngineTechnologyHandler()
                .getSignatureManager().verifySignature(message, signature);
        }
        catch (final SecurityEngineException e)
        {
            System.err.println(e.getMessage());
        }
        catch (final MXMEngineException e)
        {
            System.err.println(e.getMessage());
        }
        return false;
    }

    // public boolean verifySignature(
    // final String message,
    // final org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature signature,
    // final org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data
    // userCertificate)
    // throws Exception
    // {
    // throw new Exception("METHOD_NOT_IMPLEMENTED");
    // }
    /**
     * Causes the crypto module to generate a fresh RSA key pair.
     * 
     * @param keyIdentifier Is one of 'sig', 'conf' or 'auth' and sets the usage scenario
     *            of the generated pair.
     * @param keyLength The length of RSA modulus to be generated for this key pair.
     *            Currently supported: RSA 1024/2048.
     * @return The public key of the generated key pair.
     */
    public PublicKey generateKeyPair(String keyIdentifier, int keyLength)
    {
        try
        {
            return this.securityTE
                .getSecurityEngineTechnologyHandler()
                .getCertificateManager()
                .generateRSAKeyPair(this.securityTE.getPropertyValue("appAlias"),
                    keyIdentifier, keyLength);
        }
        catch (SecurityEngineException e)
        {
            e.printStackTrace();
        }
        catch (MXMEngineException e)
        {
            e.printStackTrace();
        }
        return null;
    }

    /**
     * Aysymmetrically wraps the provided symmetric secret key. Currently uses the public
     * key of the locally active crypto module.
     * 
     * @param secretKey The key to be wrapped. This key will be needed for decryption of
     *            data encrypted with it.
     * @return The wrapped key.
     */
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey wrap(
        SecretKey secretKey)
    {
        PublicKey key = null;
        String appAlias = this.securityTE.getPropertyValue("appAlias");
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey wrappedKey = null;
        try
        {
            key = this.securityTE.getSecurityEngineTechnologyHandler()
                .getAuthenticationManager().getPublicKey(appAlias, "conf", "");
            if (key == null)
            {
                System.out.println("No conf key found! Generating one..");
                key = this.securityTE.getSecurityEngineTechnologyHandler()
                    .getCertificateManager().generateRSAKeyPair(appAlias, "conf", 1024);
            }
            wrappedKey = this.securityTE.getSecurityEngineTechnologyHandler()
                .getKeyManager().wrap(secretKey, null, key);
        }
        catch (SecurityEngineException e)
        {
            e.printStackTrace();
        }
        catch (MXMEngineException e)
        {
            e.printStackTrace();
        }
        catch (Exception e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return wrappedKey;
    }

    /**
     * Asymmetrically unwraps the provided wrapped input key. Unwrapping will be performed
     * using the private key on the active CONVERGENCE crypto module . The wrapped key is
     * transmitted to the CONVERGENCE crypto module for unwrapping.
     * 
     * @param wrappedKey The wrapped key to be unwrapped.
     * @return The unwrapped symmetric secret key
     */
    public SecretKey unWrap(EncryptedKey wrappedKey)
    {
        String appAlias = this.securityTE.getPropertyValue("appAlias");
        SecretKey unwrappedKey = null;
        try
        {
            unwrappedKey = this.securityTE.getSecurityEngineTechnologyHandler()
                .getKeyManager().unwrap(wrappedKey, appAlias, "");
        }
        catch (SecurityEngineException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        catch (MXMEngineException e)
        {
            // TODO Auto-generated catch block
            e.printStackTrace();
        }
        return unwrappedKey;
    }
}
