package eu.convergence.middleware.tool.vdi;

import java.util.Date;

import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;


/**
 * This interface specifies the creation methods that are provided to a developer for creating a Publication VDI (P-VDI).
 * It provides methods for adding metadata, licenses, event report requests to a P-VDI. It also exposes methods that a user
 * can perform to a P-VDI, namely injecting it in the overlay, adding a digital signature or storing it. These functionalities
 * are supported by various middleware engines with the VDI TE having the central role, while the Metadata TE is responsible for
 * creating the essential CONVERGENCE metadata elements of the P-VDI.
 * 
 * An example of creating and publishing a P-VDI follows :
 * <pre>
 * PvdiCreator pubVdiTool = VdiTool.newPvdiCreator("urn:convergence:myRvdiId");
 * pubVdiTool.setExpiryDate(new Date());
 * pubVdiTool.setFractal("photo");
 * pubVdiTool.generatePVDI();
 * pubVdiTool.publish();
 * </pre>
 */
public interface PvdiCreator
{
	/**
     * Sets the VDI destination fractal, which is identified by a string (e.g. photo fractal).
     * This is later used by the OverlayTE to propagate the VDI to the semantic overlay.
     *
     * @param fractal The fractal.
     */
	public void setFractal(String fractal);
	
	/**
     * Adds an event report request associated with the VDI.
     * The user may create the ERR using the Event Report Tool.
     *
     * @param err A event report request.
     */
	public void addERR(ERR err);
	
	/**
     * Adds a keyword description to the VDI, e.g. comedy
     *
     * @param keyword A keyword.
     */
	public void addKeyword(String keyword);
	
	/**
     * This method adds a tag description to the VDI.
     * The tag differs to the keyword description in the way that tags are URIs backed up by a controlled vocabulary.
     *
     * @param tag A tag.
     */
	public void addTag(String tag);
	
	/**
     * Adds a field/value pair description to the VDI, e.g. genre, comedy. Note that value is of type object. 
     * This enables the use of non-String values, namely double or integer values or dates. 
     * This way the value preserves its data type
     *
     * @param field A field.
     * @param value A value.
     */
	public void addFieldValue(String field, Object value);
	
	/**
     * The method adds structured metadata descriptions to the VDI.
     * Users may create structured metadata objects using the Metadata APIs, producing sophisticated
     * XML descriptions based on XML schemata e.g. MPEG-7 or RDF/XML descriptions based on ontologies.
     *
     * @param structuredData A structuredData object.
     */
	public void addStructuredData(StructuredData structuredData);
	
	/**
     * Adds a license to the VDI.
     * The user may create the license object with the License Tool, stating the issuer, grants to principals etc.
     * Please note that the VDI can have multiple licenses embedded in it.
     *
     * @param license A REL license.
     */
	public void addLicense(License license);
	
	/**
     * Sets the start date of validity of the VDI.
     *
     * @param startDate A date.
     */
	public void setStartDate(Date startDate);
	
	/**
     * Sets the expiration date of the VDI.
     *
     * @param expiryDate A date.
     */
	public void setExpiryDate(Date expiryDate);
	
	/**
     * Adds a signature to the VDI.
     * The user may create the signature object with the Security Tool.
     * Please note that the signature is not a simple hash of the VDI, but a complex XML signature.
     *
     * @param signature A XML Signature.
     */
	public void addSignature(Signature signature);
	
	/**
     * With this method the user may create the P-VDI.
     * The user should have already passed, metadata, licenses, ERRs etc.
     * The Publish VDI Tool employs the Orchestrator TE to build a chain of engines 
     * and fuse all the submitted data and formulate a valid VDI object.
     */
	public DIDL generatePVDI();
	
	/**
     * Publishes the VDI to the CONVERGENCE cloud
     *
     * @return True if the operation is successfully carried out. False otherwise.
     */
	public boolean publish();
}
