package eu.convergence.middleware.tool.vdi;

import java.util.Date;
import java.util.List;
import java.util.Map;

import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;

/**
 * This interface specifies the parsing methods that are provided to a developer for accessing a Publication VDI (P-VDI).
 * It provides methods for extracting metadata, licenses, event report requests from a P-VDI.These functionalities are 
 * supported by various middleware engines with the VDI TE having the central role, while the Metadata TE is responsible 
 * for parsing the essential CONVERGENCE metadata elements of the P-VDI.
 */
public interface PvdiParser
{
	/**
     * The method returns the fractals that the VDI belongs to.
     * 
     * @return The fractal identifier.
     */
	public String getFractal();
	
	/**
     * The method returns all the keywords that describe the VDI.
     * 
     * @return A list of keywords.
     */
	public List<String> getKeywordList();
	
	/**
     * The method returns all the tags that describe the VDI.
     * 
     * @return A list of tags.
     */
	public List<String> getTagList();
	
	/**
     * The method returns all field value descriptions in a map object.
     * Please note that for a given field a user may add more than one values, hence the corresponding list.
     * 
     * @return A mapping between fields and a list of values.
     */
	public Map<String, List<Object>> getFieldValueMap();
	
	/**
     * Returns a list of all the structured data descriptors of the VDI.
     * The user may use the Metadata Tool to further parse the returned metadata.
     * 
     * @return A list of structuredData metadata.
     */
	public List<StructuredData> getStructuredDataList();
	
	/**
     * Returns the identifier of the VDI
     * 
     * @return The identifier of the digital item.
     */
	public String getIdentifier();
	
	/**
     * Returns a list containing all the licenses that are embedded in the VDI.
     * The user may employ the License Tool to parse the licenses.
     * 
     * @return A list of REL licenses.
     */
	public List<License> getLicenseList();
	
	/**
     * Returns the start validity date of the  VDI
     * 
     * @return The start date of validity of the VDI.
     */
	public Date getStartDate();
	
	/**
     * Returns the expiration date of the  VDI
     * 
     * @return The expiration date of the VDI.
     */
	public Date getExpiryDate();
	
	/**
     * Returns a list containing all the signatures that are embedded in the VDI.
     * The user may employ the Security Tool to parse the signatures.
     * 
     * @return A list of XML signatures.
     */
	public List<Signature> getSignatureList();
}
