package eu.convergence.middleware.tool.vdi;

import java.util.Date;
import java.util.List;

import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;


/**
 * This interface specifies the creation methods that are provided to a developer for creating a Resource VDI (R-VDI).
 * It provides methods for adding metadata, licenses, resource references and relationships to other VDIs.
 * It also exposes methods that a user can perform to an R-VDI, namely packaging it with resources, adding
 * a digital signature or storing it. These functionalities are supported by various middleware engines with the VDI TE
 * having the central role, while the Metadata TE is responsible for creating the CONVERGENCE metadata element of the R-VDI.
 * 
 *  An example of creating and publishing a P-VDI follows :
 * <pre>
 * RvdiCreator rvdiTool = VdiTool.newRvdiCreator();
 * rvdiTool.addResource("http://www.ict-convergence.eu/myResource.jpg", "image/jpg");
 * rvdiTool.setExpiryDate(new Date());
 * rvdiTool.addKeyword("myTag");
 * rvdiTool.generateRVDI();
 * if (rvdiTool.store("myRVDI filePath"))
 * {
 * 		rvdiTool.advertise(filePath);
 * }
 * String vdiID = rvdiTool.getIdentifier();
 * </pre>
 */
public interface RvdiCreator
{
	/**
     * Returns the identifier of the VDI
     * 
     * @return The identifier of the digital item.
     */
	public String getIdentifier();
	
	/**
     * Adds a keyword description to the VDI, e.g. comedy
     *
     * @param keyword A keyword.
     */
	public void addKeyword(String keyword);
	
	/**
     * This method adds a tag description to the VDI.
     * The tag differs to the keyword description in the way that tags are URIs backed up by a controlled vocabulary.
     *
     * @param tag A tag.
     */
	public void addTag(String tag);
	
	/**
     * Adds a field/value pair description to the VDI, e.g. genre, comedy. Note that value is of type object. 
     * This enables the use of non-String values, namely double or integer values or dates. 
     * This way the value preserves its data type
     *
     * @param field A field.
     * @param value A value.
     */
	public void addFieldValue(String field, Object value);
	
	/**
     * The method adds structured metadata descriptions to the VDI.
     * Users may create structured metadata objects using the Metadata APIs, producing sophisticated
     * XML descriptions based on XML schemata e.g. MPEG-7 or RDF/XML descriptions based on ontologies.
     *
     * @param structuredData A structuredData object.
     */
	public void addStructuredData(StructuredData structuredData);
	
	/**
     * This method adds a resource reference descriptor to the R-VDI.
     * An optional mimeType parameter gives the option of specifying the content type.
     *
     * @param resourceURL The resource URL.
     * @param mimeType The mimeType of the resource.
     */
	public Resource createResource(String resourceURL, String mimeType, String content);
	
	
	public void addAsset(Resource resource, List<License> licenses, List<ERR> err);
	
	/**
     * Adds a relationship descriptor stating a semantic relationship between another VDI and the VDI to be created.
     * The relationshipType may be, but is not compulsory, a URI identifying ontology relationship.
     *
     * @param relationshipType The relationship type.
     * @param vdiId The identifier of the  digital item having the relationship with.
     */
	public void addRelationship(String relationshipType, String vdiId);
	
	/**
     * This method allows the creation of the R-VDI, provided that metadata, licenses, etc. have been inserted. 
     * The Resource VDI Tool employs the Orchestrator TE to build a chain of engines and fuse all the submitted
     * data and formulate a valid VDI object.
     * 
     * @return DIDL The Digital Item
     */
	public DIDL generateRVDI();
	
	/**
     * Stores the VDI to the specified filePath location.
     * The tool first generates the VDI, if not already done by the user,
     * and stores it to the passed location. 
     * 
     * @return boolean True if the operation is successful. False otherwise.
     */
	public boolean store(String filePath);
	
	/**
     * Advertises the R-VDI in CoNet that is located in filePath. The filepath is optional. 
     * If no valid string is provided, then the store method is first triggered to generate 
     * and store the R-VDI and then the tool employs CoNet TE to advertise it.
     * 
     * @return boolean True if the operation is successful. False otherwise.
     */
	public boolean advertise(String filePath);
	
//	public void pack(byte[][] resources);

//	/**
//     * Adds a license to the VDI.
//     * The user may create the license object with the License Tool, stating the issuer, grants to principals etc.
//     * Please note that the VDI can have multiple licenses embedded in it.
//     *
//     * @param license A REL license.
//     */
//	public void addLicense(License license);
	
	/**
     * Sets the start date of validity of the VDI.
     *
     * @param startDate A date.
     */
	public void setStartDate(Date startDate);
	
	/**
     * Sets the expiration date of the VDI.
     *
     * @param expiryDate A date.
     */
	public void setExpiryDate(Date expiryDate);
	
	/**
     * Adds a signature to the VDI.
     * The user may create the signature object with the Security Tool.
     * Please note that the signature is not a simple hash of the VDI, but a complex XML signature.
     *
     * @param signature A XML Signature.
     */
	public void addSignature(Signature signature);
}
