package eu.convergence.middleware.tool.vdi;

import java.util.Date;
import java.util.List;

import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;

/**
 * This interface specifies the creation methods that are provided to a developer for creating a Subscription VDI (S-VDI).
 * It provides methods for adding conditions, licenses, event report requests to a S-VDI. It also exposes methods that a 
 * user can perform to an S-VDI, namely injecting it in the semantic overlay, adding a digital signature or storing it.
 * These functionalities are supported by various middleware engines with the VDI TE having the central role, while the
 * Metadata TE is responsible for creating and parsing the CONVERGENCE metadata element of the S-VDI and the MPQF TE for
 * formulating the subscription query.
 * 
 *  An example of creating and publishing a S-VDI follows :
 * <pre>
 * ERR err = EventReportTool.getInstance().createERR(EventTriggerType.MATCH);
 * SvdiCreator svdiTool = VdiTool.newSvdiCreator();
 * svdiTool.setFractal("fractal");
 * svdiTool.setExpiryDate(finalDate.getTime());
 * svdiTool.addKeywordCondition(tags);
 * svdiTool.addERR(err);	
 * svdiTool.generateSVDI();
 * String svdi_id = svdiTool.subscribe();
 * subDTO.setsVdiId(svdi_id);
 * </pre>
 */
public interface SvdiCreator
{
	/**
     * Sets the VDI destination fractal, which is identified by a string (e.g. photo fractal).
     * This is later used by the OverlayTE to propagate the VDI to the semantic overlay.
     *
     * @param fractal The fractal.
     */
	public void setFractal(String fractal);
	
	/**
     * Adds an event report request associated with the VDI.
     * The user may create the ERR using the Event Report Tool.
     *
     * @param err A event report request.
     */
	public void addERR(ERR err);
	
	/**
     * This method adds a field value condition to the users subscription query.
     * The user is able to express also inequality conditions using a graterThan or a lessThan operator.
     * Also notice that the value is of object type, preserving this way its datatype e.g. Date.
     *
     * @param field A field.
     * @param operator The comparison operator. "=", ">" or "<".
     * @param value A value.
     */
	public void addFieldValueCondition(String field, String operator, Object value);
	
	/**
	 * Adds a list of keyword conditions to the users subscription query.
	 *
	 * @param keywords A keyword list.
	 */
	public void addKeywordCondition(List<String> keywords);
	
	/**
	 * Adds a list of tag conditions to the users subscription query.
	 *
	 * @param tags A tag list.
	 */
	public void addTagCondition(List<String> tags);
	
	/**
     * With this method the user may create the S-VDI.
     * The user should have already passed, metadata, licenses, errs etc.
     * The Subscription VDI Tool employs the Orchestrator TE to build a chain of engines
     * and fuse all the submitted data and formulate a valid VDI object.
     *
     * @return DIDL The Subscription VDI.
     */
	public DIDL generateSVDI();
	
	/**
     * Sends the S-VDI to the CONVERGENCE cloud
     *
     * @return True if the operation is successfully carried out. False otherwise.
     */
	public String subscribe();
	
	/**
     * Adds a license to the VDI.
     * The user may create the license object with the License Tool, stating the issuer, grants to principals etc.
     * Please note that the VDI can have multiple licenses embedded in it.
     *
     * @param license A REL license.
     */
	public void addLicense(License license);
	
	/**
     * Sets the start date of validity of the VDI.
     *
     * @param startDate A date.
     */
	public void setStartDate(Date startDate);
	
	/**
     * Sets the expiration date of the VDI.
     *
     * @param expiryDate A date.
     */
	public void setExpiryDate(Date expiryDate);
	
	/**
     * Adds a signature to the VDI.
     * The user may create the signature object with the Security Tool.
     * Please note that the signature is not a simple hash of the VDI, but a complex XML signature.
     *
     * @param signature A XML Signature.
     */
	public void addSignature(Signature signature);
	
	/**
     * Adds a triple condition to the sparql query. Variables e.g. ?x can also be stated here.
     *
     * @param subject The subject of the condition.
     * @param predicate The predicate of the condition.
     * @param object The object of the condition.
     */
	public void addSparqlTripleCondition(String subject, String predicate, String object);
	
	/**
     * Adds a triple literal condition to the sparql query. That is, the object of the condition
     * has a datatype e.g. integer.
     *
     * @param subject The subject of the literal condition.
     * @param predicate The predicate of the literal condition.
     * @param object The object of the literal condition.
     */
	public void addSparqlTripleLiteralCondition(String subject, String predicate, Object object);
	
	/**
     * This method enables the definition of a filter condition based on the value of a property.
     * The object must be a variable that is already stated as an object in a triple condition pattern.
     *
     * @param object The variable object of a triple codition that will be used to filter the resultset.
     * @param operator The operator of the filtering e.g. equality "=".
     * @param value The value of the filtering.
     */
	public void addSparqlFilterCondition(String object, String operator, Object value);
	
	/**
     * This method sets the variable that will be returned from the query.
     *
     * @param varName The name of the variable e.g. ?x.
     */
	public void addSparqlReturnVariable(String varName);
	
	/**
     * This method sets a limit for the returned results.
     *
     * @param limit A limit.
     */
	public void setResultLimit(long limit);
}
