package eu.convergence.middleware.tool.vdi;

import java.util.Date;
import java.util.List;

import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;


/**
 * This interface specifies the parsing methods that are provided to a developer for accessing a Subscription VDI (S-VDI).
 * It provides methods for extracting subscription conditions, licenses, event report requests from a S-VDI. It also exposes
 * methods that a user can perform to a S-VDI, namely injecting it in the semantic overlay, adding a digital signature or
 * storing it. These functionalities are supported by various middleware engines with the VDI TE having the central role,
 * while the Metadata TE is responsible for creating and parsing the CONVERGENCE metadata element of the S-VDI and the MPQF TE
 * for parsing the subscription query.
 */
public interface SvdiParser
{
	/**
     * The method returns the fractals that the VDI belongs to.
     * 
     * @return The fractal identifier.
     */
	public String getFractal();
	
	/**
     * Returns the identifier of the VDI
     * 
     * @return The identifier of the digital item.
     */
	public String getIdentifier();
	
	/**
     * Returns a list containing all the licenses that are embedded in the VDI.
     * The user may employ the License Tool to parse the licenses.
     * 
     * @return A list of REL licenses.
     */
	public List<License> getLicenseList();
	
	/**
     * Returns the start validity date of the  VDI
     * 
     * @return The start date of validity of the VDI.
     */
	public Date getStartDate();
	
	/**
     * Returns the expiration date of the  VDI
     * 
     * @return The expiration date of the VDI.
     */
	public Date getExpiryDate();
	
	/**
     * Returns a list containing all the signatures that are embedded in the VDI.
     * The user may employ the Security Tool to parse the signatures.
     * 
     * @return A list of XML signatures.
     */
	public List<Signature> getSignatureList();
}
