package eu.convergence.middleware.tool.vdi;

import java.io.File;
import java.util.ArrayList;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.xml.bind.JAXBElement;


import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.pEngine.createcontentPE.CreateContentEngine;
import org.iso.mpeg.mxm.pEngine.createcontentPE.schemahandler.Asset;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Component;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDLInfo;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.IPMPGeneralInfoDescriptor;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.LicenseCollection;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.RightsDescriptor;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;

import eu.convergence.middleware.core.ConvergenceEngineName;
import eu.convergence.middleware.engine.metadataTE.ConvergenceMetadataEngine;
import eu.convergence.middleware.engine.metadataTE.schemahandler.ConvergenceMetadata;
import eu.convergence.middleware.engine.metadataTE.schemahandler.PublicationMetadata;
import eu.convergence.middleware.engine.metadataTE.technologyhandler.ConvergenceMetadataParser;
import eu.convergence.middleware.engine.metadataTE.technologyhandler.MetadataParser;
import eu.convergence.middleware.engine.mpqfTE.MpqfEngine;
import eu.convergence.middleware.engine.mpqfTE.technologyhandler.SparqlQueryEditor;
import eu.convergence.middleware.engine.overlayTE.exceptions.OverlayEngineException;
import eu.convergence.middleware.engine.vdiTE.Relationship;
import eu.convergence.middleware.engine.vdiTE.VdiEngine;
import eu.convergence.middleware.orchestratorTE.ConvergenceOrchestratorTE;

/**
 * This class offers a default implementation creating and parsing R-VDIs, P-VDIs, and S-VDIs. For this, it implements all
 * relevant interfaces and exposes their methods. Notice that the VDITool object doesn't have a public constructor but provides
 * various static methods for creating it and returning it with the corresponding interface.
 * <br>
 * For example, if the developer wants to create a P-VDI starting from a R-VDI file :
 * <pre>
 * File rvdi = new File("R-VDI filePath");
 * PvdiCreator pubVdiTool = VdiTool.newPvdiCreator(rvdi);
 * </pre>
 * or using just the rvdi identifier :
 * <pre>
 * PvdiCreator pubVdiTool = VdiTool.newPvdiCreator("urn:convergence:myRvdiId");
 * </pre>
 */
public class VdiTool implements RvdiCreator, RvdiParser, PvdiCreator, PvdiParser, SvdiCreator, SvdiParser
{
	private ConvergenceOrchestratorTE orchestrator = null;
	private CreateContentEngine createContentEngine;
	private VdiEngine vdiEngine = null;
	private IPMPEngine ipmpEngine = null;
	private ConvergenceMetadataEngine convMetadataEngine = null;
	private MpqfEngine mpqfEngine;
	
	private List<Asset> assets;
	private SparqlQueryEditor sparqlQuery;
	
	private String identifier;
	private Date startDate;
	private Date expiryDate;
	private String fractal;
	private List<ERR> errs;
	private List<License> licenses;
	private List<String> keywords;
	private List<String> tags;
	private List<StructuredData> structuredMetadata;
	private List<Signature> signatures;
	private List<Relationship> relationships;
	private Map<String,String> resourceUrl2mimeType;
	private HashMap<String,List<Object>> fieldValueEqualMap;
	private HashMap<String,Object> fieldValueGreaterThanMap;
	private HashMap<String,Object> fieldValueLessThanMap;
	private String rvdiId;
	private static String baseOntologyURI = "urn:something:basevdi";
	
	private DIDL didl;
	
	
	public static RvdiCreator newRvdiCreator()
	{
		VdiTool vdiTool = new VdiTool();
		
		vdiTool.assets = new ArrayList<Asset>();
		vdiTool.licenses = new ArrayList<License>();
		vdiTool.keywords = new ArrayList<String>();
		vdiTool.tags = new ArrayList<String>();
		vdiTool.fieldValueEqualMap = new HashMap<String, List<Object>>();
		vdiTool.structuredMetadata = new ArrayList<StructuredData> ();
		vdiTool.signatures = new ArrayList<Signature> ();
		vdiTool.relationships = new ArrayList<Relationship>();
		
		return vdiTool;
	}
	
	public static RvdiParser newRvdiParser(File rvdiFile)
	{
		VdiTool vdiTool = new VdiTool();
		
		vdiTool.assets = new ArrayList<Asset>();
		vdiTool.licenses = new ArrayList<License>();
		vdiTool.keywords = new ArrayList<String>();
		vdiTool.tags = new ArrayList<String>();
		vdiTool.fieldValueEqualMap = new HashMap<String, List<Object>>();
		vdiTool.structuredMetadata = new ArrayList<StructuredData> ();
		vdiTool.signatures = new ArrayList<Signature> ();
		vdiTool.relationships = new ArrayList<Relationship>();
		
		vdiTool.parseRVDI(rvdiFile);
		
		return vdiTool;
	}
	
	public static RvdiParser newRvdiParser(String rvdiId) // TODO check initiation of the fields
	{
		VdiTool vdiTool = new VdiTool();
		
		vdiTool.assets = new ArrayList<Asset>();
		vdiTool.rvdiId = rvdiId;
		vdiTool.errs = new ArrayList<ERR>();
		vdiTool.licenses = new ArrayList<License>();
		vdiTool.keywords = new ArrayList<String>();
		vdiTool.tags = new ArrayList<String>();
		vdiTool.fieldValueEqualMap = new HashMap<String, List<Object>>();
		vdiTool.structuredMetadata = new ArrayList<StructuredData> ();
		vdiTool.signatures = new ArrayList<Signature> ();
		
		vdiTool.parseRVDI(rvdiId);
		
		return vdiTool;
	}
	
	public static PvdiCreator newPvdiCreator(String rvdiId)
	{
		VdiTool vdiTool = new VdiTool();
		
		vdiTool.assets = new ArrayList<Asset>();
		vdiTool.rvdiId = rvdiId;
		vdiTool.errs = new ArrayList<ERR>();
		vdiTool.licenses = new ArrayList<License>();
		vdiTool.keywords = new ArrayList<String>();
		vdiTool.tags = new ArrayList<String>();
		vdiTool.fieldValueEqualMap = new HashMap<String, List<Object>>();
		vdiTool.structuredMetadata = new ArrayList<StructuredData> ();
		vdiTool.signatures = new ArrayList<Signature> ();
		
		return vdiTool;
	}
	
	public static PvdiParser newPvdiParser(File pvdiFile)
	{
		VdiTool vdiTool = new VdiTool();
		
		vdiTool.parsePVDI(pvdiFile);
		
		return vdiTool;
	}
	
	public static PvdiParser newPvdiParser(String pvdiId)
	{
		VdiTool vdiTool = new VdiTool();
		
		vdiTool.parsePVDI(pvdiId);
		
		return vdiTool;
	}

	public static SvdiCreator newSvdiCreator()
	{
		VdiTool vdiTool = new VdiTool();
		
		vdiTool.assets = new ArrayList<Asset>();
		vdiTool.errs = new ArrayList<ERR>();
		vdiTool.licenses = new ArrayList<License>();
		vdiTool.signatures = new ArrayList<Signature> ();
		vdiTool.fieldValueEqualMap = new HashMap<String,List<Object>>();
		vdiTool.fieldValueGreaterThanMap = new HashMap<String,Object>();
		vdiTool.fieldValueLessThanMap = new HashMap<String,Object>();
		vdiTool.sparqlQuery = vdiTool.mpqfEngine.getMpqfTechnologyHandler().getSparqlQueryEditor();
		
		return vdiTool;
	}
	
	public static SvdiParser newSvdiParser(File svdiFile)
	{
		VdiTool vdiTool = new VdiTool();
		
		vdiTool.parseSVDI(svdiFile);
		
		return vdiTool;
	}
	
	public static SvdiParser newSvdiParser(String svdiId)
	{
		VdiTool vdiTool = new VdiTool();
		
		vdiTool.parseSVDI(svdiId);
		
		return vdiTool;
	}
	
	private VdiTool()
	{
		try
		{
			orchestrator = (ConvergenceOrchestratorTE) MXM.getInstance().getDefaultEngine(MXMEngineName.Orchestrator.value());
			vdiEngine = (VdiEngine) MXM.getInstance().getDefaultEngine(MXMEngineName.DigitalItemTE.value());
			createContentEngine = (CreateContentEngine) MXM.getInstance().getDefaultEngine(MXMEngineName.CreateContentPE.value());
			ipmpEngine = (IPMPEngine) MXM.getInstance().getDefaultEngine(MXMEngineName.IPMPTE.value()); // TODO move this to the orchestrators ?
			convMetadataEngine = (ConvergenceMetadataEngine) MXM.getInstance().getDefaultEngine(ConvergenceEngineName.ConvergenceMetadataTE.value());
			mpqfEngine = (MpqfEngine) MXM.getInstance().getDefaultEngine(ConvergenceEngineName.MpqfTE.value());
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	private void parseRVDI(Object rvdi)
	{
		try
		{
			if (rvdi instanceof String)
			{
				String filePath = orchestrator.getConetOrchestrator().fetchResourceFromConet((String)rvdi);
				
				rvdi = new File(filePath);
			}
			
			DIDL rvdiDIDL = vdiEngine.getVDIEngineSchemaHandler().getDIDL();
			rvdiDIDL.parse(new MXMAdapter(rvdi));

			this.identifier = rvdiDIDL.getMPEGMItem().getIdentifier();

			System.out.println("Parsing R-VDI: " + identifier);
			
			// extract rvdi metadata
			orchestrator.getMetadataOrchestrator().extractConvergenceMatadata(rvdiDIDL.getMPEGMItem().getMetadata(), 
										keywords, tags, fieldValueEqualMap, structuredMetadata);
			
			MetadataParser metadataParser = convMetadataEngine.getConvergenceMetadataTechnologyHandler()
					.getMetadataParser(rvdiDIDL.getMPEGMItem().getMetadata());
			
			if (metadataParser instanceof ConvergenceMetadataParser)
			{
				ConvergenceMetadata convergenceMetadata = ((ConvergenceMetadataParser) metadataParser).getConvergenceMetadata();
				
				this.startDate = convergenceMetadata.getStartDate();
				this.expiryDate = convergenceMetadata.getExpiryDate();
			}
			
			for (DIDLInfo didlInfo : rvdiDIDL.getMPEGMDIDLInfo())
			{
				this.signatures.add(didlInfo.getSignature());
			}

			List<Item> itemsTocheck = new ArrayList<Item>();
			itemsTocheck.add(rvdiDIDL.getMPEGMItem());
			
			while (!itemsTocheck.isEmpty())
			{
				Item item = itemsTocheck.get(0);
				
				itemsTocheck.addAll(item.getItem());
				
				IPMPGeneralInfoDescriptor licenseDescriptor = (IPMPGeneralInfoDescriptor) item.getIPMPGeneralInfoDescriptor();

				if (licenseDescriptor!=null)
				{
					for (RightsDescriptor rightsDescriptor : licenseDescriptor.getLicenseCollection().getRightsDescriptor())
					{
						this.licenses.addAll(rightsDescriptor.getLicense().getLicense());
					}
				}
				
				itemsTocheck.remove(0);
			}
			
//			this.relationships = new ArrayList<Relationship>(); // TODO reenable relationships
//
//			List<MXMObject> ots= diParser.getCustomObjects();
//
//			if(ots != null){
//				for(MXMObject  ot:ots)
//				{
//					if(((JAXBElement<?>)ot.getContent()).getValue().getClass().getCanonicalName().equals(eu.convergence.middleware.relationships.schema.convRelation.Relationship.class.getCanonicalName())){
//						eu.convergence.middleware.relationships.schema.convRelation.Relationship o = (eu.convergence.middleware.relationships.schema.convRelation.Relationship) ((JAXBElement<?>)ot.getContent()).getValue();
//						relationships.add(new Relationship(o));
//					}
//				}
//			}

			this.resourceUrl2mimeType = new HashMap<String, String>();

			for (Item innerItem : rvdiDIDL.getMPEGMItem().getItem())
			{
				for (Component component : innerItem.getComponent())
				{
					for (Resource resource : component.getResource())
					{
						String mimeType = resource.getMime();
						String resRef = resource.getRef();
		
						this.resourceUrl2mimeType.put(resRef, mimeType);
					}
				}
			}
		} 
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
	
	private void parsePVDI(Object pvdi)
	{
		try
		{
			if (pvdi instanceof String)
			{
				String filePath = orchestrator.getConetOrchestrator().fetchResourceFromConet((String)pvdi);
				
				pvdi = new File(filePath);
			}
			
			DIDL pvdiDIDL = vdiEngine.getVDIEngineSchemaHandler().getDIDL();
			pvdiDIDL.parse(new MXMAdapter(pvdi));

			this.identifier = pvdiDIDL.getMPEGMItem().getIdentifier();

			System.out.println("Parsing P-VDI: " + identifier);
			
			MetadataParser metadataParser = convMetadataEngine.getConvergenceMetadataTechnologyHandler()
					.getMetadataParser(pvdiDIDL.getMPEGMItem().getMetadata());
			
			if (metadataParser instanceof ConvergenceMetadataParser)
			{
				ConvergenceMetadata convergenceMetadata = ((ConvergenceMetadataParser) metadataParser).getConvergenceMetadata();
				
				PublicationMetadata pubMetadata = convergenceMetadata.getPublicationMetadata();
				
				this.rvdiId = pubMetadata.getRVDIIdentifier();
				
				this.fractal = pubMetadata.getDimensionList().get(0).getFractalList().get(0);
				
				this.startDate = convergenceMetadata.getStartDate();
				this.expiryDate = convergenceMetadata.getExpiryDate();
			}
			
			if (pvdiDIDL.getMPEGMItem().getERR()!=null)
			{
				errs.addAll(pvdiDIDL.getMPEGMItem().getERR());
			}
			
			for (DIDLInfo didlInfo : pvdiDIDL.getMPEGMDIDLInfo())
			{
				this.signatures.add(didlInfo.getSignature());
			}

			IPMPGeneralInfoDescriptor licenseDescriptor = (IPMPGeneralInfoDescriptor) pvdiDIDL.getMPEGMItem().getIPMPGeneralInfoDescriptor();

			if (licenseDescriptor!=null)
			{
				for (RightsDescriptor rightsDescriptor : licenseDescriptor.getLicenseCollection().getRightsDescriptor())
				{
					this.licenses.addAll(rightsDescriptor.getLicense().getLicense());
				}
			}
		} 
		catch (Exception e)
		{
			e.printStackTrace();
		}
	}
	
	private void parseSVDI(Object svdi) // TODO reenable
	{
//		try
//		{
//			VDITE vdiEngine = (VDITE) MXM.getInstance().getDefaultEngine(MXMEngineName.DigitalItemTE.value());
//			GenericMetadataTE genericMetadataEngine = (GenericMetadataTE) MXM.getInstance().getDefaultEngine(ConvergenceEngineName.GenericMetadataTE.value());
//			MpqfTE mpqfEngine = (MpqfTE) MXM.getInstance().getDefaultEngine(ConvergenceEngineName.MpqfTE.value());
//			
//			if (svdi instanceof String)
//			{
//				String filePath = OrchestratorTE.dealWithDIDEngine().fetchAndSaveVDI((String) svdi);
//				
//				svdi = new File(filePath);
//			}
//			
//			DIParser diParser = vdiEngine.getDIParser();
//			diParser.parseDI((File)svdi);
//
//			this.identifier =  diParser.getContentID();
//
//			System.out.println("Parsing S-VDI: " + identifier);
//			
//			MetadataParserInterface genericMetadataParser = genericMetadataEngine.getMetadataParser(diParser.getContentMetadata());
//			
//			if (genericMetadataParser instanceof ConvergenceMetadataParser)
//			{
//				ConvergenceMetadata convergenceMetadata = ((ConvergenceMetadataParser) genericMetadataParser).getConvergenceMetadata();
//				
//				SubscriptionMetadata subMetadata = convergenceMetadata.getSubscriptionMetadata();
//				
////				InputQuery inputQuery = convergenceMetadata.getSubscriptionMetadata().getQuery();
////				InputQueryEditor inputQueryEditor = mpqfEngine.getInputQueryEditor(new MXMAdapter(inputQuery));
////				
////				if (inputQueryEditor instanceof FieldValueQueryEditor)
////				{
////					FieldValueQueryEditor subscriptionQuery = (FieldValueQueryEditor) inputQueryEditor;
////					
////					List<FieldOperatorValueWrapper> fieldOperatorValue = subscriptionQuery.getFieldOperatorValueList();
////
////					for(FieldOperatorValueWrapper fov : fieldOperatorValue)
////					{
////						
////						if (fov.getField().equals("TAG"))
////						{
//////							cdsEngine.getSemanticExpander().expandKeywordQueryByURI(uris)
////						}
////						else
////						{
////							if ("<".equals(fov.getOperator()))
////							{
////								fieldValueQuery.getLessThanMap().put(fov.getField(), fov.getValue());
////							}
////							else if (">".equals(fov.getOperator()))
////							{
////								fieldValueQuery.getGreaterThanMap().put(fov.getField(), fov.getValue());
////							}
////							else if ("=".equals(fov.getOperator()))
////							{
////								if (fieldValueEqualMap.get(fov.getField())==null)
////								{
////									fieldValueEqualMap.put(fov.getField(), new ArrayList<Object>());
////								}
////								
////								fieldValueEqualMap.get(fov.getField()).add(fov.getValue());
////							}
////						}
////					}
////				}
//				
//				this.fractal = subMetadata.getDimensions().get(0).getFractals().get(0);
//				
//				this.startDate = convergenceMetadata.getStartDate();
//				this.expiryDate = convergenceMetadata.getExpiryDate();
//				
//				this.errs = new ArrayList<ERR>();
//				
//				if (diParser.getERR()!=null && diParser.getERR().getContent()!=null)
//				{
//					errs.add((ERR) diParser.getERR().getContent());
//				}	
//				
//				this.signatures = new ArrayList<Signature>();
//				
//				MXMObject signatureObject = diParser.getContentElementSignature(identifier);
//				
//				if (signatureObject.getContent()!=null)
//				{
//					this.signatures.add((Signature) signatureObject);
//				}
//				
//				this.licenses = new ArrayList<License>();
//				
//				IPMPGeneralInfoDescriptor licenseDescriptor = (IPMPGeneralInfoDescriptor) diParser.getIpmpGeneralInfoDescriptor().getContent();
//				
//				if (licenseDescriptor!=null)
//				{
//					for (RightsDescriptor rightsDescriptor : licenseDescriptor.getRightsDescriptorParsers())
//					{
//						this.licenses.addAll(rightsDescriptor.getLicense().getLicenses());
//					}
//				}
//			}
//		} 
//		catch (Exception e)
//		{
//			e.printStackTrace();
//		}
	}
	
	public void addLicense(License license)
	{
		this.licenses.add(license);
	}

	public void setStartDate(Date startDate)
	{
		this.startDate = startDate;
	}

	public void setExpiryDate(Date expiryDate)
	{
		this.expiryDate = expiryDate;
	}

	public void addSignature(Signature signature) 
	{
		this.signatures.add(signature);
	}

	public String getIdentifier() 
	{
		return this.identifier;
	}

	public List<License> getLicenseList() 
	{
		return this.licenses;
	}
	
	public Map<String,String> getResourceMap()
	{
		return this.resourceUrl2mimeType;
	}

	public Date getStartDate() 
	{
		return this.startDate;
	}

	public Date getExpiryDate() 
	{
		return this.expiryDate;
	}

	public List<Signature> getSignatureList()
	{
		return this.signatures;
	}

	public void addFieldValueCondition(String field, String operator, Object value)
	{	
		if ("=".equals(operator))
		{
			if (fieldValueEqualMap.get(field)==null)
			{
				fieldValueEqualMap.put(field, new ArrayList<Object>());
			}
			
			fieldValueEqualMap.get(field).add(value);
		}
		else if (">".equals(operator))
		{
			fieldValueGreaterThanMap.put(field, value);
		}
		else if ("<".equals(operator))
		{
			fieldValueLessThanMap.put(field, value);
		}
	}

	public void addKeywordCondition(List<String> keywords)
	{
		for (String keyword : keywords)
		{
			addFieldValueCondition("KEYWORD", "=", keyword);
		}
	}
	
	public void addTagCondition(List<String> tags)
	{
		for (String tag : tags)
		{
			addFieldValueCondition("TAG", "=", tag);
		}
	}
	
	@Override
	public void addSparqlTripleCondition(String subject, String predicate,
			String object)
	{
		sparqlQuery.addTripleCondition(subject, predicate, object);
	}

	@Override
	public void addSparqlTripleLiteralCondition(String subject, String predicate,
			Object object)
	{
		sparqlQuery.addTripleLiteralCondition(subject, predicate, object);
	}

	@Override
	public void addSparqlFilterCondition(String object, String operator, Object value)
	{
		sparqlQuery.addFilterCondition(object, operator, value);
	}

	@Override
	public void addSparqlReturnVariable(String varName)
	{
		sparqlQuery.addReturnVariable(varName);
	}

	@Override
	public void setResultLimit(long limit)
	{
		sparqlQuery.setResultLimit(limit);
	}

	public DIDL generateSVDI()
	{
		try
		{
			// create metadata
			StructuredData metadata = orchestrator.getMetadataOrchestrator().
					generateSubscriptionVdiMetadata(startDate, expiryDate,fractal,
							fieldValueEqualMap, fieldValueGreaterThanMap, fieldValueLessThanMap,sparqlQuery);
			
			if (metadata!=null)
			{
				addAsset(null, null, errs); // TODO this is wrong, but temporarily lets do it this way
				
				// create VDI
				didl = orchestrator.getVdiOrchestrator().createVdiLocally(assets, metadata);
				
				if (didl!=null)
				{
					// identify VDI
					didl = orchestrator.getVdiOrchestrator().identifyVdiLocally(didl);
				}
				
				identifier = didl.getMPEGMItem().getIdentifier();
				
				return didl;
			}
			else
			{
				System.out.println("Error creating Svdi metadata.");
			}
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}

	public String subscribe()
	{
		//////////STORE SVDI///////////////
		String vdiFileName = orchestrator.getVdiOrchestrator().storeVDItoTemp(didl);
		
		if (vdiFileName == null) 
		{
			System.out.println("Saving locally of SVDI failed");
			return null;
		}
		
		//////////ADVERTISE PVDI////////////
		if (!orchestrator.getVdiOrchestrator().storeAndAdvertiseVDI(vdiFileName))
		{
			System.out.println("Failure in storeAndAdvertise");
			return null;
		}
		
		//////////INJECT PVDI///////////////
		try 
		{
			orchestrator.getOverlayOrchestrator().handleSubscriptionVDI(didl, expiryDate, fractal);
		} 
		catch (OverlayEngineException e) 
		{
			System.out.println("Failure in Injecting PVDI in the semantic overlay under fractal " + fractal);
			return null;
		}
		
		/////////RETURN IDENTIFIER///////////
		try 
		{
			return didl.getMPEGMItem().getIdentifier();
		}
		catch(Exception e)
		{
			System.out.println("Failure in retreiving svdi identifier");
		}
		
		return null;
	}

	public String getFractal()
	{
		return this.fractal;
	}

	public List<String> getKeywordList()
	{
		return this.keywords;
	}

	public List<String> getTagList()
	{
		return this.tags;
	}

	public void setFractal(String fractal) 
	{
		this.fractal = fractal;
	}

	public void addERR(ERR err)
	{
		this.errs.add(err);
	}

	public DIDL generatePVDI()
	{
		try
		{
			String filePath = orchestrator.getConetOrchestrator().fetchResourceFromConet(rvdiId);

			if (filePath!=null)
			{
				DIDL rvdi = vdiEngine.getVDIEngineSchemaHandler().getDIDL();
				rvdi.parse(new MXMAdapter(new File(filePath)));

				StructuredData rvdiMetadata = rvdi.getMPEGMItem().getMetadata();

				// extract rvdi metadata
				orchestrator.getMetadataOrchestrator().extractConvergenceMatadata(rvdiMetadata, keywords, tags, fieldValueEqualMap, structuredMetadata);
			
				// create pvdi metadata
				StructuredData metadata = orchestrator.getMetadataOrchestrator().
						generatePublicationVdiMetadata(startDate, expiryDate,fractal, rvdiId, 
										keywords, tags, fieldValueEqualMap, structuredMetadata);
				
				if (metadata!=null)
				{
					// create VDI
					didl = orchestrator.getVdiOrchestrator().createVdiLocally(assets, metadata);
					
					if (didl!=null)
					{
						// identify VDI
						didl = orchestrator.getVdiOrchestrator().identifyVdiLocally(didl);
					}
					
					identifier = didl.getMPEGMItem().getIdentifier();
					
					return didl;
				}
			}
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}

	public boolean publish()
	{
		//////////STORE PVDI///////////////
		String vdiFileName = orchestrator.getVdiOrchestrator().storeVDItoTemp(didl);
		
		if (vdiFileName == null) 
		{
			System.out.println("Saving locally of PVDI failed");
			return false;
		}
		
		//////////ADVERTISE PVDI////////////
		if (!orchestrator.getVdiOrchestrator().storeAndAdvertiseVDI(vdiFileName))
		{
			System.out.println("Failure in storeAndAdvertise");
			return false;
		}
		
		//////////INJECT PVDI///////////////
		try 
		{
			orchestrator.getOverlayOrchestrator().handlePublicationVDI(didl, expiryDate, fractal);
		} 
		catch (OverlayEngineException e) 
		{
			System.out.println("Failure in Injecting PVDI in the semantic overlay under fractal " + fractal);
			return false;
		}
		
		return false;
	}

	public Map<String,List<Object>> getFieldValueMap() 
	{
		return this.fieldValueEqualMap;
	}

	public List<StructuredData> getStructuredDataList() 
	{
		return this.structuredMetadata;
	}

	public Map<String, String> getRelationshipMap() // TODO revome and replace with the method below
	{
		return null;
	}
	
	public List<Relationship> getRelationshipList()
	{
		return this.relationships;
	}

	public void addKeyword(String keyword) 
	{
		this.keywords.add(keyword);
	}

	public void addTag(String tag)
	{
		this.tags.add(tag);
	}

	public void addFieldValue(String field, Object value)
	{
		if (fieldValueEqualMap.get(field)==null)
		{
			fieldValueEqualMap.put(field, new ArrayList<Object>());
		}
		
		fieldValueEqualMap.get(field).add(value);
	}

	public void addStructuredData(StructuredData structuredData)
	{
		structuredMetadata.add(structuredData);
	}

	public void addRelationship(String relationshipType, String vdiId) // TODO reenable
	{
//		Relationship rel;
//		try {
//			rel = new Relationship(baseOntologyURI, "this", relationshipType, vdiId);
//			relationships.add(rel);
//		} catch (MXMEngineException e) {
//			e.printStackTrace();
//		}
	}
	
	// TODO users should have the possibility to specify the ontology URI. We could include it either as a prefix to the relationshipType
	// or as a separate string parameter
	/*
	public void addRelationship(String ontologyURI,String relationshipType, String vdiId)
	{
		Relationship rel;
		try {
			rel = new Relationship(ontologyURI, "this", relationshipType, vdiId);
			relationships.add(rel);
		} catch (MXMEngineException e) {
			e.printStackTrace();
		}
		
	}*/

	public Resource createResource(String resourceURL, String mimtType, String content)
	{
		try
		{
			org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource resource = vdiEngine.getDIEngineSchemaHandler().getResource();
			
			resource.setMime(mimtType);
			resource.setRef(resourceURL);
			resource.getContent().add(content);
			
			return resource;
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}
	
	public void addAsset(Resource resource, List<License> licenses, List<ERR> errs)
	{
		try
		{
			Asset asset = createContentEngine.getCreateContentSchemaHandler().getAsset();
			
			if (licenses!=null && !licenses.isEmpty())
			{
				org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.License ipmpLicense = ipmpEngine.getIPMPEngineSchemaHandler().getIPMPLicense();
				
				for (License license : licenses)
				{
					ipmpLicense.addLicense(license);
				}
				
				RightsDescriptor rd = ipmpEngine.getIPMPEngineSchemaHandler().getRightsDescriptor();
				rd.setLicense(ipmpLicense);
				
				LicenseCollection lc = ipmpEngine.getIPMPEngineSchemaHandler().getLicenseCollection();
				lc.setRightsDescriptor(rd);
				
				IPMPGeneralInfoDescriptor ipmpLicenseContainer = ipmpEngine.getIPMPEngineSchemaHandler().getIPMPGeneralInfoDescriptor();
				ipmpLicenseContainer.setLicenseCollection(lc);
				
				asset.setIPMPGeneralInfoDescriptor(ipmpLicenseContainer);
			}
			
			if (resource!=null)
			{
				asset.setResource(resource);
			}
			
			if (errs!=null)
			{
				for (ERR err : errs)
				{
					asset.addERR(err);
				}
			}
			
			assets.add(asset);
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
	}
	
	public DIDL generateRVDI()
	{
		try
		{
			// create metadata
			StructuredData metadata = orchestrator.getMetadataOrchestrator().
					generateResourceVdiMetadata(startDate, expiryDate, keywords, tags, fieldValueEqualMap, structuredMetadata);
			
			if (metadata!=null)
			{
				// create VDI
				didl = orchestrator.getVdiOrchestrator().createVdiLocally(assets, metadata);
				
				if (didl!=null)
				{
					// identify VDI
					didl = orchestrator.getVdiOrchestrator().identifyVdiLocally(didl);
				}
				
				identifier = didl.getMPEGMItem().getIdentifier();
				
				return didl;
			}
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}

	public boolean store(String filePath)
	{
		try
		{
			DIDL didlWithContext = vdiEngine.getVDIEngineSchemaHandler().getDIDL();
			didlWithContext.parse(didl.createObject(JAXBElement.class.getCanonicalName()));
			
			didlWithContext.createFile(new File(filePath));
			
			return true;
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return false;
	}

	public boolean advertise(String filePath)
	{
		return orchestrator.getVdiOrchestrator().storeAndAdvertiseVDI(filePath);
	}
	
}
