/*
 * Copyright (c) 2011 Safran Morpho.
 * This software is the confidential and proprietary
 * information of Safran Morpho.
 * All rights reserved.
 *
 * Safran Morpho makes no representations or
 * warranties about the suitability of the software, either
 * express or implied, including but not limited to the implied
 * warranties of merchantability, fitness for a particular
 * purpose, or non-infringement.
 * Safran Morpho shall not be liable for any damages
 * suffered by licensee as a result of using, modifying or
 * distributing this software or its derivatives.
 *
 * This copyright notice must appear in all copies of this
 * software.
 *
 * $File: //ResearchProjects/Convergence/implementation/main/SecurityLib/src/morpho/cryptomodule/convergence/CommandDispatcher.java $
 * 
 * Last submit:
 *  $Revision: #6 $
 *  $Change: 367430 $
 *  $DateTime: 2012/06/12 16:02:18 $
 *  $Author: akohlos $
 *
 * 
 * Created on 25.10.2011
 *
 */
package morpho.cryptomodule.convergence;


import java.util.List;

import javax.smartcardio.Card;
import javax.smartcardio.CardChannel;
import javax.smartcardio.CardException;
import javax.smartcardio.CardTerminal;
import javax.smartcardio.CommandAPDU;
import javax.smartcardio.ResponseAPDU;
import javax.smartcardio.TerminalFactory;

import morpho.cryptomodule.CryptoModule;
import morpho.cryptomodule.CryptoModuleException;
import morpho.util.Converter;


public class CommandDispatcher implements CryptoModule
{
    private enum Keytype
    {
        auth, conf, sig
    }

    // TODO Ensure, that only 1,2,3 or 4 are used for every SELECT_DF command.
    // TODO support any String app aliases.
    protected final static String CRYPTO_MODULE_VERSION = "0.2";
    private Card                  card                  = null;
    private boolean               cardIsAvailable       = false;
    private String                cardProtocol          = "";
    private String                cardTerminalName      = "";
    private CardChannel           channel               = null;
    private final boolean         verboseAPDUs          = false;

    /**
     * @throws CardException
     */
    public CommandDispatcher(final String cardProtocol, final String cardTerminalName)
        throws CryptoModuleException
    {
        // try
        // {
        // properties = new Properties();
        // final BufferedInputStream bis = new BufferedInputStream(new FileInputStream(
        // "./config/SmartCard.properties"));
        // properties.load(bis);
        // bis.close();
        // this.useSmartCard = Boolean.parseBoolean(properties
        // .getProperty("enableSmartCard"));
        // // this.verboseAPDUs = Boolean.parseBoolean(properties
        // // .getProperty("verboseAPDUs"));
        // if (this.useSmartCard)
        // {
        // this.cardProtocol = "T=" + properties.getProperty("cardProtocol");
        // this.cardTerminalName = properties.getProperty("cardTerminalName");
        // this.establishCardConnection(false);
        // this.cardIsAvailable = true;
        // }
        // else
        // {
        // this.cardIsAvailable = false;
        // }
        // }
        // catch (final IOException e)
        // {
        // throw new CardException("IOException: " + e.getMessage());
        // }
        this.cardProtocol = "T=" + cardProtocol;
        this.cardTerminalName = cardTerminalName;
        this.establishCardConnection(false);
        this.cardIsAvailable = true;
    }

    /**
     * The challenge must be exactly 32 bytes long.
     * 
     * @param intChallenge
     * @param appAlias
     * @param PIN
     * @return
     * @throws CardException
     * @throws NoCardInUseException
     */
    public byte[] clientAuth(final byte[] intChallenge, final String appAlias,
        final String PIN) throws CryptoModuleException
    {
        // Check the length of the challenge.
        if (intChallenge.length > 32)
        {
            throw new CryptoModuleException(
                "The challenge for internal authentication contains "
                    + intChallenge.length
                    + " bytes which are not supported by the card. The card only supports challenges not greater than 32 bytes.");
        }
        if ((Integer.parseInt(appAlias) > Settings.NUMBER_OF_APPLICATIONS_PER_CARD)
            || (Integer.parseInt(appAlias) < 1))
        {
            throw new CryptoModuleException(
                "The card only accepts application aliases from 1 to "
                    + Settings.NUMBER_OF_APPLICATIONS_PER_CARD + ".");
        }
        // Select DF.
        this.sendCommand(CommandApduHeaders.SELECT_DF + appAlias, true);
        // Verify PIN.
        this.sendCommand(CommandApduHeaders.VERIFY_PIN_AUTH + PIN + "FFFFFFFF", false);
        // Set MSE.
        this.sendCommand(CommandApduHeaders.MSE_SET_AUTH, false);
        // Internal Authentication. Currently, intChallenge must be exactly 32 bytes long.
        final byte[] response = this.sendCommand(CommandApduHeaders.INTERNAL_AUTH
            + Converter.toHex(intChallenge) + "00", false);
        byte[] signedToken = new byte[256];
        // Strip the signed token from the card response.
        System.arraycopy(response, 0, signedToken, 0, 256);
        return signedToken;
    }

    private boolean establishCardConnection(final boolean resetConnectionBeforeEstablish)
        throws CryptoModuleException
    {
        boolean result = false;
        try
        {
            if (resetConnectionBeforeEstablish && (this.card != null))
            {
                this.card.disconnect(true);
                this.card = null;
            }
            if (this.card == null)
            {
                // Get the list of available card terminals connected to the PC.
                final TerminalFactory factory = TerminalFactory.getDefault();
                if (factory.terminals().list().isEmpty())
                {
                    throw new CryptoModuleException(
                        "No Card reader present! Please connect one to your computer.");
                }
                final List<CardTerminal> terminals = factory.terminals().list();
                // Get the right terminal from the list of available terminals.
                int ourTerminal = -1;
                for (int i = 0; i < terminals.size(); i++)
                {
                    if (terminals.get(i).getName().contains(this.cardTerminalName))
                    {
                        ourTerminal = i;
                    }
                }
                if (ourTerminal == -1)
                {
                    throw new CryptoModuleException("Couldn't find the "
                        + this.cardTerminalName
                        + " card reader! Check the connection to your computer.");
                }
                final CardTerminal terminal = terminals.get(ourTerminal);
                // Check for the presence of a card.
                if (!terminal.isCardPresent())
                {
                    throw new CryptoModuleException("No Card present! Please insert one.");
                }
                // Establish a connection to the card.
                try
                {
                    // Thread.sleep(250);
                    this.card = terminal.connect(this.cardProtocol);
                    this.channel = this.card.getBasicChannel();
                    result = true;
                }
                catch (final CardException e)
                {
                    this.card.disconnect(true);
                    this.card = null;
                    throw new CryptoModuleException(
                        "Couldn't connect to the card. The connection attempt threw the following error: "
                            + e.getMessage());
                }
                // catch (final InterruptedException e)
                // {
                // this.card.disconnect(true);
                // this.card = null;
                // throw new CardException(e.getMessage());
                // }
            }
        }
        catch (final CardException e)
        {
            throw new CryptoModuleException(e.getMessage());
        }
        return result;
    }

    /**
     * @param keyIdentifier Should be one of 'sig', 'auth' or 'conf', specifying the key
     *            to be used either for signing, authentication or confidentiality.
     * @param appAlias
     * @param keyLength
     * @param PubExp
     * @param PIN
     * @return
     * @throws CardException
     * @throws NoCardInUseException
     */
    public byte[] generateRSAKeyPair(final String keyIdentifier, final String appAlias,
        final int keyLength, final String PIN) throws CryptoModuleException
    {
        Keytype keytype;
        if (keyIdentifier.equals("sig"))
        {
            keytype = Keytype.sig;
        }
        else if (keyIdentifier.equals("conf"))
        {
            keytype = Keytype.conf;
        }
        else if (keyIdentifier.equals("auth"))
        {
            keytype = Keytype.auth;
        }
        else
        {
            throw new CryptoModuleException("The keyIdentifier '" + keyIdentifier
                + "' is not known to the card.");
        }
        if ((Integer.parseInt(appAlias) > Settings.NUMBER_OF_APPLICATIONS_PER_CARD)
            || (Integer.parseInt(appAlias) < 1))
        {
            throw new CryptoModuleException(
                "The card only accepts application aliases from 1 to "
                    + Settings.NUMBER_OF_APPLICATIONS_PER_CARD + ".");
        }
        // Select DF.
        this.sendCommand(CommandApduHeaders.SELECT_DF + appAlias, true);
        // Verify pin.
        this.sendCommand(CommandApduHeaders.VERIFY_PIN_GAKP + PIN + "FFFFFF", false);
        // Select the object KID.
        switch (keytype)
        {
            case sig:
                this.sendCommand(CommandApduHeaders.SELECT_OBJ_KID_SIG, false);
                break;
            case conf:
                this.sendCommand(CommandApduHeaders.SELECT_OBJ_KID_CONF, false);
                break;
            case auth:
                this.sendCommand(CommandApduHeaders.SELECT_OBJ_KID_AUTH, false);
                break;
        }
        // Set attributes for the key generation.
        this.sendCommand(CommandApduHeaders.SET_ATTR_P, false);
        this.sendCommand(CommandApduHeaders.SET_ATTR_Q, false);
        this.sendCommand(CommandApduHeaders.SET_ATTR_F_4, false);
        // Set MFE.
        switch (keytype)
        {
            case sig:
                this.sendCommand(CommandApduHeaders.MSE_SET_SIG, false);
                break;
            case conf:
                this.sendCommand(CommandApduHeaders.MSE_SET_CONF, false);
                break;
            case auth:
                this.sendCommand(CommandApduHeaders.MSE_SET_AUTH, false);
                break;
        }
        // Get the public key from card.
        byte[] response = this.sendCommand(CommandApduHeaders.GAKP, false);
        // Strip the modulus from the response.
        byte[] modulus = new byte[256];
        System.arraycopy(response, 9, modulus, 0, 256);
        return modulus;
    }

    // // TODO check, which things are needed here
    // // TODO fill in java doc here
    // /**
    // * @param cmdHeader
    // * @param lcBytes
    // * @param dataBytes
    // * @param leBytes
    // * @return
    // */
    // private static byte[] buildCmdApdu(final byte[] cmdHeader, final byte[] lcBytes,
    // final byte[] dataBytes, final byte[] leBytes)
    // {
    // final int cmdHeaderLength = cmdHeader.length;
    // final int lcBytesLength = lcBytes.length;
    // final int dataBytesLength = dataBytes.length;
    // final int leBytesLength = leBytes.length;
    // final byte[] cmd = new byte[cmdHeaderLength + lcBytesLength + dataBytesLength
    // + leBytesLength];
    // System.arraycopy((byte[])cmdHeader, 0, (byte[])cmd, 0, cmdHeaderLength);
    // System.arraycopy((byte[])lcBytes, 0, (byte[])cmd, cmdHeaderLength, lcBytesLength);
    // System.arraycopy((byte[])dataBytes, 0, (byte[])cmd,
    // (cmdHeaderLength + lcBytesLength), dataBytesLength);
    // System.arraycopy((byte[])leBytes, 0, (byte[])cmd, (cmdHeaderLength
    // + lcBytesLength + dataBytesLength), leBytesLength);
    // return cmd;
    // }
    public byte[] generateSignature(final byte[] hashValue, final String appAlias,
        final String PIN) throws CryptoModuleException
    {
        try
        {
            if ((Integer.parseInt(appAlias) > Settings.NUMBER_OF_APPLICATIONS_PER_CARD)
                || (Integer.parseInt(appAlias) < 1))
            {
                throw new CryptoModuleException(
                    "The card only accepts application aliases from 1 to "
                        + Settings.NUMBER_OF_APPLICATIONS_PER_CARD + ".");
            }
            // Select DF.
            this.sendCommand(CommandApduHeaders.SELECT_DF + appAlias, true);
            // Verify PIN.
            this.sendCommand(CommandApduHeaders.VERIFY_PIN_SIG + PIN + "FFFFFFFF", false);
            // Set MSE.
            this.sendCommand(CommandApduHeaders.MSE_SET_SIG, false);
            // PSO CDS hash value.
            return this.sendCommand(
                CommandApduHeaders.PSO_CDS_HASHVALUE + Converter.toHex(hashValue)
                + "00", false);
        }
        catch (final CryptoModuleException e)
        {
            throw new CryptoModuleException(e.getMessage());
        }
    }

    public byte[] getPublicKey(final String keyIdentifier, final String appAlias,
        final String PIN) throws CryptoModuleException
    {
        Keytype keytype;
        if (keyIdentifier.equals("sig"))
        {
            keytype = Keytype.sig;
        }
        else if (keyIdentifier.equals("conf"))
        {
            keytype = Keytype.conf;
        }
        else if (keyIdentifier.equals("auth"))
        {
            keytype = Keytype.auth;
        }
        else
        {
            throw new CryptoModuleException("The keyIdentifier '" + keyIdentifier
                + "' is not known to the card.");
        }
        if ((Integer.parseInt(appAlias) > Settings.NUMBER_OF_APPLICATIONS_PER_CARD)
            || (Integer.parseInt(appAlias) < 1))
        {
            throw new CryptoModuleException(
                "The card only accepts application aliases from 1 to "
                    + Settings.NUMBER_OF_APPLICATIONS_PER_CARD + ".");
        }
        // Select DF.
        this.sendCommand(CommandApduHeaders.SELECT_DF + appAlias, true);
        // Verify PIN.
        this.sendCommand(CommandApduHeaders.VERIFY_PIN_GAKP + PIN + "FFFFFF", false);
        // Set MFE.
        switch (keytype)
        {
            case sig:
                this.sendCommand(CommandApduHeaders.MSE_SET_SIG, false);
                break;
            case conf:
                this.sendCommand(CommandApduHeaders.MSE_SET_CONF, false);
                break;
            case auth:
                this.sendCommand(CommandApduHeaders.MSE_SET_AUTH, false);
                break;
        }

        // Get the public key from card.
        final byte[] response = this.sendCommand(CommandApduHeaders.GAKP_READ_PUBLIC,
            false);
        // Strip the modulus from the response.
        byte[] modulus = new byte[256];
        System.arraycopy(response, 9, modulus, 0, 256);
        return modulus;
    }

    public boolean initStreamUnwrap(final String algorithm, final String appAlias,
        final String PIN) throws CryptoModuleException
    {
        boolean result = false;
        if ((Integer.parseInt(appAlias) > Settings.NUMBER_OF_APPLICATIONS_PER_CARD)
            || (Integer.parseInt(appAlias) < 1))
        {
            throw new CryptoModuleException(
                "The card only accepts application aliases from 1 to "
                    + Settings.NUMBER_OF_APPLICATIONS_PER_CARD + ".");
        }
        // Select DF.
        this.sendCommand(CommandApduHeaders.SELECT_DF + appAlias, true);
        // Verify PIN.
        this.sendCommand(CommandApduHeaders.VERIFY_PIN_CONF + PIN + "FFFFFFFF", false);
        // Set MSE.
        this.sendCommand(CommandApduHeaders.MSE_SET_CONF, false);
        result = true;
        return result;
    }

    public byte[] readCertificate(final String keyIdentifier, final String appAlias)
        throws CryptoModuleException
    {
        byte[] binaryEntry = null;
        Keytype keytype;
        if (keyIdentifier.equals("sig"))
        {
            keytype = Keytype.sig;
        }
        else if (keyIdentifier.equals("conf"))
        {
            keytype = Keytype.conf;
        }
        else if (keyIdentifier.equals("auth"))
        {
            keytype = Keytype.auth;
        }
        else
        {
            throw new CryptoModuleException("The keyIdentifier '" + keyIdentifier
                + "' is not known to the card.");
        }
        if ((Integer.parseInt(appAlias) > Settings.NUMBER_OF_APPLICATIONS_PER_CARD)
            || (Integer.parseInt(appAlias) < 1))
        {
            throw new CryptoModuleException(
                "The card only accepts application aliases from 1 to "
                    + Settings.NUMBER_OF_APPLICATIONS_PER_CARD + ".");
        }
        // Select DF.
        this.sendCommand(CommandApduHeaders.SELECT_DF + appAlias, true);
        // Select the certificate type.
        switch (keytype)
        {
            case sig:
                this.sendCommand(CommandApduHeaders.SELECT_CERT_TYPE_SIG, false);
                break;
            case conf:
                this.sendCommand(CommandApduHeaders.SELECT_CERT_TYPE_CONF, false);
                break;
            case auth:
                this.sendCommand(CommandApduHeaders.SELECT_CERT_TYPE_AUTH, false);
                break;
        }
        // Read the binary entry via READ_BINARY.
        binaryEntry = this.sendCommand(CommandApduHeaders.READ_BINARY, false);
        // Get the length bytes (the first two bytes of the binaryEntry.
        byte[] strippedBytes = new byte[2];
        System.arraycopy(binaryEntry, 0, strippedBytes, 0, 2);
        final int numCertBytes = Converter.twoByteArrayToInt(strippedBytes);
        // Get the certificate bytes from the binaryEntry.
        strippedBytes = new byte[numCertBytes];
        System.arraycopy(binaryEntry, 2, strippedBytes, 0, numCertBytes + 1);
        return strippedBytes;
    }

    private byte[] sendCommand(final String commandAPDU,
        final boolean resetConnectionBeforeSend) throws CryptoModuleException
    {
        byte[] responseData = null;
        ResponseAPDU rspAPDU = null;
        try
        {
            this.establishCardConnection(resetConnectionBeforeSend);
            if (!this.cardIsAvailable)
            {
                throw new NoCardInUseException(
                    "The card is not available. Please check your settings.");
            }
            final CommandAPDU cmdAPDU = new CommandAPDU(
                Converter.toByteArray(commandAPDU));
            if (this.verboseAPDUs)
            {
                System.out.println("--------------------------------");
                System.out.println("-> " + commandAPDU);
            }
            rspAPDU = this.channel.transmit(cmdAPDU);
            final byte[] rspBytes = rspAPDU.getBytes();
            int rspLength = rspBytes.length;
            final String statusBytes = ResponseValidator.checkStatusBytes(rspLength,
                rspBytes);
            if (rspLength > 2)
            {
                rspLength -= 2;
                final byte[] strippedData = new byte[rspLength];
                System.arraycopy(rspBytes, 0, strippedData, 0, rspLength);
                if (this.verboseAPDUs)
                {
                    System.out.println("<- Data: " + Converter.toHex(strippedData));
                    System.out.println("<- Status: " + statusBytes);
                    System.out.println("--------------------------------");
                }
                responseData = strippedData;
            }
            else
            {
                if (this.verboseAPDUs)
                {
                    System.out.println("<- Status: " + statusBytes);
                    System.out.println("--------------------------------");
                }
            }
        }
        catch (final CardException e)
        {
            throw new CryptoModuleException(e.getMessage());
        }
        return responseData;
    }

    // private String sendCommand(final String commandString, final boolean
    // resetBeforeSend)
    // throws CardException,NoCardInUseException
    // {
    // this.establishCardConnection(resetBeforeSend);
    // if (!this.cardIsAvailable)
    // {
    // throw new NoCardInUseException(
    // "The card is not available. Please check your settings.");
    // }
    // final CommandAPDU cmdAPDU = new CommandAPDU(Converter.toByteArray(commandString));
    // if (this.verboseAPDUs)
    // {
    // System.out.println("--------------------------------");
    // System.out.println("-> " + commandString);
    // }
    // final ResponseAPDU rspAPDU = this.channel.transmit(cmdAPDU);
    // final byte[] rspBytes = rspAPDU.getBytes();
    // int rspLength = rspBytes.length;
    // final String statusBytes = ResponseValidator
    // .checkStatusBytes(rspLength, rspBytes);
    // if (rspLength > 2)
    // {
    // rspLength -= 2;
    // final byte[] strippedData = new byte[rspLength];
    // System.arraycopy(rspBytes, 0, strippedData, 0, rspLength);
    // if (this.verboseAPDUs)
    // {
    // System.out.println("<- Data: " + Converter.toHex(strippedData));
    // System.out.println("<- Status: " + statusBytes);
    // System.out.println("--------------------------------");
    // }
    // return Converter.toHex(strippedData);
    // }
    // else
    // {
    // if (this.verboseAPDUs)
    // {
    // System.out.println("<- Status: " + statusBytes);
    // System.out.println("--------------------------------");
    // }
    // return "";
    // }
    // }
    /**
     * Does only support certificates not greater than 1920 bytes. Two length bytes are
     * used to keep hold of the size of the certificate. These are put in front of the
     * certificate bytes.
     * 
     * @param cert
     * @param keyIdentifier
     * @param appAlias
     * @param PIN
     * @throws CardException
     * @throws NoCardInUseException
     */
    public boolean storeCertificate(final byte[] certificate, final String keyIdentifier,
        final String appAlias, final String PIN) throws CryptoModuleException
    {
        boolean result = false;
        // Check the length of the certificate.
        if (certificate.length > 1920)
        {
            throw new CryptoModuleException(
                "The certificate contains "
                    + certificate.length
                    + " bytes and therefore cannot be stored to the card. The card only supports certificates not greater than 1920 bytes.");
        }
        Keytype keytype;
        if (keyIdentifier.equals("sig"))
        {
            keytype = Keytype.sig;
        }
        else if (keyIdentifier.equals("conf"))
        {
            keytype = Keytype.conf;
        }
        else if (keyIdentifier.equals("auth"))
        {
            keytype = Keytype.auth;
        }
        else
        {
            throw new CryptoModuleException("The keyIdentifier '" + keyIdentifier
                + "' is not known to the card.");
        }
        if ((Integer.parseInt(appAlias) > Settings.NUMBER_OF_APPLICATIONS_PER_CARD)
            || (Integer.parseInt(appAlias) < 1))
        {
            throw new CryptoModuleException(
                "The card only accepts application aliases from 1 to "
                    + Settings.NUMBER_OF_APPLICATIONS_PER_CARD + ".");
        }
        // Select DF.
        this.sendCommand(CommandApduHeaders.SELECT_DF + appAlias, true);
        // Select the certificate type.
        switch (keytype)
        {
            case sig:
                this.sendCommand(CommandApduHeaders.SELECT_CERT_TYPE_SIG, false);
                break;
            case conf:
                this.sendCommand(CommandApduHeaders.SELECT_CERT_TYPE_CONF, false);
                break;
            case auth:
                this.sendCommand(CommandApduHeaders.SELECT_CERT_TYPE_AUTH, false);
                break;
        }
        // Verify PIN.
        this.sendCommand(CommandApduHeaders.VERIFY_PIN_GAKP + PIN + "FFFFFFFF", false);
        // Write certificate via UPDATE_BINARY. The first two bytes represent the length
        // of the certificate.
        final String certhex = Converter.toHex(certificate);
        final int Lc = certhex.length() / 2;
        final byte[] Lcb = Converter.intToTwoBytes(Lc);
        this.sendCommand(CommandApduHeaders.UPDATE_BINARY + Converter.toHex(Lcb)
            + certhex, false);
        result = true;
        return result;
    }

    public byte[] streamUnwrapAfterInit(final byte[] cryptogram)
        throws CryptoModuleException
    {
        // PSO Decipher:
        // Currently the cryptogram must be exactly 256 bytes long!
        return this
            .sendCommand(CommandApduHeaders.PSO_DECIPHER + Converter.toHex(cryptogram)
                + "0000", false);
    }

    public byte[] unwrapKeyOnce(final byte[] cryptogram, final String algorithm,
        final String appAlias, final String PIN) throws CryptoModuleException
    {
        if ((Integer.parseInt(appAlias) > Settings.NUMBER_OF_APPLICATIONS_PER_CARD)
            || (Integer.parseInt(appAlias) < 1))
        {
            throw new CryptoModuleException(
                "The card only accepts application aliases from 1 to "
                    + Settings.NUMBER_OF_APPLICATIONS_PER_CARD + ".");
        }
        // Select DF.
        this.sendCommand(CommandApduHeaders.SELECT_DF + appAlias, true);
        // Verify PIN.
        this.sendCommand(CommandApduHeaders.VERIFY_PIN_CONF + PIN + "FFFFFFFF", false);
        // Set MSE.
        this.sendCommand(CommandApduHeaders.MSE_SET_CONF, false);
        // PSO Decipher:
        // Currently the cryptogram must be exactly 256 bytes long!
        return this
            .sendCommand(CommandApduHeaders.PSO_DECIPHER + Converter.toHex(cryptogram)
                + "0000", false);
    }
}
