/*
 * Copyright (c) 2012 Safran Morpho.
 * This software is the confidential and proprietary
 * information of Safran Morpho.
 * All rights reserved.
 *
 * Safran Morpho makes no representations or
 * warranties about the suitability of the software, either
 * express or implied, including but not limited to the implied
 * warranties of merchantability, fitness for a particular
 * purpose, or non-infringement.
 * Safran Morpho shall not be liable for any damages
 * suffered by licensee as a result of using, modifying or
 * distributing this software or its derivatives.
 *
 * This copyright notice must appear in all copies of this
 * software.
 *
 * $File: //ResearchProjects/Convergence/implementation/main/SecurityLib/src/morpho/securityengine/SignatureManager.java $
 * 
 * Last submit:
 *  $Revision: #2 $
 *  $Change: 365799 $
 *  $DateTime: 2012/06/01 14:17:13 $
 *  $Author: akohlos $
 *
 * 
 * Created on 31.05.2012
 *
 */
package morpho.securityengine;


import java.io.FileInputStream;
import java.io.IOException;
import java.security.InvalidAlgorithmParameterException;
import java.security.InvalidKeyException;
import java.security.MessageDigest;
import java.security.NoSuchAlgorithmException;
import java.security.NoSuchProviderException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.Signature;
import java.security.SignatureException;
import java.security.cert.X509Certificate;
import java.security.spec.AlgorithmParameterSpec;
import java.security.spec.MGF1ParameterSpec;
import java.security.spec.PSSParameterSpec;

import morpho.cryptomodule.convergence.CommandDispatcher;
import morpho.cryptomodule.convergence.NoCardInUseException;
import morpho.securityengine.exceptions.SecurityEngineException;


public class SignatureManager
{
    private final CommandDispatcher cryptoModuleManager;

    public SignatureManager(final CommandDispatcher cryptoModuleManager)
    {
        this.cryptoModuleManager = cryptoModuleManager;
    }

    public byte[] generateCardSignature(final byte[] message, final String algorithm,
        final String appAlias, final String PIN) throws SecurityEngineException
    {
        byte[] signature = null;
        try
        {
            // Generate a hash of the input message - then sign it.
            signature = this.cryptoModuleManager.generateSignature(
                this.generateHash(message, "SHA-256"), appAlias, PIN);
        }
        catch (final NoCardInUseException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return signature;
    }

    public byte[] generateHash(final byte[] data, final String algorithm)
        throws SecurityEngineException
    {
        byte[] resultHash = null;
        MessageDigest md;
        try
        {
            md = MessageDigest.getInstance(algorithm, "BC");
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        md.update(data);
        resultHash = md.digest();
        return resultHash;
    }

    public byte[] generateHash(final FileInputStream fis, final String algorithm)
        throws SecurityEngineException
    {
        byte[] resultHash = null;
        MessageDigest md;
        final byte streamContent[] = null;
        try
        {
            md = MessageDigest.getInstance(algorithm, "BC");
            fis.read(streamContent);
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final IOException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        md.update(streamContent);
        resultHash = md.digest();
        return resultHash;
    }

    public byte[] generateSignature(final byte[] message, final String algorithm,
        final PrivateKey privKey, final String password) throws SecurityEngineException
    {
        byte[] result = null;
        Signature sign = null;
        final String mdName = "SHA-256";
        final String mgfName = "MGF1";
        final AlgorithmParameterSpec mgfSpec = MGF1ParameterSpec.SHA256;
        final int saltLen = 32;
        final int trailerField = 1;
        try
        {
            final PSSParameterSpec paramsSpec = new PSSParameterSpec(mdName, mgfName,
                mgfSpec, saltLen, trailerField);
            sign = Signature.getInstance(algorithm, "BC");
            sign.setParameter(paramsSpec);
            sign.initSign(privKey);
            sign.update(message);
            if (!privKey.getAlgorithm().equalsIgnoreCase("RSA"))
            {
                throw new SecurityEngineException("The provided key is not of Type RSA");
            }
            result = sign.sign();
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeyException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final SignatureException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidAlgorithmParameterException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
    }

    public boolean verifyHash(final byte[] data, final byte[] hashValue,
        final String algorithm) throws SecurityEngineException
    {
        boolean result = false;
        MessageDigest md;
        try
        {
            md = MessageDigest.getInstance(algorithm, "BC");
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        md.update(data);
        if (MessageDigest.isEqual(md.digest(), hashValue))
        {
            result = true;
        }
        return result;
    }

    public boolean verifySignature(final byte[] message, final byte[] signature,
        final String algorithm, final PublicKey pubKey) throws SecurityEngineException
    {
        boolean result = false;
        Signature sign;
        try
        {
            sign = Signature.getInstance(algorithm, "BC");
            sign.initVerify(pubKey);
            sign.update(message);
            result = sign.verify(signature);
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeyException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final SignatureException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
    }

    public boolean verifySignature(final byte[] message, final byte[] signature,
        final String algorithm, final X509Certificate x509Cert)
        throws SecurityEngineException
    {
        boolean result = false;
        Signature sign;
        try
        {
            sign = Signature.getInstance(algorithm, "BC");
            sign.initVerify(x509Cert);
            sign.update(message);
            result = sign.verify(signature);
        }
        catch (final NoSuchAlgorithmException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final InvalidKeyException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final NoSuchProviderException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        catch (final SignatureException e)
        {
            throw new SecurityEngineException(e.getMessage());
        }
        return result;
    }
}
