package DISignerVerifier;


import java.io.File;
import java.io.IOException;
import java.io.StringWriter;
import java.math.BigInteger;
import java.security.KeyFactory;
import java.security.KeyPair;
import java.security.PublicKey;
import java.security.spec.X509EncodedKeySpec;
import java.util.List;

import org.apache.log4j.Logger;
import org.apache.xerces.impl.dv.util.Base64;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.DIEngine;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.ApplicationSpecificData;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Component;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.dii.RelatedIdentifier;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.ipmpTE.IPMPEngine;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.IPMPGeneralInfoDescriptor;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.LicenseCollection;
import org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.RightsDescriptor;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RSAKeyValue;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature;
import org.w3c.dom.Document;


public class DISignerVerifier
{
    private Logger         logger              = Logger.getLogger(DISignerVerifier.class);
    private final String   TEMP_FOLDER         = System.getProperty("java.io.tmpdir")
                                                   + File.separator;
    private final String   OUTPUT_DI_FILE_NAME = TEMP_FOLDER + "mytestDCI.xml";
    private MXM            mxm;
    private SecurityEngine securityTE          = null;

    public DISignerVerifier() throws Exception
    {
        logger.info("\n\n---------- DISignerVerifier ----------");
        // Initialize MXM and Engines.
        File f = new File(getClass().getClassLoader()
            .getResource("MXMConfiguration2.xml").getFile());
        mxm = MXM.getInstance(f);
        mxm.listEnginesAndProperties();
        this.securityTE = (SecurityEngine)mxm.getDefaultEngine(MXMEngineName.SecurityTE
            .toString());
        this.securityTE.setProperty("conf", "223456");
        this.securityTE.setProperty("auth", "323456");
        this.securityTE.setProperty("sig", "123456");
        this.securityTE.setProperty("gakp", "12345678");
        this.logger.info("\n--- Create DIDL ---");
        DIDL didl = createDI();
        // create XML doc of the didl
        Document doc = (Document)didl.createObject(Document.class.getCanonicalName())
            .getContent();
        // print XML doc of the didl
        printDocument(doc);
        // sign doc of the didl
        Signature signature = signDOC(doc);
        // print signature of the didl
        logger.info(signature.createObject(String.class.getCanonicalName()).getContent());
        // add signature to the didl
        didl.setSignature(signature);
        // create new didl document (with signature now)
        doc = (Document)didl.createObject(Document.class.getCanonicalName()).getContent();
        // print new didl document (with signature)
        printDocument(doc);
        // get the signature from the didl
        signature = didl.getSignature();
        // verify the didl signature
        verifySignature(doc, signature);
    }

    /**
     * Return the signatuire of given mxm document
     * @param doc
     * @return
     * @throws Exception
     */
    public Signature signDOC(Document doc) throws Exception
    {
        this.logger
            .info("\n--- Generate a dummy key pair and store it in the SecureRepositoryManager ---");
        BigInteger rsaExponent = new BigInteger("10001", 16);
        int rsaKeyLength = 1024;
        String appAlias = "2";
        PublicKey publicKey = this.securityTE.getSecurityEngineTechnologyHandler()
            .getCertificateManager()
            .generateRSAKeyPair(appAlias, "sig", rsaKeyLength);
        this.logger.info("\n--- Key pair generated & stored---");
        this.logger.info("\n--- Generate the signature ---");
        return this.securityTE.getSecurityEngineTechnologyHandler().getSignatureManager()
            .generateSignature(doc, appAlias);
    }

    /**
     * Verifies the given signature based on provided document
     * @param doc
     * @param signature
     * @throws Exception
     */
    public boolean verifySignature(Document doc, Signature signature) throws Exception
    {
        this.logger.info("\n--- Verify the signature ---");
        boolean flag = this.securityTE.getSecurityEngineTechnologyHandler().getSignatureManager()
                .verifySignature(doc, signature);
        if (flag)
        {
            this.logger.info("\n--- Verification SUCCESSFUL ---");
        }
        else
        {
            this.logger.info("\n--- Verification FAILED ---");
        }
        return flag;
    }

    /**
     * C
     * @return
     * @throws Exception
     */
    private DIDL createDI() throws Exception
    {
        DIEngine didEngine = (DIEngine)mxm.getDefaultEngine(MXMEngineName.DigitalItemTE
            .toString());
        EventReportEngine erEngine = (EventReportEngine)mxm
            .getDefaultEngine(MXMEngineName.EventReportTE.toString());
        IPMPEngine ipmp = (IPMPEngine)mxm.getDefaultEngine(MXMEngineName.IPMPTE
            .toString());
        RELEngine relEngine = (RELEngine)mxm.getDefaultEngine(MXMEngineName.RELTE
            .toString());
        DIDL di = didEngine.getDIEngineSchemaHandler().getDIDL();
        Item toplevelitem = didEngine.getDIEngineSchemaHandler().getItem();
        ApplicationSpecificData asd = didEngine.getDIEngineSchemaHandler()
            .getApplicationSpecificData();
        ERR err = erEngine.getEventReportEngineSchemaHandler().getERR();
        Item innerItem1 = didEngine.getDIEngineSchemaHandler().getItem();
        Item innerItem2 = didEngine.getDIEngineSchemaHandler().getItem();
        toplevelitem.setId("SecTest");
        IPMPGeneralInfoDescriptor gen_desc = ipmp.getIPMPEngineSchemaHandler()
            .getIPMPGeneralInfoDescriptor();
        LicenseCollection lc = ipmp.getIPMPEngineSchemaHandler().getLicenseCollection();
        RightsDescriptor rd = ipmp.getIPMPEngineSchemaHandler().getRightsDescriptor();
        org.iso.mpeg.mxm.tEngine.ipmpTE.schemahandler.ipmpinfo.License ipmp_l = ipmp
            .getIPMPEngineSchemaHandler().getIPMPLicense();
        License l = relEngine.getRELEngineSchemaHandler().getLicense();
        l.setLicenseId("1234");
        ipmp_l.addLicense(l);
        rd.setLicense(ipmp_l);
        lc.setRightsDescriptor(rd);
        gen_desc.setLicenseCollection(lc);
        toplevelitem.setIdentifier("temp:id:123456:SecTest");
        toplevelitem.addApplicationSpecificData(asd);
        err.setOperation("match");
        err.setSAP("localhost://");
        err.setERRIdentifier("SecTest");
        innerItem1.setId("B0");
        innerItem1.setIdentifier("urn:Sec");
        innerItem2.setId("B1");
        innerItem2.setIdentifier("unr:Test");
        Component c = didEngine.getDIEngineSchemaHandler().getComponent();
        Resource r = didEngine.getDIEngineSchemaHandler().getResource();
       
        r.setEncoding("encoding--resource");
        r.setMime("mime--resource");
        r.setRef("href--resource");
        r.getContentEncoding().add("content--encoding--resource");
        c.addResource(r);
        innerItem1.addComponent(c);
        innerItem1.addIPMPGeneralInfoDescriptor(gen_desc);
        toplevelitem.addComponent(c);
        toplevelitem.addItem(innerItem1);
        toplevelitem.addItem(innerItem2);
        toplevelitem.addERR(err);
        RelatedIdentifier ri = didEngine.getDIEngineSchemaHandler()
            .getRelatedIdentifier();
        ri.setRelationship("relation");
        ri.setValue("value relat");
        toplevelitem.addRelatedIdentifier(ri);
        di.setMPEGMItem(toplevelitem);
        di.createFile(new File(OUTPUT_DI_FILE_NAME));
        logger.info("The DCI file was successfully created in file: "
            + OUTPUT_DI_FILE_NAME);
        return di;
    }

    private void printDocument(Document doc)
    {
        StringWriter out = new StringWriter();
        XMLSerializer serializer = new XMLSerializer(out, new OutputFormat(doc));
        try
        {
            serializer.serialize(doc);
        }
        catch (IOException e)
        {
            logger.error(e.getMessage(), e);
        }
        logger.info(out.toString());
    }

    public static void main(String[] args) throws Exception
    {
        new DISignerVerifier();
    }
}
