/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino <angelo@cedeo.net>
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.core;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.IOException;
import java.io.InputStream;

import javax.xml.bind.JAXBException;

import org.apache.log4j.Logger;

import org.iso.mpeg.mxm.core.configuration.MXMConfiguration;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.configuration.MXMGenericParameters;
import org.iso.mpeg.mxm.core.configuration.MXMParameters;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineManager;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineNotFoundException;

import org.iso.mpeg.mxm.core.event.management.MXMEventHandler;

/**
 * This class enable MPEGM Applications to obtain instances of MPEGM Engines. MPEGM
 * acts as a factory instantiating the MPEGM engines listed in the MPEGM
 * Configuration file when they are required by MPEGM Applications.
 * 
 * MPEGM is singleton, in order to instantiate it do something like:
 * 
 * <pre>
 * File f = new File(getClass().getClassLoader().getResource("MPEGMConfiguration.xml").getFile());
 * MPEGM mpegm = MPEGM.getInstance(f);
 * </pre>
 * 
 * @author Filippo Chiariglione <filippo@cedeo.net>
 * @author Angelo Difino <angelo@cedeo.net>
 * @author Az Mousas <azmousas@icbnet.ece.ntua.gr
 * @author Angelos-Christos Anadiotis <aca@icbnet.ntua.gr>
 */

public class MXM extends MXMAdapter{

	private Logger logger = Logger.getLogger(MXM.class);

	private static MXM mxm = null;

	private MXMGenericParameters mxmParameters = null;
	private MXMEngineManager engineManager = null;
    private MXMEventHandler eventHandler = null;
    
    private MXMConfiguration mxmConfiguration = null; 

	private MXM() throws MXMConfigurationException {

	}

	private MXM(File mxmConfigFile) throws MXMConfigurationException, MXMEngineException {
		this.initialize(mxmConfigFile);
	}

	private MXM(InputStream is) throws MXMConfigurationException, MXMEngineException {
		this.initialize(is);
	}

	private void initialize(InputStream is) throws MXMConfigurationException, MXMEngineException {
	    MXMEngineManager.killInstance();
		logger.info(" ------- Multimedia Extensible Middleware ------- ");
		logger.info(" - Version " + this.getMxmVersion());
        if (eventHandler == null)
        {
            eventHandler = new MXMEventHandler();
        }
		
		try {
			mxmConfiguration = new MXMConfiguration(is);
		} catch (Exception e) {
			throw new MXMConfigurationException(e.getMessage(), e);
		} 

		if (mxmConfiguration.getMXMParameters() != null)
			mxmParameters = new MXMParameters(mxmConfiguration.getMXMParameters());

		if (mxmConfiguration.getMXMEngineElements() != null)
			engineManager = MXMEngineManager.getInstance(mxmConfiguration.getMXMEngineElements());
	}

	public void writeConfig(String fileLocation) throws JAXBException, IOException{
		mxmConfiguration.generateFile(fileLocation);
	}
	
	public static MXM getInstance() throws MXMConfigurationException {
		if (mxm == null)
			throw new MXMConfigurationException("MXM has not been initialised yet.");
		else
			return mxm;
	}

	public static MXM getInstance(File mxmConfigFile) throws MXMConfigurationException, MXMEngineException {
		if (!testMXMConfigFile(mxmConfigFile))
		{
			throw new MXMConfigurationException("The MXM Configuration file: " +
					mxmConfigFile.getAbsolutePath() + " does not exist or is corrupted.");
		}
		else
		{
			mxm = new MXM(mxmConfigFile);
			return mxm;
		}	}

	public static MXM getInstance(InputStream is) throws MXMConfigurationException, MXMEngineException {
		mxm = new MXM(is);
		return mxm;
	}
	
	public static MXM getInstance(Object caller) throws MXMConfigurationException, MXMEngineException
	{
		File f = new File(caller.getClass().getClassLoader().getResource("MXMConfiguration.xml").getFile());
		mxm = new MXM(f);
		return mxm;
	}
	
	public static MXM getInstance(Object caller, String filename) throws MXMConfigurationException, MXMEngineException
	{
		File f = null;
		if ((filename == null) || (filename.isEmpty()))
		{
			return getInstance(caller);
		}
		else
		{
			f = new File(caller.getClass().getClassLoader().getResource(filename).getFile());
			Logger log = Logger.getLogger(MXM.class);
			if (!f.exists())
			{
				log.error("Could not locate file " + filename + " in the CLASSPATH; Trying with Default Settings...");
				return getInstance(caller);
			}
		}	
		mxm = new MXM(f);
		return mxm;
	}

	public void initialize(File mxmConfigFile) throws MXMConfigurationException, MXMEngineException {
	    MXMEngineManager.killInstance();
		logger.info(" ------- Multimedia Extensible Middleware ------- ");
		logger.info(" - Version " + this.getMxmVersion());
		logger.info(" - MXM Configuration file: " + mxmConfigFile.getAbsolutePath());
        if (eventHandler == null)
        {
            eventHandler = new MXMEventHandler();
        }
		mxmConfiguration = new MXMConfiguration(mxmConfigFile);

		logger.info("Initializing parameters...");
		if (mxmConfiguration.getMXMParameters() != null)
			mxmParameters = new MXMParameters(mxmConfiguration.getMXMParameters());

		logger.info("Initializing engines...");
		if (mxmConfiguration.getMXMEngineElements() != null)
			engineManager = MXMEngineManager.getInstance(mxmConfiguration.getMXMEngineElements());
		
		logger.info("DONE Initializing...");
	}

	/**
	 * Returns the engine container, containing the Engine and other information
	 * 
	 * @param engineName
	 * @param engineID
	 * @return the sought engine or null if not present
	 * @throws MXMEngineNotFoundException
	 */
	public MXMEngine getEngineContainer(String engineName, String engineID) throws MXMEngineNotFoundException {
		if (engineManager != null)
			return engineManager.getEngine(engineName, engineID);
		else {
			logger.warn("No MXM Engines available");
			return null;
		}
	}

	/**
	 * Returns the default engine container, having engineID = 0, containing the
	 * Engine and other information
	 * 
	 * @param engineName
	 * @param engineID
	 * @return the sought engine or null if not present
	 * @throws MXMEngineNotFoundException
	 */
	public MXMEngine getDefaultEngineContainer(String engineName) throws MXMEngineNotFoundException {
		if (engineManager != null)
			return engineManager.getDefaultEngine(engineName);
		else {
			logger.warn("No MXM Engines available");
			return null;
		}
	}

	/**
	 * Returns the MXM Engine
	 * 
	 * @param engineName
	 * @param engineID
	 * @return the sought engine or null if not present
	 * @throws MXMEngineNotFoundException
	 */
	public MXMEngine getEngine(String engineName, String engineID) throws MXMEngineNotFoundException {
		if (engineManager != null)
			return engineManager.getEngine(engineName, engineID);
		else {
			logger.warn("No MXM Engines available");
			return null;
		}
	}

	/**
	 * Returns the default MXM engine having engineID = 0
	 * 
	 * @param engineName
	 * @param engineID
	 * @return the sought engine or null if not present
	 * @throws MXMEngineNotFoundException
	 */
	public MXMEngine getDefaultEngine(String engineName) throws MXMEngineNotFoundException {
		if (engineManager != null)
			return engineManager.getDefaultEngine(engineName);
		else {
			logger.warn("No MXM Engines available");
			return null;
		}
	}

	public MXMGenericParameters getMxmParameters() {
		return mxmParameters;
	}

	// ----- utlities methods -----

	public void listEngines() {
		if (engineManager != null)
			engineManager.listEngines();
		else
			logger.warn("No MXM Engines available");
	}

	public void listEnginesAndProperties() {
		if (mxmParameters != null)
			mxmParameters.printProperties();
		else
			logger.warn("No MXM Parameters were set.");

		if (engineManager != null)
			engineManager.printInfoAndProperties();
		else
			logger.warn("No MXM Engines available");
	}

	public void listProperties() {
		if (mxmParameters != null)
			mxmParameters.printProperties();
		else
			logger.warn("No MXM Parameters were set.");
	}

	private static boolean testMXMConfigFile(File f) {
		if (f == null)
			return false;
		return (f.exists() && f.isFile() && f.canRead() && f.length() > 0);
	}
    public MXMEventHandler getMXMEventHandler()
    {
        return eventHandler;
    }

}
