/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is the Turin Chillout task force ;-)  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: the Turin Chillout task force ;-)  
 * Contributor(s): Angelo Difino <angelo@cedeo.net>
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.core;

/**
 * This is a basic class implementing the <code>MPEGMObject</code> interface which is a wrapper of specific objects
 * that can therefore be exchanged by MPEGM Engines. By means of an MPEGMAdapter it is possible to convert any object
 * into an MPEGMObject by doing the following:
 *
 * <pre>
 * Foo foo = new Foo();
 * MPEGMObject fooObject = new MPEGMAdapter(foo);
 * </pre>
 * 
 * @author Filippo Chiariglione <filippo@cedeo.net>
 * @author Walter Allasia <allasia@eurix.it>
 * @author Francesco Gallo <gallo@eurix.it>
 * @author Angelo Difino <angelo@cedeo.net> 
 */

public class MXMAdapter implements MXMObject {

	private Object content = null;
	
	private String contentType = null;

	/**
	 * Default constructor
	 */
	public MXMAdapter() {
	}

	/**
	 * Constructor.
	 * 
	 * @param content the object becoming an MXMObject.
	 */
	public MXMAdapter(Object content) {
		this.content = content;
		if (content != null)
			this.contentType = content.getClass().getCanonicalName();
	}

	/**
	 * This method is used to obtain the current version of MXM.
	 * 
	 * @return the current version of MXM.
	 */
	public String getMxmVersion() {
		return mxmVersion;
	}

	/**
	 * This method is used to obtain the object wrapped by an MXMObject.
	 * 
	 * @return the object wrapped by the MXMObject, if any, or null otherwise.
	 */
	public Object getContent() {
		return content;
	}

	/**
	 * This method is used to force the content of the MXMObject to a specific object.
	 */
	public void setContent(Object content) {
		this.content = content;
		if (content != null)
			this.contentType = content.getClass().getCanonicalName();
	}

	/**
	 * This method is employed to get the type of the object, i.e. the canonical name of its class.
	 * 
	 * @return the canonical name of the class of the object wrapped by the MXMObject.
	 */
	public String getContentType() {
		return contentType;
	}
	
	/**
	 * This method is used to determine whether the MXMObject is actually wrapping an object.
	 * 
	 * @return true if the MXMObject is actually wrapping an instance of a class.
	 */
	public boolean hasContent() {
		return (content != null) ? true : false;
	}
		
}
