/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.core.configuration;

import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.util.List;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.apache.xerces.dom.DocumentImpl;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.iso.mpeg.mxm.configuration.schema.MXMConfigurationType;
import org.iso.mpeg.mxm.configuration.schema.MXMEngineType;
import org.iso.mpeg.mxm.configuration.schema.MXMParameterType;
import org.iso.mpeg.mxm.configuration.schema.ObjectFactory;
import org.iso.mpeg.mxm.tEngine.utils.SchemaNamespacePrefixMapperImpl;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * MPEGMConfiguration
 * 
 * @author Angelo Difino <angelo@cedeo.net>
 * @author Filippo Chiariglione
 */

public class MXMConfiguration {

	private MXMConfigurationType mxmConfigurationType = null;
	private ObjectFactory factory = null;
	private JAXBContext jc = null;
	private Marshaller m = null;

	private Logger logger = Logger.getLogger(MXMConfiguration.class);
	
	
	public MXMConfiguration() throws JAXBException {
		this.initialise();
		this.mxmConfigurationType = factory.createMXMConfigurationType();
	}

	public MXMConfiguration(MXMConfigurationType mxmConfigurationType) throws JAXBException {
		this.initialise();
		this.mxmConfigurationType = mxmConfigurationType;
	}
	
	public MXMConfiguration(File mxmConfigurationFile) throws MXMConfigurationException {
		try {
			this.initialise();
			Unmarshaller u = jc.createUnmarshaller();
			DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
			domFactory.setNamespaceAware(true);
			DocumentBuilder builder = domFactory.newDocumentBuilder();
			Document aDOM = builder.parse(mxmConfigurationFile);
			JAXBElement<?> anElement = (JAXBElement<?>) u.unmarshal(aDOM);
			this.mxmConfigurationType = (MXMConfigurationType) anElement.getValue();
		} catch (Exception e) {
			String message = "An Exception occurred while parsing the MXMConfiguration file: " + mxmConfigurationFile.getAbsolutePath();
			logger.error(message, e);
			throw new MXMConfigurationException(message);
		}
	}
	
	public MXMConfiguration(String aString) throws IOException, SAXException, JAXBException {
        this.initialise();
		DOMParser domParser = new DOMParser(); 
        domParser.parse(new InputSource(new StringReader(aString))); 
        Document aDOM = domParser.getDocument(); 
		Unmarshaller u = jc.createUnmarshaller();
		JAXBElement<?> anElement = (JAXBElement<?>) u.unmarshal(aDOM);
		this.mxmConfigurationType = (MXMConfigurationType) anElement.getValue();
	}

	public MXMConfiguration(Document aDOM) throws JAXBException {
		this.initialise();
		Unmarshaller u = jc.createUnmarshaller();
		JAXBElement<?> anElement = (JAXBElement<?>) u.unmarshal(aDOM);
		this.mxmConfigurationType = (MXMConfigurationType) anElement.getValue();
	}
	
	public MXMConfiguration(InputStream is) throws JAXBException, ParserConfigurationException, SAXException, IOException{
		this(parseInputStream(is));
	}

	private void initialise() throws JAXBException{
		factory = new ObjectFactory();
		jc = JAXBContext.newInstance("org.iso.mpeg.mxm.configuration.schema");
		m = jc.createMarshaller();
		m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);		
		m.setProperty("com.sun.xml.bind.namespacePrefixMapper", SchemaNamespacePrefixMapperImpl.getInstance());
	}
	
	public Document generateDOM() throws JAXBException	{
		Document doc = new DocumentImpl();
		JAXBElement<?> anElement = factory.createMXMConfiguration(this.mxmConfigurationType);
		m.marshal(anElement, doc);
		return doc;
	}
	public FileOutputStream generateFile(String fileLocation) throws JAXBException, FileNotFoundException {
		FileOutputStream outputFile = new FileOutputStream(fileLocation);
		JAXBElement<?> anElement = factory.createMXMConfiguration(this.mxmConfigurationType);
		m.marshal(anElement, outputFile);
		try{
			outputFile.flush();
			outputFile.close();
		} catch (IOException e)
		{
			throw new FileNotFoundException(e.getMessage());
		}
		return outputFile;
	}

	public String generateString() throws JAXBException, IOException {
		StringWriter out = new StringWriter(); 
		Document doc = this.generateDOM();
		XMLSerializer serializer = new XMLSerializer(out, new OutputFormat(doc)); 
        serializer.serialize(doc); 
        return out.toString(); 
	}
	
	public MXMConfigurationType getMXMConfigurationType() {
		return this.mxmConfigurationType;
	}

	public MXMParameterType getMXMParameters() {
		return this.mxmConfigurationType.getMXMParameters();
	}

	public List<MXMEngineType> getMXMEngineElements() {
		//return this.mxmConfigurationType.getMXMEngine();
		return this.mxmConfigurationType.getMXMEngine();
	}
	
	private static Document parseInputStream(InputStream is) throws ParserConfigurationException, SAXException, IOException{
		 DocumentBuilderFactory dbf = DocumentBuilderFactory.newInstance();

	      dbf.setValidating(false);
	      dbf.setIgnoringComments(false);
	      dbf.setIgnoringElementContentWhitespace(true);
	      dbf.setNamespaceAware(true);
	      DocumentBuilder db = null;
	      db = dbf.newDocumentBuilder();
	      return db.parse(is);
	}
}
