/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.core.configuration;

import java.util.Enumeration;
import java.util.Iterator;
import java.util.List;
import java.util.Properties;
import java.util.Set;
import java.util.Map.Entry;

import org.apache.log4j.Logger;

import org.iso.mpeg.mxm.configuration.schema.ComplexParameterType;
import org.iso.mpeg.mxm.configuration.schema.KeyValueParameterType;
import org.iso.mpeg.mxm.configuration.schema.MXMGenericParameterType;

/**
 * MPEGMGenericParameters
 * 
 * @author Angelo Difino <angelo@cedeo.net>
 * @author Filippo Chiariglione
 */

public class MXMGenericParameters {

	private List<KeyValueParameterType> entries;
	
	protected Properties properties = null;
	private List<ComplexParameterType> complexParams = null;

	private Logger logger = Logger.getLogger(MXMGenericParameters.class);

	public MXMGenericParameters (MXMGenericParameterType mxmGenericParametersType) throws MXMConfigurationException {
		initialise(mxmGenericParametersType);
	}

	public MXMGenericParameters () {
	}

	protected void initialise(MXMGenericParameterType mxmParametersType) throws MXMConfigurationException {
		if (mxmParametersType != null) {
			properties = new Properties();
	
			complexParams = mxmParametersType.getComplexParameter();
	
			List<KeyValueParameterType> entries = mxmParametersType.getEntry();
			manageKeyValueParameters(entries);
		}
	}

	private void manageKeyValueParameters(List<KeyValueParameterType> entries) {
		for(KeyValueParameterType param : entries) {
			String key = param.getKey();
			String value = param.getValue();
			properties.setProperty(key, value);
		}
		
		//save temporally the entries so we'll be able to update them
		this.entries=entries;
	}

	public String getProperty (String key) {
		return properties.getProperty(key);
	}

	public void setProperty (String key, String value) {
		properties.setProperty(key, value);
		
		for(KeyValueParameterType param : entries) {
			if (param.getKey().equals(key)){
				param.setValue(value);
				break;
			}
		}
	}

	public List<ComplexParameterType> getComplexParameters() {
		return complexParams;
	}

	public void add(MXMGenericParameters more) {
		if (more == null) {
			logger.warn("Null parameters object was given in input.");
			return;
		}
		
		//keyValue parameters
		if (properties == null)
			properties = new Properties();

		Set<Entry<Object, Object>> entries = more.properties.entrySet(); 
		for (Iterator<Entry<Object, Object>> iterator = entries.iterator(); iterator.hasNext();) {
			Entry<Object, Object> entry = iterator.next();
			properties.put(entry.getKey(), entry.getValue());
		}
		
		//complex parameters
		if (more.getComplexParameters() != null && more.getComplexParameters().size() > 0) {
			this.complexParams.addAll(more.getComplexParameters());
		}
	}

	public void add(Properties newProps) {
		if (newProps == null) {
			logger.warn("Null properties object was given in input.");
			return;
		}
		
		//keyValue parameters
		if (properties == null)
			properties = new Properties();

		Set<Entry<Object, Object>> entries = newProps.entrySet(); 
		for (Iterator<Entry<Object, Object>> iterator = entries.iterator(); iterator.hasNext();) {
			Entry<Object, Object> entry = iterator.next();
			properties.put(entry.getKey(), entry.getValue());
		}
	}

	public void printProperties() {
		logger.info("\n------------- Properties:");
		Enumeration<Object> keys = properties.keys();
		while (keys.hasMoreElements()) {
			String key = (String) keys.nextElement();
			String value = (String) properties.get(key);
			logger.info(key + ": " + value);
		}
	}

}