/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.core.engine.management;

/**
 * MPEGMEngineLoader
 * 
 * @author Angelo Difino <angelo@cedeo.net>
 * @author Filippo Chiariglione
 * @author Angelos-Christos Anadiotis <aca@icbnet.ntua.gr>
 */

import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.configuration.schema.DependentMXMEngineType;
import org.iso.mpeg.mxm.configuration.schema.MXMDependenciesType;
import org.iso.mpeg.mxm.configuration.schema.MXMEngineType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngine;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.configuration.MXMGenericParameters;

public class MXMEngineLoader extends MXMAdapter {
	
	private MXMEngine mxmEngine = null;
	
	private Logger logger = Logger.getLogger(MXMEngineLoader.class);
	
	public MXMEngineLoader() throws MXMConfigurationException {
	}

	public MXMEngineLoader(MXMEngineType engine) throws MXMConfigurationException {
		this.load(engine);
	}
	
	public List<MXMEngine> load(List<MXMEngineType> mxmEngineElements) throws MXMConfigurationException, MXMEngineException
	{
		List<MXMEngine> loadedEngines = new ArrayList<MXMEngine>();
		List<MXMEngine> preMatureLoadedEngines = new ArrayList<MXMEngine>();
		int nofLoadedEngines = 0;
		int nofEngines = 0;
		if (mxmEngineElements != null)
		{
			nofEngines = mxmEngineElements.size();
			int currentEngine = 0;
			int nofLoops = 0;
			int upperBound = nofEngines * nofEngines;
			while ((nofLoadedEngines < nofEngines) && (nofLoops < upperBound)) 
			{
				MXMEngineType mxmEngineElement = mxmEngineElements.get(currentEngine);
				if (!engineListContainsId(loadedEngines, mxmEngineElement.getType(), mxmEngineElement.getId()))
				{
					// first load the engines with no / only schemahandler dependencies
					MXMDependenciesType mxmDependencies = mxmEngineElement.getEngineDependencies();
					List<DependentMXMEngineType> engineDependencies = new ArrayList<DependentMXMEngineType>();
					if (mxmDependencies != null)
					{
						engineDependencies = mxmDependencies.getDependentMXMEngine();
					}
					if ((engineDependencies == null) || (engineDependencies.size() == 0))
					{
						MXMEngine engine = load(mxmEngineElement);
						loadedEngines.add(engine);
						logger.info("Loaded engine: " + engine.getEngineName());
						nofLoadedEngines ++;
					}
					else
					{
						boolean dependenciesSatisfied = true;
						boolean prematureDependenciesSatisfied = true;
						for (DependentMXMEngineType engineDependencyType : engineDependencies)
						{
//							logger.info("Checking dependency: " + engineDependencyType.getId());
							if ((!engineListContainsId(loadedEngines, null, engineDependencyType.getId()))
									&& (!engineListContainsId(preMatureLoadedEngines, null, engineDependencyType.getId())))
							{
								dependenciesSatisfied = false;
							}
						}
						if (dependenciesSatisfied)
						{
							MXMEngine engine = load(mxmEngineElement);
//							logger.info("Loading engine: " + mxmEngineElement.getType());
							List<MXMEngine> copyOfloadedEngines = new ArrayList<MXMEngine>(loadedEngines);
							List<MXMEngine> copyOfPrematureLoadedEngines = new ArrayList<MXMEngine>(preMatureLoadedEngines);
							engine.setDependencies(copyOfloadedEngines);
							engine.setDependencies(copyOfPrematureLoadedEngines);
							loadedEngines.add(engine);
							logger.info("Loaded engine: " + engine.getEngineName());
							nofLoadedEngines ++;
						}
						else
						{
							if (!engineListContainsId(preMatureLoadedEngines, mxmEngineElement.getType(), mxmEngineElement.getId()))
							{
								// if all not satisfied dependencies are only technology handlers
								// then premature engine loading is supported
								for (DependentMXMEngineType engineDependencyType : engineDependencies)
								{
									if ((!engineListContainsId(loadedEngines, null, engineDependencyType.getId()))
											&& ((engineDependencyType.isSchemahandler())))
									{
										prematureDependenciesSatisfied = false;
									}
								}
								if (prematureDependenciesSatisfied)
								{
//									logger.info("Loading premature engine: " + mxmEngineElement.getType() + " Id: " + mxmEngineElement.getId());
									MXMEngine engine = load(mxmEngineElement);
									List<MXMEngine> copyOfloadedEngines = new ArrayList<MXMEngine>(loadedEngines);
									List<MXMEngine> copyOfPrematureLoadedEngines = new ArrayList<MXMEngine>(preMatureLoadedEngines);
									engine.setDependencies(copyOfloadedEngines);
									engine.setDependencies(copyOfPrematureLoadedEngines);
									preMatureLoadedEngines.add(engine);
									logger.info("Loaded premature engine: " + engine.getEngineName() + " Id: " + engine.getEngineID());
								}
							}
						}
					}
				}
				currentEngine = (currentEngine + 1) % nofEngines;
				nofLoops ++;
			}
		}
		if (nofLoadedEngines < nofEngines)
		{
			String failedEngines = "Engines Failed: ";
			for (MXMEngineType mxmEngineType : mxmEngineElements)
			{
				if (!engineListContainsId(loadedEngines, mxmEngineType.getType(), mxmEngineType.getId()))
				{
					failedEngines += "\n|" + mxmEngineType.getType() + " (Id = " + mxmEngineType.getId() + ")| ";
				}
			}
			throw new MXMConfigurationException("There is possibly a cycle in MXMEngines Dependencies. " + failedEngines);
		}
		return loadedEngines;
	}
	
	private boolean engineListContainsId(final List<MXMEngine> mxmEngines, String type, String id)
	{
		boolean contains = false;
		
		if (mxmEngines != null)
		{
			for (MXMEngine mxmEngine : mxmEngines)
			{
				if ((mxmEngine.getEngineID().equals(id)) && ((type == null) || (mxmEngine.getEngineName().equals(type))))
				{
					contains = true;
					break;
				}
			}
		}
		
		return contains;
	}

	public MXMEngine load(MXMEngineType mxmEngineElement) throws MXMConfigurationException {
		
/*		//Get the name of the engine
		MXMEngineName engineName = null;
		boolean found = false;
		for (MXMEngineName tmpName : MXMEngineName.values()) {
			if (mxmEngineElement.getType().equals(tmpName.toString())) {
				engineName = tmpName;
				found = true;
			}
		}
		
		if (!found) {
			throw new MXMConfigurationException("The engine: " + mxmEngineElement.getType() + 
					"\nspecified in the MXM Configuration file was not recognised.");
		}*/
		
		MXMEngineType engineType = mxmEngineElement;

		//get the engine ID
		String engineID = engineType.getId();

		MXMGenericParameters engineParameters = null;
		//get the engine properties
		if (engineType.getEngineParameters() != null)
			engineParameters = new MXMGenericParameters(engineType.getEngineParameters());
		
		//load the engine
		String engineClassName = engineType.getClassName();
		try {
			Class<?> cls = Class.forName(engineClassName);
			mxmEngine = (MXMEngine)cls.newInstance();
			mxmEngine.initialise(engineID, engineParameters, engineType.isIsDefaultEngine());
			return mxmEngine;
			
		} catch (ClassCastException e) {
			e.printStackTrace();
			logger.error("A ClassCastException occurred.", e);
			throw new MXMConfigurationException("The class: " + engineClassName + 
					"\nspecified for the MXMEngine: " /*+ engineName */+ " is not an MXMEngine.");
		} catch (ClassNotFoundException e) {
			e.printStackTrace();
			logger.error("A ClassNotFoundException occurred.", e);
			throw new MXMConfigurationException("The class: " + engineClassName + 
					"\nspecified for the MXMEngine: "  /*+engineName */+ " was not found in the classpath.");
		} catch (InstantiationException e) {
			e.printStackTrace();
			logger.error("An InstantiationException occurred.", e);
			throw new MXMConfigurationException("The class: " + engineClassName + 
					"\nspecified for the MXMEngine: "  /*+engineName*/ + " Could not be instantiated.");
		} catch (IllegalAccessException e) {
			e.printStackTrace();
			logger.error("An IllegalAccessException occurred.", e);
			throw new MXMConfigurationException("The class: " + engineClassName + 
					"\nspecified for the MXMEngine: "  /*+engineName */+ " Could not be instantiated.");
		}
	}

	public MXMEngine getMxmEngine() {
		return mxmEngine;
	}
	
}
