/**
 * The copyright in this software is being made available under the BSD License,
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 * 
 * Copyright (c) 2012, CEDEO
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *     - Redistributions of source code must retain the above copyright notice, this 
 *     list of conditions and the following disclaimer.
 *     - Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *     - Neither the name of the CEDEO nor the names of its contributors may be used 
 *     to endorse or promote products derived from this software without specific 
 *     prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS �AS IS� AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
*/

package org.iso.mpeg.mxm.tEngine.digitalitemTE;

import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.ApplicationSpecificData;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Component;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Container;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDL;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.DIDLInfo;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Descriptor;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Resource;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Statement;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl.Item;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.dii.RelatedIdentifier;

/**
 * Classes extending this abstract classes are entry point classes for operating on ISO/IEC 21000-2 
 * Digital Item Declaration (DID) data structures
 * 
 * The MP21 profile used by default by this Engine is specified in MPEG-M Part-4 and in MSAF specification
 * (ISO/IEC FDIS 23000-5) (The APIs here retrieved refers to the package �didl�)
 * 
 * @author Angelo Difino <angelo@cedeo.net>
 */
public interface DIEngineSchemaHandler {

		/**
		 * This method returns an instance of class <code>ApplicationSpecificData</code> which is 
		 * needed to parse/create a ApplicationSpecificData XML fragment
		 * @return an object of class <code>ApplicationSpecificData</code> or null if this method 
		 * has not been implemented by the specific DIEngine.
		 */	
		public abstract ApplicationSpecificData getApplicationSpecificData() throws MXMEngineException;

		/**
		 * This method returns an instance of class <code>Component</code> which is 
		 * needed to parse/create a Component XML fragment
		 * @return an object of class <code>Component</code> or null if this method 
		 * has not been implemented by the specific DIEngine.
		 */		
		public abstract Component getComponent() throws MXMEngineException;
		
		/**
		 * This method returns an instance of class <code>Container</code> which is 
		 * needed to parse/create a Container XML fragment
		 * @return an object of class <code>Container</code> or null if this method 
		 * has not been implemented by the specific DIEngine.
		 */			
		public abstract Container getContainer() throws MXMEngineException;
		
		/**
		 * This method returns an instance of class <code>Descriptor</code> which is 
		 * needed to parse/create a Descriptor XML fragment
		 * @return an object of class <code>Descriptor</code> or null if this method 
		 * has not been implemented by the specific DIEngine.
		 */				
		public abstract Descriptor getDescriptor() throws MXMEngineException;
		
		/**
		 * This method returns an instance of class <code>DIDL</code> which is 
		 * needed to parse/create a DIDL XML fragment
		 * @return an object of class <code>DIDL</code> or null if this method 
		 * has not been implemented by the specific DIEngine.
		 */				
		public abstract DIDL getDIDL() throws MXMEngineException, MXMException;
		
		/**
		 * This method returns an instance of class <code>DIDLInfo</code> which is 
		 * needed to parse/create a DIDLInfo XML fragment
		 * @return an object of class <code>DIDLInfo</code> or null if this method 
		 * has not been implemented by the specific DIEngine.
		 */				
		public abstract DIDLInfo getDIDLInfo() throws MXMEngineException;
		
		/**
		 * This method returns an instance of class <code>Item</code> which is 
		 * needed to parse/create a Item XML fragment
		 * @return an object of class <code>Item</code> or null if this method 
		 * has not been implemented by the specific DIDEngine.
		 */				
		public abstract Item getItem() throws MXMEngineException;
		
		/**
		 * This method returns an instance of class <code>Resource</code> which is 
		 * needed to parse/create a Resource XML fragment
		 * @return an object of class <code>Resource</code> or null if this method 
		 * has not been implemented by the specific DIEngine.
		 */				
		public abstract Resource getResource() throws MXMEngineException;

		/**
		 * This method returns an instance of class <code>Statement</code> which is 
		 * needed to parse/create a Statement XML fragment
		 * @return an object of class <code>Statement</code> or null if this method 
		 * has not been implemented by the specific DIEngine.
		 */			
		public abstract Statement getStatement() throws MXMEngineException;
		
		/**
		 * This method returns an instance of class <code>RelatedIdentifier</code> which is 
		 * needed to parse/create a RelatedIdentifier XML fragment
		 * @return an object of class <code>RelatedIdentifier</code> or null if this method 
		 * has not been implemented by the specific DIEngine.
		 */					
		public abstract RelatedIdentifier getRelatedIdentifier() throws MXMEngineException;
		
		/**
		 * This method returns an instance of class <code>StructuredData</code> which is 
		 * needed to parse/create a StructuredData XML fragment
		 * @return an object of class <code>StructuredData</code> or null if this method 
		 * has not been implemented by the specific DIEngine.
		 */					
		public abstract StructuredData getStructuredData() throws MXMEngineException;		
}
