/**
 * The copyright in this software is being made available under the BSD License,
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, CEDEO
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *     - Redistributions of source code must retain the above copyright notice, this 
 *     list of conditions and the following disclaimer.
 *     - Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *     - Neither the name of the CEDEO nor the names of its contributors may be used 
 *     to endorse or promote products derived from this software without specific 
 *     prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 */

package org.iso.mpeg.mxm.tEngine.eventreportTE;

import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ER;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;

/**
 * Classes implementing this interface are entry point classes for operating on <tt>ISO/IEC 21000-15 Part 15:
 * Event Reporting</tt>, Event Report (ER) and Event Report Request (ERR) data structures
 * <p/>
 * The ERR (Event Report Request) is a request to report an Event(s)
 * <br/>
 * The ER (Event Report) is the representation of an Event(s) as specified by the related Event Report Request
 * <p/>
 * An Event Report Request (ERR) is used to define the conditions (predicates) under which an Event is
 * deemed to have occurred. Events defined by ERRs trigger the creation of an associated Event Report (ER),
 * which contains information describing the Event, as specified in the associated ERR.
 * <br/>
 * Event Reporting is used to report on the occurrence of Events which may be either directly or indirectly related
 * either to a DI (or a Peer).
 * <br/>
 * Upon the Event occurring, an Event Report may be generated and delivered to the specified recipient(s). The
 * Event Report Request may specify that the generation and delivery of an Event Report is mandatory; this
 * would be implemented using, for example, the Event Report Request as an <tt>obligation</tt> of a Rights Expression.
 * <br/>
 * Both ER and ER-Rs may be represented within Digital Items, and therefore can natively inherit all of the
 * properties of Digital Items.
 * <br/>
 * When Event Reports Requests and Event Reports are packaged inside a Digital Item, they are encapsulated
 * in descriptor elements.
 * <br/>
 * A Digital Item may contain any number of Event Report Requests or Event Reports.
 * <br/>
 * Event Report Request descriptors may be the only descriptor in a DI or may be provided alongside other
 * resources.
 * <p/>
 * Event Report Requests and Event Reports may be embedded within Digital Items. When this is the case, it is
 * possible to make use of ISO/IEC 21000-3 (Digital Item Identification) to identify both Event Report Requests
 * and their related Event Reports using standard Digital Item Identification (DII) mechanisms. For example, if an
 * Event Report Request (ERR) is included inside a Digital Items Item, then DII can be used to identify that
 * ER-R. The same applies for Event Reports.
 * <p/>
 * The basic model of Event Reporting indicates that Events that need to be reported
 * may be specified by interested parties through the use of an Event Report Request.
 * <br/>The ER-Rs purpose is to:
 * <br/>- describe the Event which is to be reported,
 * <br/>- indicate which Peers the Event should be reported to, and
 * <br/>- the data items that are to be included in such an Event Report(s).
 * <p/>
 *
 * @author <a href="mailto:bogdan.ardeleanu@uti.ro">Bogdan Ardeleanu</a>, UTI Systems
 * @author <a href="mailto:angelo@cedeo.net">Angelo Difino</a>
 */
public interface EventReportEngineSchemaHandler {

    /**
     * This method returns an instance of class <code>ER</code> which is
     * needed to parse/create a ER xml fragment.
     * <br/>
     * The ER (Event Report) is the representation of an Event(s) as specified by the related Event Report Request.
     *
     * @return an object of class <code>ER</code> or null if this method
     *         has not been implemented by the specific EventReportEngine.
     * @throws org.iso.mpeg.mxm.core.engine.management.MXMEngineException
     *          In case of any internal exception occur.
     */
    public abstract ER getER() throws MXMEngineException;

    /**
     * This method returns an instance of class <code>ERR</code> which is
     * needed to parse/create a ERR xml fragment.
     * <br/>
     * The ERR (Event Report Request) is a request to report an Event(s)
     *
     * @return an object of class <code>ERR</code> or null if this method
     *         has not been implemented by the specific EventReportEngine.
     * @throws org.iso.mpeg.mxm.core.engine.management.MXMEngineException
     *          In case of any internal exception occur.
     */
    public abstract ERR getERR() throws MXMEngineException;
}
