/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino
 * Contributor(s): Bogdan Ardeleanu
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler;

import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.tEngine.eventreportTE.exception.EventReportException;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;

/**
 * The <tt>ERRKeeper</tt> class is responsible for storing/loading Event Report Requests to local repository
 * identify by <tt>repositoryKey</tt>.
 * <br/>
 * The ERR repository location is defined under the engine parameter: <tt>rootERRStorageLocation</tt>
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro"> Bogdan Ardeleanu</a>
 * Date: 8/29/11
 * Time: 4:45 PM
 * <p/>
 * UTI Systems, TIC
 */
public interface ERRKeeper {

    /**
     * Method that stores an event report request (ERR) using a digital item identifier.
     *
     * @param err    The event report request to store
     * @param DIDLId The digital item identifier.
     * @return The operation status using a <tt>MXMEngineResponse</tt> object.
     * @throws EventReportException In case of internal exception occur
     */
    public MXMEngineResponse storeERR(ERR err, String DIDLId) throws EventReportException;

    /**
     * Method that loads an ERR object from current ERR repository using the ERR identifier.
     *
     * @param errId The ERR identifier
     * @return Null if the ERR object not found
     * @throws EventReportException In case of internal exception occur
     */
    public ERR getERR(String errId) throws EventReportException;

    /**
     * Method that loads an ERR object from current ERR repository using the corresponding digital item identifier.
     * <p/>
     * Hint: We may safely assume a one-to-one relationship for the moment.
     * Otherwise this method is irrelevant.
     * It should return a collection of ERR objects.
     *
     * @param DIDLId The digital item identifier
     * @return Null if the ERR object not found
     * @throws EventReportException In case of internal exception occur
     */
    public ERR getERRByDIDLId(String DIDLId) throws EventReportException;

    /**
     * Method that loads an ERR object from current ERR repository using the corresponding digital item identifier.
     * <p/>
     * Hint: We may safely assume a one-to-one relationship for the moment.
     * Otherwise this method is irrelevant.
     * It should return a collection of ERR ids.
     * <p/>
     *
     * @param DIDLId The digital item identifier
     * @return Null if the ERR object not found
     * @throws EventReportException In case of internal exception occur
     */
    public String getERRIdByDIDLId(String DIDLId) throws EventReportException;

    /**
     * Method that loads an ERR object from current ERR repository using the ERR identifier.
     * It return the coresponding digital item identifier.
     * <p/>
     * Hint: We may safely assume a one-to-one relationship for the moment.
     * Otherwise this method is irrelevant.
     * <p/>
     *
     * @param errId The ERR identifier
     * @return Null if the ERR object not found
     * @throws EventReportException In case of internal exception occur
     */
    public String getDIDLIdOfERR(String errId) throws EventReportException;
}
