/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): 
 *
 * ***** END LICENSE BLOCK ***** */


package org.iso.mpeg.mxm.tEngine.relTE.technologyhandler;

import java.util.List;

import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.relTE.exception.RELEngineException;
import org.iso.mpeg.mxm.tEngine.relTE.exception.RELParsingException;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Grant;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Issuer;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.License;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Principal;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Resource;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;

/**
 * Classes implementing this interface are used to determine whether a user is authorised to 
 * exercise a right as specified in a REL query against the rights granted in a REL license.
 * 
 * @author Filippo Chiariglione <filippo@cedeo.net>
 *
 */
public interface AuthorisationManager {

	/**
	 * This method is employed to check the authorization of a user to exercise a right over a resource.
	 * The license passed should refer to the principal directly, that is using identity holder or keyholder.
	 * In case that a DI has a license with a forAll propertyPossessor element, this means, that for each principal
	 * there will be an indirect way to obtain a license to get authorized. This happens mostly when the right to
	 * issue a grant has been given to someone other than the owner of a resource. e.g. a distributor.
	 * 
	 * @param license the license stating who can perform what on some resources. 
	 * @param principal the principal that wants to get authorized to perform an action on a resource .
	 * @param right the action
	 * @param resource the resource
	 * @return an object of type <code>AuthorisationResult</code> indicating the response of the authorisation 
	 * procedure.
	 * @throws MXMEngineException 
	 * @throws SecurityEngineException
	 */
	public AuthorisationResult authoriseUser(License license, Principal user, RELRight right, Resource resource)
			throws  MXMEngineException;
	
	/**
	 * This method shall be called after the authorised method has returned a successful result in order to 
	 * obtain the issuer to be verified who granted the right that was successfully authorised.
	 * 
	 * @return an object of type <code>IssuerParser</code> representing the issuer to be verified.
	 */
	public List<Issuer> getIssuersToVerify() ;
	
	/**
	 * This method shall be called after the authorised method has returned a successful result in order to 
	 * obtain the grant which is responsible for the successful authorisation.
	 * 
	 * @return an object of type <code>GrantParser</code> representing the grant to be verified.
	 */
	public Grant getSuccessfulGrant();
	
	
//	/**
//	 * This method is employed to verify if a user may exercise a right over a resource as indicated in
//	 * the query given in input against the license given in input.
//	 * 
//	 * @param license the license against which the user query has to be assessed 
//	 * @param query an ISO/IEC 21000-5 statement specifying what right is a principal willing to exercise
//	 * over a resource 
//	 * @param securityEngine an object of type SecurityEngine providing access to confidential information 
//	 * that might be used to assess whether the principal is authorised to exercise the right. 
//	 * @param userAlias an alias identifying the principal indicated in the query as known by the SecurityEngine
//	 * @param password the password that might be required to access confidential information stored in the
//	 * secure repository managed by the SecurityEngine
//	 * @return an object of type <code>AuthorisationResult</code> indicating the response of the authorisation 
//	 * procedure.
//	 * 
//	 * @throws RELParsingException
//	 * @throws SecurityEngineException
//	 */
//	public AuthorisationResult authorise(License license, License query, SecurityEngine securityEngine) 
//	throws RELEngineException, SecurityEngineException ;
//	
//	// OLD TODO this should be a AuthorizationResponse better
//	public AuthorisationResult authorise(License license, Issuer issuer, Principal user, RELRight right, Resource resource,
//			SecurityEngine securityEngine) throws RELEngineException, SecurityEngineException;
//	
//	public AuthorisationResult authoriseIssue(License license, Issuer issuer,
//			Principal user, RELRight right, Resource resource,Principal principalToIssue,
//			SecurityEngine securityEngine) throws RELEngineException, SecurityEngineException;
	
}
