/**
 * The copyright in this software is being made available under the BSD License,
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 * 
 * Copyright (c) 2012, CEDEO
 * All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 * 
 *     - Redistributions of source code must retain the above copyright notice, this 
 *     list of conditions and the following disclaimer.
 *     - Redistributions in binary form must reproduce the above copyright notice, 
 *     this list of conditions and the following disclaimer in the documentation 
 *     and/or other materials provided with the distribution.
 *     - Neither the name of the CEDEO nor the names of its contributors may be used 
 *     to endorse or promote products derived from this software without specific 
 *     prior written permission.
 * 
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS �AS IS� AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT 
 * NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, 
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 * OF SUCH DAMAGE.
 */
package org.iso.mpeg.mxm.tEngine.securityTE;


import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.AuthenticationManager;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.CertificateManager;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.FileAndObjectHandler;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.KeyManager;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SecureDeviceManager;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.SignatureManager;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.UserDataBaseManager;


public interface SecurityEngineTechnologyHandler
{
    /**
     * Retrieves the AuthenticationManager which is responsible for providing
     * functionalities related to client and server authentication deploying user's
     * CONVERGENCE smart cards as secure repositories. Both server- and client
     * challenge-response authentication is supported.
     * 
     * @return AuthenticationManager
     * @throws SecurityEngineException
     */
    public abstract AuthenticationManager getAuthenticationManager()
        throws SecurityEngineException;

    /**
     * Retrieves the CertificateManager which is responsible for the generation of
     * asymmetric key pairs (both on- and off-card), as well as the generation and
     * verification of dedicated digital certificates.
     * 
     * @return CertificateManager
     * @throws SecurityEngineException
     */
    public abstract CertificateManager getCertificateManager()
        throws SecurityEngineException;

    /**
     * Retrives the FileAndObjectHandler which is contains utility methods to for File and
     * security objects convertion
     * 
     * @return
     * @throws SecurityEngineException
     */
    public abstract FileAndObjectHandler getFileAndObjectHandler()
        throws SecurityEngineException;

    /**
     * Retrieves the KeyManager which is responsible for providing a number of
     * security-related functionalities such as generation of keys for symmetric
     * cryptographic algorithms, symmetric encryption and decryption, and
     * wrapping/unwrapping of symmetric keys.
     * 
     * @return KeyManager
     * @throws SecurityEngineException
     */
    public abstract KeyManager getKeyManager() throws SecurityEngineException;

    /**
     * Retrieves the SecureDeviceManager which can be used to certify and verify the
     * integrity of MXM Devices, when needed. The normal operation should be as follows:
     * 1. A Device is certified the first time is used. It means, that a fingerprint is
     * calculated with the hardware data and basic software installed. The Device is thus
     * uniquely identified. This information is sent to a secure repository.<BR>
     * 2. A Device is verfied each time its integrity is critical. Verification
     * recalculates the fingerprint and checks against the Secure repository if the
     * fingerprint has changed.<BR>
     * There are also operations to disable a device and to self-verify.
     * 
     * @return SecureDeviceManager
     * @throws SecurityEngineException
     */
    public abstract SecureDeviceManager getSecureDeviceManager()
        throws SecurityEngineException;

    /**
     * Retrieves the SignatureManager which is responsible for providing a number of
     * functionalities related to digital (asymmetric) signatures, such as the generation
     * and verification of hash-values, issuance and verification of digital signatures.
     * 
     * @return SignatureManager
     * @throws SecurityEngineException
     */
    public abstract SignatureManager getSignatureManager() throws SecurityEngineException;

    /**
     * Retrieves the UserDataBaseManager which is used to store, retrieve and manage user
     * information in a local database. Currently MySql is supported. MXMConfiguration.xml
     * files allow to set up the MySql settings for your local installation.
     * 
     * @return UserDataBaseManager
     * @throws SecurityEngineException
     */
    public abstract UserDataBaseManager getUserDataBaseManager()
        throws SecurityEngineException;
}
