/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino
 * Contributor(s): Andreas Kohlos
 *
 * ***** END LICENSE BLOCK ***** */
package org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler;


import java.security.PublicKey;
import java.security.interfaces.RSAPublicKey;

import javax.security.cert.X509Certificate;

import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;


/**
 * Classes implementing this interface are responsible for providing functionalities
 * related to client and server authentication deploying user's CONVERGENCE smart cards as
 * secure repositories. Both server- and client challenge-response authentication is
 * supported.
 * 
 * @author Andreas Kohlos
 */
public interface AuthenticationManager
{
    /**
     * Is part of the client authentication towards a server based on RSA. Signs a
     * challenge (and optionally additional information) retrieved from the server by the
     * user's CONVERGENCE smart card connected to the client. It is invoked on the client
     * connected to the user's smart card.
     * 
     * @param serverChallenge The 16 byte random challenge retrieved from the server.
     * @param keyIdentifier The identifier associated to the private key to be used by the
     *            smart card to sign the requested authentication token.
     * @param PIN The user's PIN to be checked on the smart card (authorizing the use of
     *            the card holder's private authentication key associated to
     *            keyIdentifier).
     * @return The signed authentication token cardAuthToken retrieved from the smart
     *         card.
     * @throws SecurityEngineException
     */
    public String clientAuth(String serverChallenge, String appalias, String PIN)
        throws SecurityEngineException;

    /**
     * It is a part of client authentication towards a server based on RSA. Client
     * verification consists of verifying the authentication token retrieved from the
     * client's smart card on the server. This method is invoked on the server.
     * 
     * @param cardAuthToken The authentication token retrieved from the client's smart
     *            card and sent to the server.
     * @param keyAlias The alias associated to the client user�s public key to be used for
     *            verification on the server, and contained in the client certificate
     *            validated through the previous verifyCertificate command .
     * @return True, if the client's authentication token has been verified, false
     *         otherwise.
     * @throws SecurityEngineException
     */
    public boolean clientTokenVerify(final String challenge, String cardAuthToken,
        final String algorithm, final PublicKey pubKey) throws SecurityEngineException;

    /**
     * It is a part of server authentication towards a client based on RSA. Retrieves a
     * random challenge from the client's smart card. The retrieved challenge shall then
     * be sent to the remote server for signing. This method is invoked on the client
     * connected to the user's smart card.
     * 
     * @return A random challenge retrieved from the smart card.
     * @throws SecurityEngineException
     */
    public String getCardChallenge() throws SecurityEngineException;

    /**
     * It is a part of server authentication towards a client based on RSA. Retrieves a
     * random challenge from the client's smart card. The retrieved challenge shall then
     * be sent to the remote server for signing. This method is invoked on the client
     * connected to the user's smart card.
     * 
     * @param keyIdentifier The identifier of the certificate to be retrieved from the
     *            smart card (corresponding to the key pair associated to keyIdentifier).
     * @return The certificate containing the public key from the key pair associated to
     *         keyIdentifier.
     * @throws SecurityEngineException
     */
    public String getCertificate(final String appAlias, final String PIN)
        throws SecurityEngineException;

    /**
     * It is a part of client authentication towards a server based on RSA. Generates a
     * random challenge on the server to be sent to the client for signing. This method is
     * invoked on the server.
     * 
     * @return A 16-byte random serverChallenge (to be sent to the client).
     * @throws SecurityEngineException
     */
    public String getChallenge() throws SecurityEngineException;

    RSAPublicKey getPublicKey(String appAlias, String PIN) throws SecurityEngineException;

    /**
     * It is a part of server authentication towards a client based on RSA. Server
     * authentication is based on signing a challenge (and optionally additional
     * information) retrieved from the client's smart card by the server�s private key.
     * This method is invoked on the server.
     * 
     * @param cardChallenge The random challenge retrieved from the client�s smart card.
     * @param keyAlias The alias associated to the private key to be used by the server to
     *            sign the requested authentication token.
     * @param password The password to access the key repository containing the server's
     *            private authentication key associated to keyAlias.
     * @return The signed authentication token serverAuthToken.
     * @throws SecurityEngineException
     */
    public String serverAuth(String cardChallenge, String keyAlias, String password)
        throws SecurityEngineException;

    /**
     * It is a part of client authentication towards a server based on RSA. This method is
     * employed to verify a certificate issued on a public key. This method is invoked on
     * the server.
     * 
     * @param certificate The certificate.
     * @return True, if the certificate is valid, false otherwise.
     * @throws SecurityEngineException
     */
    public boolean verifyCertificate(X509Certificate certificate, PublicKey pubKey)
        throws SecurityEngineException;

    /**
     * It is a part of server authentication towards a client based on RSA. Passes a
     * certificate retrieved from the server to the user's smart card. This method is
     * invoked on the client connected to the user's smart card. The smart card verifies
     * the certificate's signature and extracts the encapsulated public key for subsequent
     * verification of the server�s response (authentication token).
     * 
     * @param CVcertificate A card verifiable certificate to be processed by the smart
     *            card connected to client.
     * @param pubKey The public key of the holder to be certified (shall contain
     *            information on the key/algorithm type through an object identifier)
     * @return True if the certificate is valid, false otherwise.
     * @throws SecurityEngineException
     */
    public boolean verifyCVCertificate(X509Certificate CVcertificate, PublicKey pubKey)
        throws SecurityEngineException;

    RSAPublicKey getPublicKey(String appAlias, String keyIdentifier, String PIN)
        throws SecurityEngineException;
}
