/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino
 * Contributor(s): Andreas Kohlos
 *
 * ***** END LICENSE BLOCK ***** */
package org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler;


import java.math.BigInteger;
import java.security.NoSuchAlgorithmException;
import java.security.PrivateKey;
import java.security.PublicKey;
import java.security.cert.X509Certificate;
import java.security.spec.InvalidKeySpecException;

import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.RSAKeyValue;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data;


/**
 * Classes implementing this interface are responsible for the generation of asymmetric
 * key pairs (both on- and off-card), as well as the generation and verification of
 * dedicated digital certificates.
 * 
 * @author Andreas Kohlos
 */
public interface CertificateManager
{
    /**
     * Signs and issues a certificate on a public key compatible to the CONVERGENCE smart
     * card.
     * 
     * @param serialNumber The serial number of the generated certificate. The creating
     *            entity must ensure the uniqueness of the serial.
     * @param holderReference The reference of the user owning the public key which is to
     *            be certified (the certificate holder). It is required to be in ASN.1,
     *            RFC 2253 format. The following distinguished names are currently
     *            supported: CN, C, ST, L, O, OU, STREET, DC, UID where UID is used as the
     *            identifier of the certificate holder and DC is used as the application
     *            domain identifier. The application domain defines the CONVERGENCE
     *            applications which the certificate is allowed to be used for.
     * @param issuerReference The reference of the certificate issuer. It is required to
     *            be in ASN.1, RFC 2253 format. The following distinguished names are
     *            currently supported: CN, C, ST, L, O, OU, STREET, DC, UID where UID is
     *            used as the identifier of the certificate issuer and DC is used as the
     *            application domain identifier. The application domain defines the
     *            CONVERGENCE applications which the certificate is allowed to be used
     *            for.
     * @param expiryDays The number of days after which the certificate shall expire.
     * @param pubKey The public key of the holder to be certified.
     * @param privKey The private key of the issuer.
     * @param SignOID The identifier encoding the algorithm type to be selected for the
     *            certificate signature. Is currently hard coded to
     *            'SHA256withRSAEncryption'.
     * @return The signed certificate.
     * @throws SecurityEngineException
     */
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data generateCertificate(
        BigInteger serialNumber, String holderReference, String issuerReference,
        int expiryDays, PublicKey pubKey, PrivateKey privKey, String SignOID)
        throws SecurityEngineException;

    X509Data generateCertificate(BigInteger serialNumber, String holderReference,
        String issuerReference, int expiryDays, PublicKey pubKey, String SignOID)
        throws SecurityEngineException;

    /**
     * Signs and issues a card-verifiable (CV) certificate on a public key (belonging to a
     * public-private key pair which has been generated off card).
     * 
     * @param holderReference The reference of the user owning the public key which is to
     *            be certified (the certificate holder).
     * @param issuerReference The reference of the certificate issuer.
     * @param expiryDays The number of days after which the certificate shall expire.
     * @param pubKey The public key of the holder to be certified.
     * @param privKey The private key of the issuer.
     * @param SignOID The identifier encoding the algorithm type to be selected for the
     *            certificate signature.
     * @return The signed CV certificate.
     * @throws SecurityEngineException
     */
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data generateCVCertificate(
        String holderReference, String issuerReference, int expiryDays, PublicKey pubKey,
        PrivateKey privKey, String SignOID) throws SecurityEngineException;

    /**
     * Generates an RSA key pair (a private key and associated public key) in the crypto
     * module manager.
     * 
     * @param keyLength The length of RSA modulus to be generated for this key pair.
     *            Currently supported: RSA 1024/2048.
     * @param keyIdentifier
     * @return The public key of the generated key pair.
     * @throws SecurityEngineException
     */
    public PublicKey generateRSAKeyPair(String appAlias, String keyIdentifier,
        int keyLength) throws SecurityEngineException;

    // /**
    // * Generates an RSA key pair (a private key and associated public key) on the
    // * CONVERGENCE smart card and returns its public key. Also, internally validates
    // * possession of a private key. The RSA Exponent is currently always 0x10001.
    // *
    // * @param keyIdentifier The key identifier (for referencing the corresponding
    // private
    // * key on card). Should be one of 'sig', 'auth' or 'conf', specifying the
    // * key to be used either for signing, authentication or confidentiality.
    // * @param appAlias Specifies the application domain which is currently used. Should
    // be
    // * one of 1,2,3 or 4 where 1 is assigned to the Identity Provider domain
    // * and 2,3, or 4 for a Service Provider/application domain.
    // * @param keyLength The length of RSA modulus to be generated for this key pair.
    // * Currently supported: RSA 1024/2048.
    // * @param PIN The PIN to authorize the operation on the CONVERGENCE smart card.
    // * @return The public key of the pair.
    // * @throws SecurityEngineException
    // */
    // public PublicKey generateRSAKeyPairCard(String keyIdentifier, String appAlias,
    // int keyLength, String PIN) throws SecurityEngineException;
    /**
     * @param key
     * @return
     * @throws NoSuchAlgorithmException
     * @throws InvalidKeySpecException
     */
    public RSAKeyValue toPrivateRSAKeyValue(PrivateKey key)
        throws NoSuchAlgorithmException,InvalidKeySpecException;

    /**
     * @param key
     * @return
     * @throws NoSuchAlgorithmException
     * @throws InvalidKeySpecException
     */
    public RSAKeyValue toPublicRSAKeyValue(PublicKey key)
        throws NoSuchAlgorithmException,InvalidKeySpecException;

    public X509Data getUserCertificate(String keyIdentifier, String appAlias)
        throws SecurityEngineException;
}
