/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino
 * Contributor(s): Andreas Kohlos
 *
 * ***** END LICENSE BLOCK ***** */
package org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler;


import java.security.PublicKey;

import javax.crypto.SecretKey;

import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;


/**
 * Classes implementing this interface are responsible for providing a number of
 * security-related functionalities such as generation of keys for symmetric cryptographic
 * algorithms, symmetric encryption and decryption, and wrapping/unwrapping of symmetric
 * keys.
 * 
 * @author Andreas Kohlos
 */
public interface KeyManager
{
    /**
     * Symmetrically decrypts the input data by using the provided symmetric key.
     * Currently only uses 'AES/CBC/PKCS7Padding' algorithm.
     * 
     * @param data The data to be decrypted.
     * @param decryptionKey The key to be used for decrypting the data.
     * @param algorithm The algorithm to be used to decrypt the data. Is currently not
     *            used.
     * @return The decrypted data.
     * @throws SecurityEngineException
     */
    public byte[] decrypt(byte[] data, final SecretKey decryptionKey,
        final String algorithm) throws SecurityEngineException;

    /**
     * Symmetrically decrypts the input data by using the provided symmetric key.
     * Currently only uses 'AES/CBC/PKCS7Padding' algorithm.
     * 
     * @param data The data to be decrypted.
     * @param decryptionKey The key to be used for decrypting the data.
     * @param algorithm The algorithm to be used to decrypt the data. Is currently not
     *            used.
     * @return The decrypted data.
     * @throws SecurityEngineException
     */
    public byte[] decrypt(
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedData data,
        SecretKey decryptionKey, String algorithm) throws SecurityEngineException;

    /**
     * Symmetrically encrypts the input data by using the provided symmetric key.
     * Currently only uses 'AES' algorithm. The implementation uses the zero vector as IV.
     * 
     * @param data The data to be encrypted.
     * @param encryptionKey The key to be used for encrypting the data.
     * @param algorithm The algorithm to be used to encrypt the data. Is currently not
     *            used and hard coded to "AES/CBC/PKCS7Padding".
     * @return The encrypted data as a byte array.
     * @throws SecurityEngineException
     */
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedData encrypt(
        byte[] data, SecretKey encryptionKey, String algorithm)
        throws SecurityEngineException;

    /**
     * Generates a random symmetric key using the specified algorithm. Currently only uses
     * 'AES' algorithm.
     * 
     * @param algorithm The algorithm to be used with the key. Is currently not used.
     * @param keyLength The length of the key to be generated, in bytes.
     * @return The generated key.
     * @throws SecurityEngineException
     */
    public SecretKey generateRandomKey(String algorithm, int keyLength)
        throws SecurityEngineException;

    /**
     * @param appAlias
     * @param PIN
     * @return
     * @throws SecurityEngineException
     */
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data getCertificate(
        final String appAlias, final String PIN) throws SecurityEngineException;

    /**
     * @param algorithm
     * @param appAlias
     * @param PIN
     * @return
     * @throws SecurityEngineException
     */
    public boolean initStreamUnwrap(final String algorithm, final String appAlias,
        final String PIN) throws SecurityEngineException;

    /**
     * @param wrappedSymmKey
     * @return
     * @throws SecurityEngineException
     */
    public SecretKey streamUnwrapAfterInit(
        final org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey wrappedSymmKey)
        throws SecurityEngineException;

    /**
     * Asymmetrically unwraps the provided wrapped input key. Unwrapping will be performed
     * using the private key on the user's CONVERGENCE smart card (belonging to the key
     * pair associated to appAlias designated for asymmetric encryption/decryption). The
     * wrapped key (usually included in a license) is transmitted to the CONVERGENCE smart
     * card for unwrapping on-card.
     * 
     * @param wrappedSymmKey The wrapped key to be unwrapped.
     * @param appAlias Specifies the application domain which is currently used. Should be
     *            one of 1,2,3 or 4 where 1 is assigned to the Identity Provider domain
     *            and 2,3, or 4 for a Service Provider/application domain.
     * @param PIN The PIN to authorize the operation on the CONVERGENCE smart card.
     * @return The unwrapped symmetric key.
     * @throws SecurityEngineException
     */
    public SecretKey unwrap(
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey wrappedSymmKey,
        String appAlias, String PIN) throws SecurityEngineException;

    /**
     * @param symmKey The key to be wrapped. This key will be needed for decryption of
     *            data encrypted with it.
     * @param algorithm The algorithm to be used to wrap the key. The parameter is
     *            currently not used and the algorithm is hard coded to
     *            "RSA/NONE/OAEPWithSHA256AndMGF1Padding".
     * @param x509Data The certificate containing the public key to be used to encrypt
     *            symmKey.
     * @return The wrapped (asymmetrically encrypted) encryption key.
     * @throws SecurityEngineException
     */
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey wrap(
        SecretKey symmKey, String algorithm,
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data x509Data)
        throws SecurityEngineException;

    /**
     * @param symmKey
     * @param algorithm
     * @param publicKey
     * @return
     * @throws SecurityEngineException
     */
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey wrap(
        SecretKey symmKey, String algorithm, PublicKey publicKey)
        throws SecurityEngineException;
}
