/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Victor Rodriguez
 *
 * ***** END LICENSE BLOCK ***** */
package org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler;


import org.iso.mpeg.mxm.core.MXMObject;


/**
 * Classes implementing this interface can be used to certify and verify the integrity of
 * MXM Devices, when needed. The normal operation should be as follows: 1. A Device is
 * certified the first time is used. It means, that a fingerprint is calculated with the
 * hardware data and basic software installed. The Device is thus uniquely identified.
 * This information is sent to a secure repository.<BR>
 * 2. A Device is verfied each time its integrity is critical. Verification recalculates
 * the fingerprint and checks against the Secure repository if the fingerprint has
 * changed.<BR>
 * There are also operations to disable a device and to self-verify.
 * 
 * @author Victor Torres, Victor Rodriguez, Silvia Llorente
 */
public interface SecureDeviceManager
{
    /**
     * This method is employed to check the integrity of a software device prior to its
     * first usage. Some features regarding the device (operating system) and/or the
     * software device that is requesting the authorisation of a user action will be
     * registered so that they can be later checked for integrity.
     * 
     * @param deviceFingerprint An XML piece that represents the relevant features about
     *            the device (operating system) and/or the software device.
     * @param userToken An XML piece that represents the SAML token, which contains the
     *            credentials of the user.
     * @return CertificationResult, a class that contains the following information: - *
     *         Int certificationResult: the result of the certifydevice process - * Byte[]
     *         devicePKCS12: an X509 certificate and private key that identify the
     *         certified device. The X509 certificate includes a unique device identifier
     *         and an enabling code that can be used for the selfVerify process.
     */
    public MXMObject certifydevice(String deviceFingerprint, String userToken);

    /**
     * This method is employed to disable the operation of a software device due to any
     * problem detected during the verifydevice process. The device will be deactivated in
     * the device where it is running.
     * 
     * @return The result of the disabledevice operation
     */
    public String disableDevice();

    /**
     * This method is employed to extract relevant features about the device (operating
     * system) and/or the software device that is requesting the authorisation of a user
     * action. This information can be used to be registered during the first usage
     * attempt of the device in the system so that it can be later checked for integrity.
     * 
     * @return an XML piece that represents the relevant features about the device
     *         (operating system) and/or the software device
     */
    public String estimateDeviceFingerprint();

    /**
     * This method is employed to locally check the integrity of a software device during
     * its whole life operation. The features regarding the device are used to compute a
     * security code and determine if it matches the enabling code received during
     * certification. In this way, this method determines whether the device has been
     * manipulated.
     * 
     * @param deviceFingerprint An XML piece that represents the relevant features about
     *            the device (operating system) and/or the software device.
     * @return "OK" if the operation was successful, error message if not.
     */
    public String selfVerifyDevice(String deviceFingerprint);

    /**
     * This method is employed to remotely check the integrity of a software device during
     * its whole life operation. Some features regarding the device (operating system)
     * and/or the software device that is requesting the authorisation of a user action
     * are sent to be compared to those registered in the certifydevice process.
     * 
     * @param deviceFingerprint An XML piece that represents the relevant features about
     *            the device (operating system) and/or the software device.
     * @param userToken An XML piece that represents the SAML token, which contains the
     *            credentials of the user.
     * @return The result of the verifydevice process
     */
    public String verifyDevice(String deviceFingerprint, String userToken);
}
