/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino
 * Contributor(s): Andreas Kohlos
 *
 * ***** END LICENSE BLOCK ***** */
package org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler;


import java.io.FileInputStream;

import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.w3c.dom.Document;


// TODO go over the javadocs
/**
 * Classes implementing this interface are responsible for providing a number of
 * functionalities related to digital (asymmetric) signatures, such as the generation and
 * verification of hash-values, issuance and verification of digital signatures.
 * 
 * @author Andreas Kohlos
 */
public interface SignatureManager
{
    /**
     * Generates a digital signature for the input data using the CONVERGENCE smart card.
     * The card currently uses a 'SHA256withRSA/PSS' signing algorithm.
     * 
     * @param message The data to be signed.
     * @param algorithm The algorithm used to sign the data. Is currently not supported.
     * @param appAlias Specifies the application domain which is currently used. Should be
     *            one of 1,2,3 or 4 where 1 is assigned to the Identity Provider domain
     *            and 2,3, or 4 for a Service Provider/application domain.
     * @param PIN The PIN to authorize the operation on the CONVERGENCE smart card.
     * @return The signature of the input message.
     * @throws SecurityEngineException
     * @throws MXMException
     */
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature generateCardSignature(
        byte[] message, String algorithm, String appAlias, String PIN)
        throws SecurityEngineException,MXMException;

    /**
     * Generates a hash value of the input data using the provided algorithm. Currently
     * uses only 'SHA-256' algorithm.
     * 
     * @param data The data to be hashed.
     * @param algorithm The algorithm used to calculate a hash value of the input data. Is
     *            currently not used.
     * @return The hash value.
     * @throws SecurityEngineException
     */
    public byte[] generateHash(byte[] data, String algorithm)
        throws SecurityEngineException;

    /**
     * Generates a hash value of the provided FileInputStream using the provided
     * algorithm.
     * 
     * @param fis The FileInputStream of the file containing the data to be hashed.
     * @param algorithm The algorithm used to calculate a hash value of the input data.
     * @return The hash value.
     * @throws SecurityEngineException
     */
    public byte[] generateHash(FileInputStream fis, String algorithm)
        throws SecurityEngineException;

    /**
     * Generates a digital signature of the input data using the provided algorithm and a
     * private key provided by the input key pair. Does currently only support RSA keys
     * and uses 'SHA256withRSA/PSSandMGF1' algorithm to sign.
     * 
     * @param message The data to be signed.
     * @param appAlias
     * @param sigPIN The PIN to access the signing function of the crypto manager.
     * @param keyPIN The PIN to retrieve the needed public key from the crypto manager.
     * @return The digital signature of the input message.
     * @throws SecurityEngineException
     */
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature generateSignature(
        byte[] message, String appAlias)
        throws SecurityEngineException;

    /**
     * Generates a digital signature of the input DOM object using the provided algorithm
     * and a private key provided by the input key pair. Does currently only support RSA
     * keys and uses 'SHA256withRSA/PSSandMGF1' algorithm to sign.
     * 
     * @param inputDomObject The DOM object which is to be signed.
     * @param appAlias
     * @param sigPIN The PIN to access the signing function of the crypto manager.
     * @param keyPIN The PIN to retrieve the needed public key from the crypto manager.
     * @return The digital signature of the input message.
     * @throws SecurityEngineException
     */
    public org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature generateSignature(
        Document inputDomObject, String appAlias)
        throws SecurityEngineException;

    /**
     * Verifies the hash value with the input data. Currently uses only 'SHA-256'
     * algorithm.
     * 
     * @param data The data on which the hash value has to be calculated.
     * @param hashValue The hash value which has to be verified against the one
     *            calculated.
     * @param algorithm The algorithm used to calculate the hash value of the data. Is
     *            currently not used.
     * @return True if the calculated value is equal to the generated one. Otherwise
     *         false.
     * @throws SecurityEngineException
     */
    public boolean verifyHash(byte[] data, byte[] hashValue, String algorithm)
        throws SecurityEngineException;

    // /**
    // * Verifies a signature on the input message by using the public key from the
    // provided
    // * X509 Certificate. Currently uses only 'SHA256withRSA/PSS' algorithm.
    // *
    // * @param message The data upon which the digital signature has been generated.
    // * @param signature The signature to be verified.
    // * @param algorithm The algorithm which the signature has been generated with. Is
    // * currently not used.
    // * @param x509Data The certificate which contains the public key to be used to
    // verify
    // * the signature.
    // * @return True if the signature is successfully verified. Otherwise false.
    // * @throws SecurityEngineException
    // */
    // public boolean verifySignature(byte[] message,
    // org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature signature,
    // String algorithm,
    // org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.X509Data x509Data)
    // throws SecurityEngineException;
    /**
     * Verifies a signature on the input message by using the provided public key.
     * Currently uses only 'SHA256withRSA/PSS' algorithm.
     * 
     * @param message The data upon which the digital signature has been generated.
     * @param signature The signature to be verified.
     * @param algorithm The algorithm which the signature has been generated with. Is
     *            currently not used.
     * @param pubKey The public key to be used to verify the signature.
     * @return True if the signature is successfully verified. Otherwise false.
     * @throws SecurityEngineException
     * @throws MXMEngineException
     */
    public boolean verifySignature(byte[] message,
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature signature)
        throws SecurityEngineException,MXMEngineException;

    /**
     * Verifies a signature on the input DOM document by using the provided Signature.
     * Currently the underlying algorithm is only 'SHA256withRSA/PSS'.
     * 
     * @param inputDomObject The original DOM object that has been signed.
     * @param signature The signature to be verified. Includes the public key and the
     *            signature ID. The method looks for that ID in the DOM document, strips
     *            the related signature and then verifies the stripped document against
     *            the signature.
     * @return True if the signature is successfully verified. Otherwise false.
     * @throws SecurityEngineException
     * @throws MXMEngineException
     */
    public boolean verifySignature(Document inputDomObject,
        org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.dsig.Signature signature)
        throws SecurityEngineException,MXMEngineException;
}
