/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the MPEG Multimedia Extensible Middleware (MXM) Project 
 *
 * The Initial Developer of the Original Code is CEDEO.net  
 * Portions created by the Initial Developer are Copyright (C) 2009
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione  
 * Contributor(s): Angelo Difino
 * Contributor(s): Andreas Kohlos
 *
 * ***** END LICENSE BLOCK ***** */
package org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler;


import java.security.KeyPair;
import java.security.PublicKey;
import java.sql.Date;

import javax.security.cert.X509Certificate;

import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.schemahandler.AuthenticateUserRequest;
import org.iso.mpeg.mxm.pEngine.authenticateuserPE.schemahandler.AuthenticateUserResponse;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.AuthnRequest;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.Response;


/**
 * Classes implementing this interface are used to store, retrieve and manage user
 * information in a local database. Currently MySql is supported. MXMConfiguration.xml
 * files allow to set up the MySql settings for your local installation.
 * 
 * @author Andreas Kohlos
 */
public interface UserDataBaseManager
{
    /**
     * @param userID
     */
    public void deleteUser(String userID);

    /**
     * Returns the x509 root certificate from the database.
     * 
     * @return java.security.cert.X509Certificate
     * @throws SecurityEngineException
     */
    public X509Certificate getIPRootCertificate() throws SecurityEngineException;

    /**
     * Returns the KeyPair from the database.
     * 
     * @return
     * @throws SecurityEngineException
     */
    public KeyPair getIssuerKeyPair() throws SecurityEngineException;

    /**
     * @return
     * @throws SecurityEngineException
     */
    public X509Certificate getIssuerRootCertificate() throws SecurityEngineException;

    /**
     * Returns the next highest id from the table users.
     * 
     * @return
     */
    public int getNextUserID();

    /**
     * @param hashCode
     * @return
     */
    public PublicKey getPublicKey(String hashCode);

    /**
     * @param publicKeyObj
     * @return
     */
    public String getUserId(PublicKey publicKeyObj);

    /**
     * Sets the status of a user to active.
     * 
     * @param userID
     */
    public void grantUser(String userID);

    /**
     * Checks if the user is created.
     * 
     * @param userID
     * @return
     */
    public boolean isUserCreated(String userID);

    /**
     * Checks if the user is revoked.
     * 
     * @param userID
     * @return
     */
    public boolean isUserRevoked(String userID);

    /**
     * @param key
     * @param firstName
     * @param lastName
     * @param birthday
     * @param country
     * @param locality
     * @param organization
     * @return
     * @throws SecurityEngineException
     */
    public int registerUser(PublicKey key, String firstName, String lastName,
        Date birthday, String country, String locality, String organization)
        throws SecurityEngineException;

    /**
     * @param userCertificate
     * @param firstName
     * @param lastName
     * @param birthday
     * @param country
     * @param locality
     * @param organization
     * @return
     * @throws SecurityEngineException
     */
    int registerUser(X509Certificate userCertificate, String firstName, String lastName,
        Date birthday, String country, String locality, String organization)
        throws SecurityEngineException;

    /**
     * Sets the status of a user to inactive.
     * 
     * @param userID
     * @param callbackReason
     */
    public void revokeUser(String userID, String callbackReason);

    /**
     * @param iPRootCertificate
     * @param sPRootCertificate
     * @param issuerKeyPair
     * @throws SecurityEngineException
     */
    public void storeIssuerCredentials(X509Certificate iPRootCertificate,
        X509Certificate sPRootCertificate, KeyPair issuerKeyPair)
        throws SecurityEngineException;

    /**
     * @param PIN
     * @param userCertificates
     * @throws SecurityEngineException
     */
    public void storeUserCertsToCard(String PIN, X509Certificate[] userCertificates)
        throws SecurityEngineException;


    Response authenticateUser(AuthnRequest authenticateUserRequest)
        throws MXMEngineException;
}
