/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-20012
 * All Rights Reserved.
 *
 * The Initial Developer: Angelo Difino
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.baseprotocol.schema;

import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.io.OutputStream;
import java.io.StringReader;
import java.io.StringWriter;
import java.lang.reflect.Method;

import javax.xml.bind.JAXBContext;
import javax.xml.bind.JAXBElement;
import javax.xml.bind.JAXBException;
import javax.xml.bind.Marshaller;
import javax.xml.bind.Unmarshaller;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.log4j.Logger;
import org.apache.xerces.dom.DocumentImpl;
import org.apache.xerces.parsers.DOMParser;
import org.apache.xml.serialize.OutputFormat;
import org.apache.xml.serialize.XMLSerializer;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.MXMEngineResponse;
import org.iso.mpeg.mxm.core.MXMObject;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.utils.SchemaNamespacePrefixMapperImpl;
import org.w3c.dom.Document;
import org.xml.sax.InputSource;
import org.xml.sax.SAXException;

/**
 * @author dodo
 * @author Angelo Difino
 *
 */
public abstract class SuperAbstractLevel1 
{
	/** The schema type */
	protected Object type;
	
	/** The factory. */
	protected Object factory;
	
	/** The m. */
	protected Marshaller m = null;
	
	/** The jc. */
	protected JAXBContext jc = null;

	/** The logger. */
	protected Logger logger = Logger.getLogger(this.getClass());
	
	/**
	 * CONSTRUCTORS----------------------------------------------------------------------------------------------------------------------------------
	 */
	public SuperAbstractLevel1(Object type) throws MXMEngineException {	
		try {
			this.setLevel0(type);
			this.config();
		} catch (JAXBException e) {
			//e.printStackTrace();
			this.throwEngineSpecificException(e.getMessage(),e);
		}
		this.initialise();
	}	

	public SuperAbstractLevel1(Object type, JAXBContext context) throws MXMEngineException {	
		try {
			this.setLevel0(type);
			this.config(context);
		} catch (JAXBException e) {
			this.throwEngineSpecificException(e.getMessage(),e);
		}
		this.initialise();
	}	

	private void initialise() throws MXMEngineException {
		try {
			m = jc.createMarshaller();
			m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);	
			m.setProperty("com.sun.xml.bind.namespacePrefixMapper", SchemaNamespacePrefixMapperImpl.getInstance());
		} catch (JAXBException e) {
			String message = "A JAXBException occurred. The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			this.throwEngineSpecificException(message,e);
		}
	}
	
	protected void config() throws MXMEngineException, JAXBException{
		this.config(null);
	}
	
	/**
	 * CREATORS----------------------------------------------------------------------------------------------------------------------------------
	 */
	
	/**
	 * Generate a java.util.File representation, basing on the passed argument
	 * 
	 * @params f the java.util.File where the representation is saved
	 * 
	 * @throws MXMEngineException
	 */
	public void createFile(File f) throws MXMEngineException {								//file
		try { 
			FileOutputStream outputFile = new FileOutputStream(f);
			this.marshall(outputFile);
			outputFile.flush();
			outputFile.close();
			logger.debug("... closed the FileOutputStream");
		} catch (IOException e) {
			String message = "Unable to write XML data to file: " + f.getAbsolutePath() + ". The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			throw new MXMEngineException(message);
		} catch (JAXBException e) {
			String message = "A JAXBException occurred. The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			throw new MXMEngineException(message);
		} catch (Exception e) {
			String message = "A Java reflection exception occurred. The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			throw new MXMEngineException(message);
		}
	}

	/**
	 * Generate a java.util.String/org.w3c.dom.Document/level0/JAXBelement representation, basing on the passed argument
	 * 
	 * @params objectType the String specifing the format of the returned representation (can be on of 
	 * java.util.String/org.w3c.dom.Document/level0/JAXBelement)
	 * 
	 * @return The java.util.String/org.w3c.dom.Document/level0/JAXBelement representation, basing on the passed argument
	 * 
	 * @throws MXMEngineException
	 */
	public MXMObject createObject(String objectType) throws MXMEngineException {
		if (String.class.getCanonicalName().equals(objectType))								//string
			return new MXMAdapter(this.generateString());
		else if (Document.class.getCanonicalName().equals(objectType))						//document
			return new MXMAdapter(this.generateDOM());
		else if (getLevel0().getClass().getName().equals(objectType))						//level0
			return new MXMAdapter(this.getLevel0());
		else if ( JAXBElement.class.getCanonicalName().equals(objectType) )					//JAXB element
			return new MXMAdapter(createJAXBElement());
		else if ( Byte.class.getCanonicalName().equals(objectType) )						//Byte rapresentation element
			return new MXMAdapter(this.generateByteRepresenation());
		else
		{
			try
			{
				if (Class.forName(objectType).isInstance(getLevel0()))
				{
					return new MXMAdapter(this.getLevel0());								//Superclass
				}
			}
			catch (Exception ex)
			{
				ex.printStackTrace();
			}
		}
		
		throw new MXMEngineException("Unable to return an MXMObject of type " + objectType);	
	}

	/**
	 * Generate java.util.String representation
	 * 
	 * @return The java.util.String representation
	 * 
	 * @throws MXMEngineException
	 */
	private String generateString() throws MXMEngineException {
		StringWriter out = new StringWriter(); 
		Document doc = this.generateDOM();
		XMLSerializer serializer = new XMLSerializer(out, new OutputFormat(doc));
		try {
	        serializer.serialize(doc); 
		} catch (IOException e) {
			String message = "Unable to write XML data to a string. The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			throw new MXMEngineException(message);
		} 
        return out.toString(); 
	}

	/**
	 * Generate a org.w3c.dom.Document representation
	 * 
	 * @return The org.w3c.dom.Document representation
	 * 
	 * @throws MXMEngineException
	 */
	private Document generateDOM() throws MXMEngineException {
		Document doc = new DocumentImpl();
		JAXBElement<?> acElement = null;
		
		try
		{
			acElement = createJAXBElement();
		}
		catch(Exception ex)
		{
			ex.printStackTrace();
		}
				
		try {
			if( acElement != null )
				m.marshal(acElement, doc);
			else
				m.marshal(getLevel0(), doc);
		} catch (JAXBException e) {
			String message = "A JAXBException occurred. The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			throw new MXMEngineException(message);
		}
		return doc;
	}
	
	
	/**
	 * Generate Byte Represenation
	 */
	public byte[] generateByteRepresenation() throws MXMEngineException 
	{
		ByteArrayOutputStream outputStream = new ByteArrayOutputStream();
		try {
			m.marshal(createJAXBElement(), outputStream);
			return outputStream.toByteArray();
		} catch (JAXBException e) {
			e.printStackTrace();
		}
		return null;
	}
	
	//tocheck
	private void marshall(OutputStream os) throws  MXMEngineException, JAXBException{
		try {
							//			Object level1 = this.createJAXBLevel0(getFactory(), getLevel0Class(), getLevel0());
			JAXBElement<?> myJaxElement = createJAXBElement();  // XXX this is needed right ?
			m.marshal(myJaxElement, os);
		} catch (MXMEngineException e) { // XXX exception must be killed
			if(e.getCause() instanceof NoSuchMethodException){
				m.marshal(getLevel0(), os); 
			}
			else
				throw e;
		}
	}
	
//	//tocheck
//	private Object createLevel0(Object factory,Class<?> level0Class) throws MXMEngineException { // XXX remove
//		String methodName = "create"+level0Class.getSimpleName();
//		try{
//			Object level0 =  null;
//			Method level0CreationMethod = factory.getClass().getMethod(methodName);		
//			level0 = level0CreationMethod.invoke(factory);
//			
//			return level0;
//		} catch (Exception e) {
//			throw new MXMEngineException("It was impossibe to call the method "+methodName,e);
//		}
//		
//	}

//	//tocheck
//	private JAXBElement<?> createJAXBLevel0(Object jaxbFactory, Class<?> level0Class, Object level0) throws MXMEngineException { // XXX remove
//		String methodName = "create"+this.getClass().getSimpleName();
//		try{
//			
//			Method level0CreationMethod = jaxbFactory.getClass().getMethod(methodName, level0Class );
//			return (JAXBElement<?>) level0CreationMethod.invoke(jaxbFactory, level0);
//		} catch (Exception e) {
//			throw new MXMEngineException("It was impossibe to call the method "+methodName,e);
//		}
//	}
	
	/**
	 * PARSER----------------------------------------------------------------------------------------------------------------------------------
	 */
	
	/**
	 * Parse the passed representation
	 * 
	 * @params object is the representation to parse (can be java.util.File/java.util.String/org.w3c.dom.Document/level0)
	 * 
	 * @return MXMEngineResponse.SUCCESS if the parsing was completed well
	 * @return MXMEngineResponse.FAILURE if some trouble were reaching parsing the representation
	 * 
	 * @throws MXMEngineException
	 */
	public MXMEngineResponse parse(MXMObject object) throws MXMEngineException {
		if (object.getContent() instanceof InputStream)													//inputstream
			return parse((InputStream) object.getContent());
		else if (object.getContentType().equals(File.class.getCanonicalName()))							//file
			return parse((File) object.getContent());
		else if (object.getContentType().equals(String.class.getCanonicalName()))						//string
			return parse((String) object.getContent());
		else if (object.getContentType().equals(Document.class.getCanonicalName()))						//document
			return parse((Document) object.getContent());
		else if(object.getContentType().equals(this.getLevel0().getClass().getName()))					//level0
			return parse(object.getContent());
		else if(object.getContentType().equals(JAXBElement.class.getCanonicalName()))					//jaxb
	 		return parse(((JAXBElement)object.getContent()).getValue());
		else
		{
			Class level0class = getLevel0().getClass();
			if (level0class.isInstance(object.getContent()))
			{
				return parse(object.getContent());														// subclass
			}
		}
		
		return MXMEngineResponse.FAILURE;
	}

	/**
	 * Parse the passed File
	 * 
	 * @param aFile the File to parse 
	 * 
	 * @return MXMEngineResponse.SUCCESS if the parsing was completed well
	 * @return MXMEngineResponse.FAILURE if some trouble were reaching parsing the representation
	 * 
	 * @throws MXMEngineException
	 */
	private MXMEngineResponse parse(File aFile) throws MXMEngineException {
		try {
			Unmarshaller u = jc.createUnmarshaller();
			DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
			domFactory.setNamespaceAware(true);
			DocumentBuilder builder = domFactory.newDocumentBuilder();
			Document aDOM = builder.parse(aFile);
			Object unmarshal = u.unmarshal(aDOM);
			if(unmarshal instanceof JAXBElement<?>){
				this.setJAXBElementValue((JAXBElement<?>) unmarshal);
			}
			else
				this.setLevel0(unmarshal);
			
			return MXMEngineResponse.SUCCESS;
		} catch (ParserConfigurationException e) {
			String message = "A ParserConfigurationException occurred. The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			throw new MXMEngineException(message);
		} catch (IOException e) {
			String message = "The inputStream cannot be read. The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			throw new MXMEngineException(message);
		} catch (JAXBException e) {
			String message = "A JAXBException occurred. The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			throw new MXMEngineException(message);
		} catch (SAXException e) {
			String message = "A SAXException occurred. The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			throw new MXMEngineException(message);
		}
	}
	
	/**
	 * Parse the passed InputStream
	 * 
	 * @param in the InputStream to parse 
	 * 
	 * @return MXMEngineResponse.SUCCESS if the parsing was completed well
	 * @return MXMEngineResponse.FAILURE if some trouble were reaching parsing the representation
	 * 
	 * @throws MXMEngineException
	 */
	private MXMEngineResponse parse(InputStream in) throws MXMEngineException {
		try {
			Unmarshaller u = jc.createUnmarshaller();
			DocumentBuilderFactory domFactory = DocumentBuilderFactory.newInstance();
			domFactory.setNamespaceAware(true);
			DocumentBuilder builder = domFactory.newDocumentBuilder();
			Document aDOM = builder.parse(in);
			Object unmarshal = u.unmarshal(aDOM);
			if(unmarshal instanceof JAXBElement<?>){
				this.setJAXBElementValue((JAXBElement<?>) unmarshal);
			}
			else
				this.setLevel0(unmarshal);
			
			return MXMEngineResponse.SUCCESS;
		} catch (ParserConfigurationException e) {
			String message = "A ParserConfigurationException occurred. The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			throw new MXMEngineException(message);
		} catch (IOException e) {
			String message = "The inputStream cannot be read. The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			throw new MXMEngineException(message);
		} catch (JAXBException e) {
			String message = "A JAXBException occurred. The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			throw new MXMEngineException(message);
		} catch (SAXException e) {
			String message = "A SAXException occurred. The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			throw new MXMEngineException(message);
		}
	}

	/**
	 * Parse the passed String
	 * 
	 * @param aString the String to parse
	 * 
	 * @return MXMEngineResponse.SUCCESS if the parsing was completed well
	 * @return MXMEngineResponse.FAILURE if some trouble were reaching parsing the representation
	 * 
	 * @throws MXMEngineException
	 */
	private MXMEngineResponse parse(String aString) throws MXMEngineException {
		try {
			DOMParser domParser = new DOMParser(); 
	        domParser.parse(new InputSource(new StringReader(aString))); 
	        Document aDOM = domParser.getDocument(); 
			Unmarshaller u = jc.createUnmarshaller();
			Object unmarshal = u.unmarshal(aDOM);
			if(unmarshal instanceof JAXBElement<?>){
				this.setJAXBElementValue((JAXBElement<?>) unmarshal);
			}
			else
				this.setLevel0(unmarshal);
			return MXMEngineResponse.SUCCESS;
		} catch (JAXBException e) {
			String message = "A JAXBException occurred. The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			throw new MXMEngineException(message);
		} catch (IOException e) {
			String message = "Unable to parse the XML string.";
			logger.error(message, e);
			throw new MXMEngineException(message);
		} catch (SAXException e) {
			String message = "A SAXException occurred. The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			throw new MXMEngineException(message);
		}
	}

	/**
	 * Parse the passed Document
	 * 
	 * @param aDom the Document to parse
	 * 
	 * @return MXMEngineResponse.SUCCESS if the parsing was completed well
	 * @return MXMEngineResponse.FAILURE if some trouble were reaching parsing the representation
	 * 
	 * @throws MXMEngineException
	 */
	private MXMEngineResponse parse(Document aDom) throws MXMEngineException {
		try {
			Unmarshaller u = jc.createUnmarshaller();
			Object unmarshal = u.unmarshal(aDom);
			if(unmarshal instanceof JAXBElement<?>){
				this.setJAXBElementValue((JAXBElement<?>) unmarshal);
			}
			else
				this.setLevel0(unmarshal);
			return MXMEngineResponse.SUCCESS;
		} catch (JAXBException e) {
			String message = "A JAXBException occurred. The detailed exception is: " + e.getMessage();
			logger.error(message, e);
			throw new MXMEngineException(message);
		}
	}
	

	/**
	 * Parse the passed level0
	 * 
	 * @param object the level0 class to parse
	 * 
	 * @return MXMEngineResponse.SUCCESS if the parsing was completed well
	 * @return MXMEngineResponse.FAILURE if some trouble were reaching parsing the representation
	 * 
	 * @throws MXMEngineException
	 */
	private MXMEngineResponse parse(Object object) throws MXMEngineException {
		this.setLevel0(object);
		return MXMEngineResponse.SUCCESS;
	}

//	//tocheck
//	protected Class<?> getLevel0Class(String classPackage) throws MXMEngineException { // XXX remove
//		if(classPackage == null || "".equals(classPackage)){
//			classPackage = this.getClass().getPackage().getName();
//		}
//		String className = classPackage +"."+this.getLevel0ClassName();
//		try {
//			return Class.forName(className);
//		} catch (ClassNotFoundException e) {
//			throw new MXMEngineException("It was impossible to instanciate the class "+className,e);
//		}
//	}

//	//tocheck
//	protected Class<?> getLevel0Class() throws MXMEngineException{ // XXX remove
//		return this.getLevel0Class(null);
//	}
	
//	//tocheck
//	protected String getLevel0ClassName() { // XXX remove
//		return this.getClass().getSimpleName();
//	}
	
	
	/**
	 * EXCEPTIONS----------------------------------------------------------------------------------------------------------------------------------
	 */
	
	//remove setexception if this routine become abstract 
//	private void throwEngineSpecificException(String message, Exception exception) throws MXMEngineException{ // XXX make it abstract ? or make a standard exception
//		try{
//			Class<?> classException = setSpecificException().getClass();
//			Constructor<?> constructor = classException.getConstructor(new Class[]{String.class,Exception.class});
//			throw (MXMEngineException) constructor.newInstance(new Object[]{message,exception});
//		} catch(Exception e){
//			logger.error(e.getMessage(),e);
//			throw new MXMInvalidConfigurationException(e.getMessage());
//		}
////		throw new MXMEngineException(message,e);
//	}
	
//	/**
//	 *  Method to be extended if I need to throw more specialized exceptions
//	 * @param message
//	 * @throws MXMEngineException
//	 */
//	private void throwEngineSpecificException(String message) throws MXMEngineException{ // XXX remove
//		try{
//			Class<?> classException = setSpecificException().getClass();
//			Constructor<?> constructor = classException.getConstructor(new Class[]{String.class});
//			throw (MXMEngineException) constructor.newInstance(new Object[]{message});
//		} catch(Exception e){
//			logger.error(e.getMessage(),e);
//			throw new MXMInvalidConfigurationException(e.getMessage());
//		}
////		throw new MXMEngineException(message);
//	}


	/**
	 * UTILS----------------------------------------------------------------------------------------------------------------------------------
	 * @throws MXMEngineException 
	 */
	
//	protected JAXBElement<?> createJAXBElement()
//	{
//		String methodName = "create"+getLevel0().getClass().getSimpleName();
//		try{
//			Object level0 =  null;
//			
//			Method creationMethod = getFactory().getClass().getMethod(methodName, null); // XXX is this a test ???
//			level0 = creationMethod.invoke(getFactory(),null);
//			
//			Object jaxbElement =  null;
//			
//			String new_methodName=methodName.substring(0,methodName.indexOf("Type"));
//			
//			Method jaxbElementCreationMethod = getFactory().getClass().getMethod(new_methodName, getLevel0().getClass());
//			jaxbElement = jaxbElementCreationMethod.invoke(getFactory(),getLevel0());
//			
//			return (JAXBElement<?>) jaxbElement;
//		} catch (Exception e) {
////			throw new MXMEngineException("It was impossibe to call the method "+methodName,e); 
//		}
//		
//		return null;
//	}
	
	protected JAXBElement<?> createJAXBElement() throws MXMEngineException
	{
		String methodName = new String();
		try
		{
			methodName = "create"+getLevel0().getClass().getSimpleName();
			
			int indexOfType = methodName.indexOf("Type");
			
			if (indexOfType!=-1)
			{
				methodName = methodName.substring(0,indexOfType);
			}			
			
			Method level0CreationMethod = getFactory().getClass().getMethod(methodName, getLevel0().getClass() );
			   
			return (JAXBElement<?>) level0CreationMethod.invoke(getFactory(), getLevel0());
			
			
//			Method[] methods = getFactory().getClass().getMethods();
//			for (Method method : methods)
//			{
//				if (method.getName().equals(methodName))
//				{
//					System.out.println("Calling method " + method.getName());
//					JAXBElement<?> jaxbElement = (JAXBElement<?>)method.invoke(getFactory(),getLevel0());
//					return jaxbElement;
//				}
//			}			
//			throw new MXMEngineException("It was impossibe to find the method "+methodName);
		}
		catch (Exception e) 
		{
			throw new MXMEngineException("It was impossibe to call the method "+methodName,e); 
		}
	}


	protected void setJAXBElementValue(JAXBElement<?> anElement)
	{
		setLevel0(anElement.getValue());
	}
	
	protected Object getFactory() {
		return this.factory;
	}
	
	protected void setLevel0(Object level0) 
	{
		this.type = level0;
	}

	protected Object getLevel0() {
		return type;
	}
	
	/**
	 * ABSTRACT----------------------------------------------------------------------------------------------------------------------------------
	 */
	
//	protected abstract Object getFactory(); // XXX remove ???

	protected abstract void config(JAXBContext context) throws MXMEngineException, JAXBException;
	
	protected abstract void throwEngineSpecificException(String message, Exception exception) throws MXMEngineException;
	
//	protected abstract Object setSpecificException(); // XXX remove ???
	
//	protected abstract void setJAXBElementValue(JAXBElement<?> anElement);
//	protected abstract JAXBElement<?> createJAXBElement();

//	protected abstract void setLevel0(Object level0); XXX remove ???
//	protected abstract Object getLevel0(); XXX remove ???
	
}
