/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2012
 * All Rights Reserved.
 *
 * The Initial Developer: Angelo Difino
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm;

import java.io.File;
import java.io.FileNotFoundException;

import javax.xml.bind.JAXBException;

import junit.framework.TestCase;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.baseprotocol.schema.ContentEntityType;
import org.iso.mpeg.mxm.core.MXMAdapter;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.DIDLType;
import org.iso.mpeg.mxm.engine.digitalitemTE.schema.didl.SuperDidlAbstractLevel1;
import org.iso.mpeg.mxm.engine.eventreportTE.schema.er.ERRType;
import org.iso.mpeg.mxm.engine.eventreportTE.schema.er.ERType;
import org.iso.mpeg.mxm.engine.eventreportTE.schema.er.SuperErAbstractLevel1;
import org.iso.mpeg.mxm.engine.ipmpTE.schema.ipmpinfo.IPMPGeneralInfoDescriptorType;
import org.iso.mpeg.mxm.engine.ipmpTE.schema.ipmpinfo.SuperIpmpinfoAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.SuperProtocolEngineAbstractLevel1;
import org.iso.mpeg.mxm.engine.protocol.authenticatecontentPE.schema.authenticatecontentprotocol.AuthenticateContentRequestType;
import org.iso.mpeg.mxm.engine.protocol.createcontentPE.schema.createcontentprotocol.CreateContentRequestType;
import org.iso.mpeg.mxm.engine.protocol.identifycontentPE.schema.identifycontentprotocol.IdentifyContentRequestType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.KeyInfoType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SignatureType;
import org.iso.mpeg.mxm.engine.securityTE.schema.dsig.SignatureValueType;

/**
 * @author Angelo Difino
 *
 */
public class DataobjectTest extends TestCase {

	private Logger logger = Logger.getLogger(DataobjectTest.class);
	
	private final String TEMP_FOLDER = System.getProperty("java.io.tmpdir") + File.separator;
	
	private final String OUTPUT_DI_FILE_NAME = TEMP_FOLDER + "mytestDI.xml";
	private final String SOURCE_DI_FILE_NAME = "testDI.xml";
	
	private final String OUTPUT_ER_FILE_NAME = TEMP_FOLDER + "mytestER.xml";
	private final String SOURCE_ER_FILE_NAME = "testER.xml";
	
	private final String OUTPUT_ERR_FILE_NAME = TEMP_FOLDER + "mytestERR.xml";
	private final String SOURCE_ERR_FILE_NAME = "testERR.xml";
	
	private final String OUTPUT_IPMPGD_FILE_NAME = TEMP_FOLDER + "mytestIPMPGD.xml";
	private final String SOURCE_IPMPGD_FILE_NAME = "testIPMPGeneralInfoDescriptor.xml";	
	
	private final String OUTPUT_IDENTIFY_CONTENT_REQUEST_FILE_NAME = TEMP_FOLDER + "mytestIdentifyContentRequest.xml";
	private final String SOURCE_IDENTIFY_CONTENT_REQUEST_FILE_NAME = "testIdentifyContentRequest.xml";
	
	private final String OUTPUT_AUTHENTICATE_CONTENT_REQUEST_FILE_NAME = TEMP_FOLDER + "mytestAuthenticateContentRequest.xml";
	private final String SOURCE_AUTHENTICATE_CONTENT_REQUEST_FILE_NAME = "testAuthenticateContentRequest.xml";
	
	private final String OUTPUT_CREATE_CONTENT_REQUEST_FILE_NAME = TEMP_FOLDER + "mytestCreateContentRequest.xml";
	private final String SOURCE_CREATE_CONTENT_REQUEST_FILE_NAME = "testCreateContentRequest.xml";
	

	@Override
	protected void setUp() throws Exception {
	}
		
	public void testDataObject() throws MXMConfigurationException, FileNotFoundException, JAXBException, MXMEngineException {
		logger.info("\n\n---------- testDataObject ----------");
		
		logger.info("\n\n---------- DI ----------");
		
		File didl_file_in=new File(getClass().getClassLoader().getResource(SOURCE_DI_FILE_NAME).getFile());
		File didl_file_out=new File(OUTPUT_DI_FILE_NAME);

		SuperDidlAbstractLevel1 didl=new SuperDidlAbstractLevel1((Object)new DIDLType());	
		
		didl.parse(new MXMAdapter(didl_file_in));
		didl.createFile(didl_file_out);	
				
		logger.info("\n\n---------- ER/ERR ----------");
		
		File er_file_in=new File(getClass().getClassLoader().getResource(SOURCE_ER_FILE_NAME).getFile());
		File er_file_out=new File(OUTPUT_ER_FILE_NAME);

		SuperErAbstractLevel1 er=new SuperErAbstractLevel1((Object)new ERType());	
		
		er.parse(new MXMAdapter(er_file_in));
		er.createFile(er_file_out);	
		
		File err_file_in=new File(getClass().getClassLoader().getResource(SOURCE_ERR_FILE_NAME).getFile());
		File err_file_out=new File(OUTPUT_ERR_FILE_NAME);

		SuperErAbstractLevel1 err=new SuperErAbstractLevel1((Object)new ERRType());	
		
		err.parse(new MXMAdapter(err_file_in));
		err.createFile(err_file_out);	
		
		logger.info("\n\n---------- IPMPGD ----------");
		
		File ipmpgd_file_in=new File(getClass().getClassLoader().getResource(SOURCE_IPMPGD_FILE_NAME).getFile());
		File ipmpgd_file_out=new File(OUTPUT_IPMPGD_FILE_NAME);

		SuperIpmpinfoAbstractLevel1 ipmpgd=new SuperIpmpinfoAbstractLevel1((Object)new IPMPGeneralInfoDescriptorType());	
		
		ipmpgd.parse(new MXMAdapter(ipmpgd_file_in));
		ipmpgd.createFile(ipmpgd_file_out);	
		
		logger.info("\n\n---------- IDENTIFY CONTENT PROTOCOL ----------");
		
		SuperProtocolEngineAbstractLevel1 protocol = new SuperProtocolEngineAbstractLevel1(new IdentifyContentRequestType());
		
		File identifyContentRequestFileIn = new File(
									getClass().getClassLoader().getResource(SOURCE_IDENTIFY_CONTENT_REQUEST_FILE_NAME).getFile());
		File identifyContentRequestFileOut = new File(OUTPUT_IDENTIFY_CONTENT_REQUEST_FILE_NAME);
		
		protocol.parse(new MXMAdapter(identifyContentRequestFileIn));
		
		protocol.createFile(identifyContentRequestFileOut);
		
		logger.info("\n\n---------- CREATE CONTENT PROTOCOL ----------");
		
		SuperProtocolEngineAbstractLevel1 protocol_create_content = new SuperProtocolEngineAbstractLevel1(new CreateContentRequestType());
		
		File cContentRequestFileIn = new File(
									getClass().getClassLoader().getResource(SOURCE_CREATE_CONTENT_REQUEST_FILE_NAME).getFile());
		File cContentRequestFileOut = new File(OUTPUT_CREATE_CONTENT_REQUEST_FILE_NAME);
		
		protocol_create_content.parse(new MXMAdapter(cContentRequestFileIn));
		
		protocol_create_content.createFile(cContentRequestFileOut);
		
		File authContentRequestFileIn = new File(
				getClass().getClassLoader().getResource(SOURCE_AUTHENTICATE_CONTENT_REQUEST_FILE_NAME).getFile());
        File authContentRequestFileOut = new File(OUTPUT_AUTHENTICATE_CONTENT_REQUEST_FILE_NAME);
        
        
        System.out.println("==== AUTHENTICATE CONTENT ====");
        /*
         * Authenticate Content Test
         */
        protocol = new SuperProtocolEngineAbstractLevel1(new AuthenticateContentRequestType());
        
//        AuthenticateContentRequestType authenticateContentRequestType = new AuthenticateContentRequestType();
//        DIDLType prevDIDL = (DIDLType) didl.createObject(DIDLType.class.getName()).getContent();
//        ContentEntityType contentEntity = new ContentEntityType();
//        contentEntity.setDIDL(prevDIDL);
//        authenticateContentRequestType.setAuthenticationEntity(contentEntity);
//        SignatureType signatureType = new SignatureType();
//        KeyInfoType keyInfoType = new KeyInfoType();
//        keyInfoType.setId("keyid");
//        SignatureValueType signatureValueType = new SignatureValueType();
//        signatureValueType.setId("value");
//        signatureValueType.setValue(String.valueOf("value").getBytes());
//        signatureType.setId("signatureid");
//        signatureType.setKeyInfo(keyInfoType);
//        signatureType.setSignatureValue(signatureValueType);
//		  authenticateContentRequestType.setEntitySignature(signatureType);
//		
//		  protocol.parse(new MXMAdapter(authenticateContentRequestType));
		        
        protocol.parse(new MXMAdapter(authContentRequestFileIn));
		protocol.createFile(authContentRequestFileOut);
		
		logger.info("\n\n---------- DONE!");
	}
}