package org.iso.mpeg.mxm.eventreportTE.technologyhandler.kernel.keeper;

import org.apache.commons.io.FileUtils;
import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.core.*;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.tEngine.eventreportTE.exception.EventReportException;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERRKeeper;
import org.iso.mpeg.mxm.tEngine.relTE.RELEngine;

import java.io.File;
import java.io.IOException;
import java.util.Map;

/**
 * The default implementation of the <tt>ERRKeeper</tt> interface.
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro"> Bogdan Ardeleanu</a>
 * Date: 8/31/11
 * Time: 6:25 PM
 * <p/>
 * UTI Systems, TIC
 */
public class DefaultERRKeeper implements ERRKeeper {
    private static final String XML_EXT = ".xml";

    // <K, V> = <DIDLId, ERRId>
    // we may safely assume a one-to-one relationship for the demo
    private final GenericKeeper<String, String> keeperDIDL;

    private static final Logger logger = Logger.getLogger(DefaultERKeeper.class);

    private final String rootERRStorageLocation;

    private RELEngine relEngine;

    private void setupMXMEngineDependencies() throws MXMConfigurationException, MXMEngineException {
        MXM mxm = MXM.getInstance();
//        mxm.listEnginesAndProperties();

        relEngine = (RELEngine) mxm.getDefaultEngine(MXMEngineName.RELTE.toString());
    }


    public DefaultERRKeeper(String repositoryKey, String rootERRStorageLocation) {
        this.rootERRStorageLocation = rootERRStorageLocation;

        try {
            setupMXMEngineDependencies();
        } catch (Exception e) {
            throw new RuntimeException("Connot setup MXMEngine dependencies", e);
        }

        if (null == repositoryKey || repositoryKey.isEmpty()) {
            throw new IllegalArgumentException("repositoryKey cannot be null or empty!");
        }

        try {
            File f = new File(rootERRStorageLocation);
            if (!f.exists()) {
                FileUtils.forceMkdir(f);
                logger.info("rootERRStorageLocation created to: " + rootERRStorageLocation);
            }
        } catch (IOException e) {
            throw new RuntimeException("Can not create rootERRStorageLocation to: " + rootERRStorageLocation);
        }

        keeperDIDL = new GenericKeeper<String, String>(
            rootERRStorageLocation + File.separatorChar + "mapping_" + repositoryKey + XML_EXT
        );
    }

    @Override
    public MXMEngineResponse storeERR(ERR err, String DIDLId) throws EventReportException {
//    	ERR err = (ERR) errMXMObject.getContent();

        String errIdentifier = err.getERRIdentifier();
        keeperDIDL.put(DIDLId, errIdentifier);
        try {
            err.createFile(new File(buildFilePath(errIdentifier)));
        } catch (MXMException e) {
            throw new EventReportException("Trouble creating File in storeERRForDIDL", e);
        }

        return MXMEngineResponse.SUCCESS;
    }

    @Override
    public ERR getERR(String errId) throws EventReportException {
        String absFilePath = buildFilePath(errId);

        try {
            ERR er = new org.iso.mpeg.mxm.eventreportTE.ERR(relEngine);
            er.parse(new MXMAdapter(new File(absFilePath)));
            return er;
        } catch (MXMEngineException e) {
            throw new EventReportException("Cannot get ERR object from location: " + absFilePath, e);
        }
    }

    @Override
    public ERR getERRByDIDLId(String DIDLId) throws EventReportException {
        String errIdentifier = getERRIdByDIDLId(DIDLId);
        if (null == errIdentifier) {
//            throw new EventReportException("No ERR found for DIDL with ID = " + DIDLId);
        	logger.error("No ERR found for DIDL with ID = " + DIDLId);
        	return null;
        }

        return getERR(errIdentifier);
    }


    @Override
    public String getERRIdByDIDLId(String DIDLId) throws EventReportException {
        return keeperDIDL.get(DIDLId);
    }

    @Override
    public String getDIDLIdOfERR(String errId) throws EventReportException {
        Map<String, String> mapping = keeperDIDL.getMapping();
        for (String key : mapping.keySet()) {
            if (mapping.get(key).equals(errId)) {
                return key;
            }
        }

        return null;
    }

    private String buildFilePath(String fName) {
        fName = fName.replaceFirst(IDGenerator.ERR_IDENTIFIER_PREFIX, "");
        return rootERRStorageLocation + File.separatorChar + fName + XML_EXT;
    }
}
