/** ***** BEGIN LICENSE BLOCK *****
 *
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.orchestrator.requestevent;

import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.BasicFailureCode;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ContentEntity;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolFailure;
import org.iso.mpeg.mxm.pEngine.requesteventPE.RequestEventEngine;
import org.iso.mpeg.mxm.pEngine.requesteventPE.schemahandler.RequestEventRequest;
import org.iso.mpeg.mxm.pEngine.requesteventPE.schemahandler.RequestEventResponse;
import org.iso.mpeg.mxm.pEngine.requesteventPE.schemahandler.RequestEventSchemaHandler;
import org.iso.mpeg.mxm.pEngine.requesteventPE.schemahandler.RequestEventSuccess;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERKeeper;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERRKeeper;

import java.util.List;

/**
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro"> Bogdan Ardeleanu</a>, Angelos-Christos Anadiotis
 * Date: 8/29/11
 * Time: 6:48 PM
 * <p/>
 * UTI Systems, TIC
 */
public class RequestEventEngineOrchestrator implements org.iso.mpeg.mxm.orchestratorEngine.requestevent.RequestEventEngineOrchestrator {
    private RequestEventEngine requestEventEngine;
    private EventReportEngine erEngine;

    public RequestEventEngineOrchestrator(RequestEventEngine requestEventEngine, EventReportEngine erEngine) {
        this.requestEventEngine = requestEventEngine;
        this.erEngine = erEngine;
    }

    /* (non-Javadoc)
      * @see org.iso.mpeg.mxm.orchestratorEngine.requestevent.RequestEventEngineOrchestrator#getRequestEventEngine()
      */
    @Override
    public RequestEventEngine getRequestEventEngine() {
        return requestEventEngine;
    }

    /**
     * The Client sends a RequestEventRequest message.
     * The message contains a content reference or an Event Report Request.
     * <p/>
     * The SP sends a RequestEventResponse that according to the request may contain an event report Request or a set of Event Reports.
     *
     * @param request The message for creating a request for RequestEvent
     * @return The response contains the result of the RequestEvent Service
     */
    @Override
    public RequestEventResponse requestEvent(RequestEventRequest request) {
        RequestEventResponse requestEventResponse = requestEventEngine.getRequestEventSchemaHandler().getRequestEventResponse();

        try {
            if (null != request.getERR()) {
                processRequestEvent(request.getERR(), requestEventResponse);
            } else if (null != request.getContentEntity()) {
                processRequestEvent(request.getContentEntity(), requestEventResponse);
            } else {
                ProtocolFailure requestEventFailure = requestEventResponse.getRequestEventFailure();
                requestEventFailure.addResultCode(BasicFailureCode.UNABLE_TO_PROCESS.value());
                requestEventFailure.setDisplayString("Bad requestEventRequest!");
                requestEventResponse.setRequestEventFailure(requestEventFailure);
            }
        } catch (Exception e) {
            e.printStackTrace();

            try {
                ProtocolFailure requestEventFailure = requestEventResponse.getRequestEventFailure();
                requestEventFailure.addResultCode(BasicFailureCode.UNKNOWN_ERROR.value());
                requestEventFailure.setDisplayString(e.getMessage());
                requestEventResponse.setRequestEventFailure(requestEventFailure);
            } catch (Exception e2) {
                e2.printStackTrace();
            }
        }

        return requestEventResponse;
    }


    private void processRequestEvent(ContentEntity contentEntity, RequestEventResponse requestEventResponse) throws Exception {
        RequestEventSchemaHandler schemaHandler = requestEventEngine.getRequestEventSchemaHandler();

        String DIDLId = contentEntity.getDIDL().getItem().getId();

        List<String> erIds = null;
        List<ERKeeper> erKeepers = erEngine.getEventReportEngineTechHandler().getEventReportKeepers();
        ERKeeper erKeeper = null;
        for (ERKeeper erKeeperCurrent : erKeepers) {
            erIds = erKeeperCurrent.getERIdsByDIDLId(DIDLId);
            if (null != erIds && !erIds.isEmpty()) {
                erKeeper = erKeeperCurrent;
                break;
            }
        }

        ERR err = null;
        List<ERRKeeper> errKeepers = erEngine.getEventReportEngineTechHandler().getEventReportRequestKeepers();
        for (ERRKeeper errKeeperCurrent : errKeepers) {
            err = errKeeperCurrent.getERRByDIDLId(DIDLId);
        }

        if (null != erIds && !erIds.isEmpty() && null != erKeeper && null != err) {
            RequestEventSuccess requestEventSuccess = schemaHandler.getRequestEventSuccess();
            requestEventSuccess.setERR(err);
            for (String erId : erIds) {
                requestEventSuccess.addER(erKeeper.getER(erId));
            }

            requestEventResponse.setRequestEventSuccess(requestEventSuccess);
        } else {
            ProtocolFailure requestEventFailure = requestEventResponse.getRequestEventFailure();
            requestEventFailure.addResultCode("NO_ER");
            requestEventFailure.setDisplayString("No er found!");
            requestEventResponse.setRequestEventFailure(requestEventFailure);
        }
    }

    private void processRequestEvent(ERR err, RequestEventResponse requestEventResponse) throws Exception {
        RequestEventSchemaHandler schemaHandler = requestEventEngine.getRequestEventSchemaHandler();

        List<String> erIds = null;
        List<ERKeeper> erKeepers = erEngine.getEventReportEngineTechHandler().getEventReportKeepers();
        ERKeeper erKeeper = null;
        for (ERKeeper erKeeperCurrent : erKeepers) {
            erIds = erKeeperCurrent.getERIdsByERRId(err.getERRIdentifier());
            if (null != erIds && !erIds.isEmpty()) {
                erKeeper = erKeeperCurrent;
                break;
            }
        }

        if (null != erIds && !erIds.isEmpty() && null != erKeeper) {
            RequestEventSuccess requestEventSuccess = schemaHandler.getRequestEventSuccess();
            requestEventSuccess.setERR(err);
            for (String erId : erIds) {
                requestEventSuccess.addER(erKeeper.getER(erId));
            }

            requestEventResponse.setRequestEventSuccess(requestEventSuccess);
        } else {
            ProtocolFailure requestEventFailure = requestEventResponse.getRequestEventFailure();
            requestEventFailure.addResultCode("NO_ER");
            requestEventFailure.setDisplayString("No er found!");
            requestEventResponse.setRequestEventFailure(requestEventFailure);
        }
    }
}
