/** ***** BEGIN LICENSE BLOCK *****
 *
 * The copyright in this software is being made available under the BSD License, 
 * included below. This software may be subject to other third party and contributor 
 * rights, including patent rights, and no such rights are granted under this license.
 *
 * Copyright (c) 2012, ICBNet Lab, ICCS
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification, 
 * are permitted provided that the following conditions are met:
 *
 *    - Redistributions of source code must retain the above copyright notice, 
 *    	this list of conditions and the following disclaimer.
 *    - Redistributions in binary form must reproduce the above copyright notice, 
 *    	this list of conditions and the following disclaimer in the documentation 
 *    	and/or other materials provided with the distribution.
 *    - Neither the name of the ICBNet Lab, ICCS nor the names of its contributors 
 *    	may be used to endorse or promote products derived from this software 
 *    	without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND 
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED 
 * WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. 
 * IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, 
 * INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, 
 * BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, 
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF 
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE 
 * OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED 
 * OF THE POSSIBILITY OF SUCH DAMAGE.
 *
 ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.orchestrator.storeevent;

import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.BasicFailureCode;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.BasicSuccessCode;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolFailure;
import org.iso.mpeg.mxm.pEngine.basePE.schemahandler.ProtocolSuccess;
import org.iso.mpeg.mxm.pEngine.storeeventPE.StoreEventEngine;
import org.iso.mpeg.mxm.pEngine.storeeventPE.schemahandler.StoreEventRequest;
import org.iso.mpeg.mxm.pEngine.storeeventPE.schemahandler.StoreEventResponse;
import org.iso.mpeg.mxm.tEngine.eventreportTE.EventReportEngine;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ER;
import org.iso.mpeg.mxm.tEngine.eventreportTE.schemahandler.er.ERR;
import org.iso.mpeg.mxm.tEngine.eventreportTE.technologyhandler.ERKeeper;

import java.util.List;

/**
 * <p/>
 * User: <a href="mailto:bogdan.ardeleanu@uti.ro">Bogdan Ardeleanu</a>, Angelos-Christos Anadiotis
 * Date: 04/06/2012
 * Time: 15:53 PM
 * <p/>
 * UTI Systems, TIC
 */
public class StoreEventEngineOrchestrator implements org.iso.mpeg.mxm.orchestratorEngine.storeevent.StoreEventEngineOrchestrator {
    private StoreEventEngine storeEventEngine;
    private EventReportEngine erEngine;

    /**
     * Default repository key for storing ERR objects
     */
    private static final String DEFAULT_ERR_PE_REPOSITORY_KEY = "DEFAULT_PE_ERR";

    /**
     * Default repository key for storing ER objects
     */
    private static final String DEFAULT_ER_PE_REPOSITORY_KEY = "DEFAULT_PE_ER";

    public StoreEventEngineOrchestrator(StoreEventEngine storeEventEngine, EventReportEngine erEngine) {
        this.storeEventEngine = storeEventEngine;
        this.erEngine = erEngine;
    }

    /* (non-Javadoc)
      * @see org.iso.mpeg.mxm.orchestratorEngine.storeevent.StoreEventEngineOrchestrator#getStoreEventEngine()
      */
    @Override
    public StoreEventEngine getStoreEventEngine() {
        return storeEventEngine;
    }

    /**
     * An User sends a <tt>StoreEventRequest</tt> message, to report that an event occurred or to store an event report request.
     * The SP sends a <tt>ReportEventResponse</tt> indicating whether the operation was successful or not.
     *
     * @param request The Request for Store Event as an Event Report or an Event Report Request..
     * @return The response contains the result of the Store Event Service.
     */
    @Override
    public StoreEventResponse storeEvent(StoreEventRequest request) {
        StoreEventResponse storeEventResponse = storeEventEngine.getStoreEventSchemaHandler().getStoreEventResponse();

        try {
            if (null != request.getER() && !request.getER().isEmpty()) {
                processStoreEvent(request.getER(), storeEventResponse);
            } else if (null != request.getERR()) {
                processStoreEvent(request.getERR(), storeEventResponse);
            } else {
                ProtocolFailure storeEventFailure = storeEventEngine.getBaseProtocolSchemaHandler().getProtocolFailure();
                storeEventFailure.addResultCode(BasicFailureCode.UNABLE_TO_PROCESS.value());
                storeEventFailure.setDisplayString("Bad storeEventRequest!");
                storeEventResponse.setStoreEventFailure(storeEventFailure);
            }
        } catch (Exception e) {
            e.printStackTrace();

            try {
                ProtocolFailure storeEventFailure = storeEventEngine.getBaseProtocolSchemaHandler().getProtocolFailure();
                storeEventFailure.addResultCode(BasicFailureCode.UNKNOWN_ERROR.value());
                storeEventFailure.setDisplayString(e.getMessage());
                storeEventResponse.setStoreEventFailure(storeEventFailure);
            } catch (Exception e2) {
                e2.printStackTrace();
            }
        }

        return storeEventResponse;
    }

    private void processStoreEvent(ERR err, StoreEventResponse storeEventResponse) throws Exception {
        //ERRKeeper errKeeper = erEngine.getEventReportEngineTechHandler().getEventReportRequestKeeper(DEFAULT_ERR_PE_REPOSITORY_KEY);

        ProtocolFailure storeEventFailure = storeEventEngine.getBaseProtocolSchemaHandler().getProtocolFailure();
        storeEventFailure.addResultCode(BasicFailureCode.OPERATION_NOT_SUPPORTED.value());
        storeEventFailure.setDisplayString("Cannot store ERR. The ERR is related to a particular Digital Item");
        storeEventResponse.setStoreEventFailure(storeEventFailure);
    }

    private void processStoreEvent(List<ER> ers, StoreEventResponse storeEventResponse) throws Exception {
        ERKeeper erKeeper = erEngine.getEventReportEngineTechHandler().getEventReportKeeper(DEFAULT_ER_PE_REPOSITORY_KEY);

        try {
            StringBuilder sb = new StringBuilder();
            for (ER er : ers) {
                erKeeper.storeER(er);
                sb.append(" ERId=").append(er.getERIdentifier());
            }

            ProtocolSuccess success = storeEventEngine.getBaseProtocolSchemaHandler().getProtocolSuccess();
            success.addSuccessCode(BasicSuccessCode.OK.value());
            success.setDisplayString("List of ERs IDs: "+ sb.toString());
            storeEventResponse.setStoreEventSuccess(success);
        } catch (Exception e) {
            e.printStackTrace();

            ProtocolFailure storeEventFailure = storeEventEngine.getBaseProtocolSchemaHandler().getProtocolFailure();
            storeEventFailure.addResultCode(BasicFailureCode.UNKNOWN_ERROR.value());
            storeEventFailure.setDisplayString("Cannot store ERs. Details: " + e.getMessage());
            storeEventResponse.setStoreEventFailure(storeEventFailure);
        }
    }

}
