/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is CEDEO.net
 * Portions created by the Initial Developer are Copyright (C) 2006-2007
 * All Rights Reserved.
 *
 * The Initial Developer: Filippo Chiariglione 
 * Contributor(s):
 *
 * ***** END LICENSE BLOCK ***** */

package org.iso.mpeg.mxm.relTE.technologyhandler;

import java.util.GregorianCalendar;
import java.util.List;

import javax.xml.datatype.DatatypeConfigurationException;
import javax.xml.datatype.DatatypeConstants;
import javax.xml.datatype.DatatypeFactory;
import javax.xml.datatype.XMLGregorianCalendar;
import javax.xml.namespace.QName;

import org.apache.log4j.Logger;
import org.iso.mpeg.mxm.tEngine.relTE.exception.RELAuthorisationException;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relm2x.SecuritySystem;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relm3x.CopyrightNotice;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relm3x.NonCommercialUse;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.Condition;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relr.ValidityInterval;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.ExerciseLimit;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.FeeFlat;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.FeeMetered;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.FeePerInterval;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.FeePerUse;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.FeePerUsePrePay;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.Territory;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.TerritoryDomain;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.TerritoryLocation;
import org.iso.mpeg.mxm.tEngine.relTE.schemahandler.relsx.ValidityTimeMetered;
import org.iso.mpeg.mxm.tEngine.relTE.technologyhandler.EvaluatedConditionResult;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;

public class ConditionManager 
{
	private Logger logger = Logger.getLogger(ConditionManager.class);
	
	public ConditionManager()
	{
	}
	
	public ConditionResult verifyConditions (List<Condition> conditions, QName currentCountry, 
			QName currentRegion, boolean isCommercial) throws RELAuthorisationException
	{
		//prepare the result to be returned by this method
		ConditionResult conditionResult = new ConditionResult();		
		
		if (conditions == null)
		{
			// if there are no conditions, the default value is: SUCCESSFULLY_MET
			return conditionResult;
		}

		for (Condition condition : conditions)
		{
			EvaluatedCondition evalCondition = new EvaluatedCondition(condition);
			
			//VALIDITY INTERVAL
			if (condition instanceof ValidityInterval)
			{
				ValidityInterval validityInterval = (ValidityInterval) condition;
				if (this.verifyValidityInterval(validityInterval)) 
				{
					evalCondition.setConditionType(EvaluatedConditionResult.SUCCESSFULLY_MET);
				}
				else
				{
					evalCondition.setConditionType(EvaluatedConditionResult.NOT_MET);
				}
			}

			//EXERCISE_LIMIT
			else if (condition instanceof ExerciseLimit)
			{
				ExerciseLimit exerciseLimit = (ExerciseLimit) condition;
				if (exerciseLimit.getCount().intValue() >= 0)
				{
					evalCondition.setConditionType(EvaluatedConditionResult.SUCCESSFULLY_MET);
				}
				else
				{
					evalCondition.setConditionType(EvaluatedConditionResult.NOT_MET);
				}
			}

			//TERRITORY
			else if (condition instanceof Territory)
			{
				Territory territory = (Territory) condition;
				List<Object> contents = territory.getLocationOrDomain();
				if (contents.get(0) instanceof TerritoryLocation)
				{
					TerritoryLocation location = (TerritoryLocation)contents.get(0);
					evalCondition.setConditionType(this.verifyLocation(location, currentCountry, currentRegion));
				}
				else if (contents.get(0) instanceof TerritoryDomain)
				{
					evalCondition.setConditionType(EvaluatedConditionResult.UNSUPPORTED);
				}
			}

			//ORC
			else if (condition instanceof NonCommercialUse)
			{
				if (isCommercial)
				{
					evalCondition.setConditionType(EvaluatedConditionResult.UNSUPPORTED);
				}
				else
				{
					evalCondition.setConditionType(EvaluatedConditionResult.SUCCESSFULLY_MET);
				}
			}
			
			//COPYRIGHT
			else if (condition instanceof CopyrightNotice)
			{
				evalCondition.setConditionType(EvaluatedConditionResult.COPYRIGHT_NOTICE);
			}
			
			//VALIDITY_TIME_METERED
			else if (condition instanceof ValidityTimeMetered)
			{
				evalCondition.setConditionType(EvaluatedConditionResult.UNSUPPORTED);
			}

			//SECURITY_SYSTEM
			else if (condition instanceof SecuritySystem)
			{
				evalCondition.setConditionType(EvaluatedConditionResult.UNSUPPORTED);
			}

			//PAYMENT
			else if (condition instanceof FeeFlat || condition instanceof FeeMetered || condition instanceof FeePerInterval 
					|| condition instanceof FeePerUse || condition instanceof FeePerUsePrePay) 
			{
				evalCondition.setConditionType(EvaluatedConditionResult.PAYMENT);
			}
			
			//all other conditions are not supported
			else 
			{
				evalCondition.setConditionType(EvaluatedConditionResult.UNSUPPORTED);				
			}
			
			conditionResult.addEvaluatedCondition(evalCondition);
		}

		return conditionResult;
	}
	
	private boolean verifyValidityInterval(ValidityInterval validityInterval) throws RELAuthorisationException
	{	
		try 
		{
			DatatypeFactory dateTypeFactory = DatatypeFactory.newInstance();
			XMLGregorianCalendar now = dateTypeFactory.newXMLGregorianCalendar(new GregorianCalendar());
			return verifyNotAfter(validityInterval.getNotAfter(), now) && verifyNotBefore(validityInterval.getNotBefore(), now);
		} 
		catch (DatatypeConfigurationException e)
		{
			e.printStackTrace();
			String message = "A DataTypeConfigurationException occurred. The detailed exception is: " + e.getMessage();
			logger.error(message);
			throw new RELAuthorisationException(message);
		}
	}
	
	private boolean verifyNotAfter(XMLGregorianCalendar notAfter, XMLGregorianCalendar now)
	{
		logger.debug("Not After: " + notAfter.toString());
		logger.debug("Now: " + now.toString());
		
		if (notAfter == null) 
		{
			return true;
		}
		else
		{
			boolean value = now.compare(notAfter) == DatatypeConstants.LESSER;
			return value;
		}
	}

	private boolean verifyNotBefore(XMLGregorianCalendar notBefore, XMLGregorianCalendar now) 
	{
		logger.debug("Not Before: " + notBefore.toString());
		logger.debug("Now: " + now.toString());
		
		if (notBefore == null)
		{
			return true;
		}
		else
		{
			boolean value = now.compare(notBefore) == DatatypeConstants.GREATER;
			return value;
		}
	}

	private EvaluatedConditionResult verifyLocation (TerritoryLocation licenseLocation, QName currentCountry, QName currentRegion) 
			throws RELAuthorisationException 
	{
		QName destCountry = licenseLocation.getCountry();
		QName destRegion = licenseLocation.getRegion();
		if (destCountry != null) 
		{
			if (currentCountry != null)
			{
				if(currentCountry.equals(destCountry))
				{
					if (destRegion != null)
					{
						if (currentRegion != null) 
						{					
							if(currentRegion.equals(destRegion))
							{
								return EvaluatedConditionResult.SUCCESSFULLY_MET;
							}
							else
							{
								return EvaluatedConditionResult.NOT_MET;
							}
						}
						else
						{
							return EvaluatedConditionResult.UNDETERMINED;
						}
					}
					else
					{
						return EvaluatedConditionResult.SUCCESSFULLY_MET;
					}
				}
				else
				{
					return EvaluatedConditionResult.NOT_MET;
				}
			}
			else
			{
				return EvaluatedConditionResult.UNDETERMINED;
			}
		}
		else
		{
			return EvaluatedConditionResult.SUCCESSFULLY_MET;
		}
	}

}
