package org.iso.mpeg.mxm.securityTE.technologyhandler;


import java.security.KeyPair;
import java.security.PublicKey;
import java.sql.Date;
import java.sql.PreparedStatement;
import java.sql.ResultSet;
import java.sql.SQLException;

import javax.security.cert.X509Certificate;

import org.apache.commons.dbcp.BasicDataSource;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.securityTE.technologyhandler.util.FileAndObjectHandler;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.AuthnRequest;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.saml.protocol.Response;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.jdbc.core.RowMapper;
import org.springframework.jdbc.core.support.AbstractLobCreatingPreparedStatementCallback;
import org.springframework.jdbc.support.lob.DefaultLobHandler;
import org.springframework.jdbc.support.lob.LobCreator;


/**
 * Implements a MySql Gateway for the Service Provider Repository Manager.
 * 
 * @author Christoph Topmoeller, Andreas Kohlos
 */
public class SPUserDataBaseManager implements
    org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.UserDataBaseManager
{
    private X509Certificate   cert                   = null;
    private boolean           isRevoked              = false;
    private X509Certificate   issuerCert             = null;
    private JdbcTemplate      jdbcTemplate           = null;
    private KeyPair           keyPair                = null;
    private DefaultLobHandler lobhandler             = null;
    private boolean           userPseudonymIsCreated = false;

    /**
     * Constructor - Adds a CryptoModuleManager, creates the jdbcTemplate and the MySql
     * Database. Creates the database and the tables "users" and "sp_credentials" if they
     * do not exist already.
     * 
     * @param cryptoModuleManager
     * @param mySqlUrl
     * @param mySqlUsername
     * @param mySqlPassword
     * @param dataBaseName
     */
    public SPUserDataBaseManager(final CryptoModuleManagerInterface cryptoModuleManager,
        final String mySqlUrl, final String mySqlUsername, final String mySqlPassword,
        final String dataBaseName)
    {
        if (this.jdbcTemplate == null)
        {
            final BasicDataSource dataSource = new BasicDataSource();
            dataSource.setDriverClassName("com.mysql.jdbc.Driver");
            dataSource.setUrl("jdbc:mysql://" + mySqlUrl);
            dataSource.setUsername(mySqlUsername);
            dataSource.setPassword(mySqlPassword);
            this.jdbcTemplate = new JdbcTemplate(dataSource);
        }
        this.jdbcTemplate.execute("CREATE DATABASE if not exists " + dataBaseName
            + " DEFAULT CHARSET=utf8");
        this.jdbcTemplate.execute("USE " + dataBaseName);
        this.jdbcTemplate
            .execute("CREATE TABLE if not exists users ( "
                + "id Integer,"
                + " pseudonym Varchar(80),"
                + " callback Boolean,"
                + " callback_date Date,"
                + " callback_reason Varchar(80),"
                + " auth_certificate BLOB,"
                + " sign_certificate BLOB,"
                + " ciph_certificate BLOB,"
                + " ip_usercertificate BLOB,"
                + " UNIQUE KEY `ID` (`id`), UNIQUE KEY `PSEUDONYM` (`pseudonym`) ) ENGINE=InnoDB DEFAULT CHARSET=utf8");
        this.jdbcTemplate.execute("CREATE TABLE if not exists sp_credentials ( "
            + "ip_root_certificate BLOB," + " sp_keypair BLOB,"
            + " sp_root_certificate BLOB)" + " ENGINE=InnoDB DEFAULT CHARSET=utf8");
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#deleteUser(java.
     * lang.String)
     */
    @Override
    public void deleteUser(final String userPseudonym)
    {
        this.jdbcTemplate.update("DELETE FROM users WHERE pseudonym = ?",
            new Object[] {userPseudonym});
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#getIPRootCertificate ()
     */
    @Override
    public X509Certificate getIPRootCertificate() throws SecurityEngineException
    {
        this.cert = null;
        this.lobhandler = new DefaultLobHandler();
        this.jdbcTemplate.query("SELECT ip_root_certificate FROM sp_credentials",
            new RowMapper() {
                @Override
                public String mapRow(final ResultSet rs, final int i) throws SQLException
                {
                    final byte[] blobBytes = SPUserDataBaseManager.this.lobhandler
                        .getBlobAsBytes(rs, "ip_root_certificate");
                    try
                    {
                        SPUserDataBaseManager.this.cert = (X509Certificate)FileAndObjectHandler
                            .blobToObject(blobBytes);
                    }
                    catch (final SecurityEngineException e)
                    {
                        throw new SQLException(e.getMessage());
                    }
                    return null;
                }
            });
        return this.cert;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#getIssuerKeyPair()
     */
    @Override
    public KeyPair getIssuerKeyPair() throws SecurityEngineException
    {
        this.keyPair = null;
        this.lobhandler = new DefaultLobHandler();
        this.jdbcTemplate.query("SELECT sp_keypair FROM sp_credentials", new RowMapper() {
            @Override
            public String mapRow(final ResultSet rs, final int i) throws SQLException
            {
                final byte[] blobBytes = SPUserDataBaseManager.this.lobhandler
                    .getBlobAsBytes(rs, "sp_keypair");
                try
                {
                    SPUserDataBaseManager.this.keyPair = (KeyPair)FileAndObjectHandler
                        .blobToObject(blobBytes);
                }
                catch (final SecurityEngineException e)
                {
                    throw new SQLException(e.getMessage());
                }
                return null;
            }
        });
        return this.keyPair;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#getIssuerRootCertificate
     * ()
     */
    @Override
    public X509Certificate getIssuerRootCertificate() throws SecurityEngineException
    {
        this.issuerCert = null;
        this.lobhandler = new DefaultLobHandler();
        this.jdbcTemplate.query("SELECT sp_root_certificate FROM sp_credentials",
            new RowMapper() {
                @Override
                public String mapRow(final ResultSet rs, final int i) throws SQLException
                {
                    final byte[] blobBytes = SPUserDataBaseManager.this.lobhandler
                        .getBlobAsBytes(rs, "sp_root_certificate");
                    try
                    {
                        SPUserDataBaseManager.this.issuerCert = (X509Certificate)FileAndObjectHandler
                            .blobToObject(blobBytes);
                    }
                    catch (final SecurityEngineException e)
                    {
                        throw new SQLException(e.getMessage());
                    }
                    return null;
                }
            });
        return this.issuerCert;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#getNextUserID()
     */
    @Override
    public int getNextUserID()
    {
        int result = -1;
        result = this.jdbcTemplate.queryForInt("SELECT MAX(id) FROM users;");
        result++;
        return result;
    }

    @Override
    public PublicKey getPublicKey(final String hashCode)
    {
        // TODO Auto-generated method stub
        return null;
    }

    @Override
    public String getUserId(final PublicKey publicKeyObj)
    {
        // TODO Auto-generated method stub
        return null;
    }

    // public X509Certificate getSPAuthCertificate() throws
    // SecurityEngineException
    // {
    // this.cert = null;
    // this.lobhandler = new DefaultLobHandler();
    // this.jdbcTemplate.query("SELECT auth_certificate FROM sp_credentials",
    // new RowMapper() {
    // @Override
    // public String mapRow(final ResultSet rs, final int i) throws SQLException
    // {
    // final byte[] blobBytes = SPRepositoryManager.this.lobhandler
    // .getBlobAsBytes(rs, "auth_certificate");
    // try
    // {
    // SPRepositoryManager.this.cert = (X509Certificate)FileAndObjectHandler
    // .blobToObject(blobBytes);
    // }
    // catch (final SecurityEngineException e)
    // {
    // throw new SQLException(e.getMessage());
    // }
    // return null;
    // }
    // });
    // return this.cert;
    // }
    //
    // /**
    // * This method returns the x509 root certificate from the database.
    // *
    // * @return java.security.cert.X509Certificate
    // * @throws IOException
    // */
    // public X509Certificate getSPCipherCertificate() throws
    // SecurityEngineException
    // {
    // // TODO gets all three certificates
    // this.cert = null;
    // this.lobhandler = new DefaultLobHandler();
    // this.jdbcTemplate.query("SELECT ciph_certificate FROM sp_credentials",
    // new RowMapper() {
    // @Override
    // public String mapRow(final ResultSet rs, final int i) throws SQLException
    // {
    // final byte[] blobBytes = SPRepositoryManager.this.lobhandler
    // .getBlobAsBytes(rs, "ciph_certificate");
    // try
    // {
    // SPRepositoryManager.this.cert = (X509Certificate)FileAndObjectHandler
    // .blobToObject(blobBytes);
    // }
    // catch (final SecurityEngineException e)
    // {
    // throw new SQLException(e.getMessage());
    // }
    // return null;
    // }
    // });
    // return this.cert;
    // }
    //
    // public X509Certificate getSPSignCertificate() throws
    // SecurityEngineException
    // {
    // this.cert = null;
    // this.lobhandler = new DefaultLobHandler();
    // this.jdbcTemplate.query("SELECT sign_certificate FROM sp_credentials",
    // new RowMapper() {
    // @Override
    // public String mapRow(final ResultSet rs, final int i) throws SQLException
    // {
    // final byte[] blobBytes = SPRepositoryManager.this.lobhandler
    // .getBlobAsBytes(rs, "sign_certificate");
    // try
    // {
    // SPRepositoryManager.this.cert = (X509Certificate)FileAndObjectHandler
    // .blobToObject(blobBytes);
    // }
    // catch (final SecurityEngineException e)
    // {
    // throw new SQLException(e.getMessage());
    // }
    // return null;
    // }
    // });
    // return this.cert;
    // }
    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#grantUser(java.lang
     * .String)
     */
    @Override
    public void grantUser(final String userPseudonym)
    {
        this.jdbcTemplate.update("UPDATE users SET callback = ? WHERE pseudonym = ?",
            new Object[] {false, userPseudonym});
        this.jdbcTemplate.update(
            "UPDATE users SET callback_date = ? WHERE pseudonym = ?", new Object[] {null,
                userPseudonym});
        this.jdbcTemplate.update(
            "UPDATE users SET callback_reason = ? WHERE pseudonym = ?", new Object[] {
                null, userPseudonym});
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#isUserCreated(java
     * .lang.String )
     */
    @Override
    public boolean isUserCreated(final String userPseudonym)
    {
        this.jdbcTemplate.query("SELECT * FROM users WHERE pseudonym='" + userPseudonym
            + "'", new RowMapper() {
            @Override
            public String mapRow(final ResultSet result, final int arg1)
                throws SQLException
            {
                if (result != null)
                {
                    SPUserDataBaseManager.this.userPseudonymIsCreated = true;
                }
                return null;
            }
        });
        return this.userPseudonymIsCreated;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#isUserRevoked(java
     * .lang.String )
     */
    @Override
    public boolean isUserRevoked(final String userPseudonym)
    {
        this.jdbcTemplate.query("SELECT callback FROM users WHERE pseudonym="
            + userPseudonym, new RowMapper() {
            @Override
            public String mapRow(final ResultSet result, final int arg1)
                throws SQLException
            {
                SPUserDataBaseManager.this.isRevoked = result.getBoolean("callback");
                return null;
            }
        });
        return this.isRevoked;
    }

    @Override
    public int registerUser(final PublicKey key, final String firstName,
        final String lastName, final Date birthday, final String country,
        final String locality, final String organization) throws SecurityEngineException
    {
        // TODO Auto-generated method stub
        return 0;
    }

    @Override
    public int registerUser(final X509Certificate userCertificate,
        final String firstName, final String lastName, final Date birthday,
        final String country, final String locality, final String organization)
        throws SecurityEngineException
    {
        // TODO Auto-generated method stub
        return 0;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#revokeUser(java.
     * lang.String, java.lang.String)
     */
    @Override
    public void revokeUser(final String userPseudonym, final String callbackReason)
    {
        this.jdbcTemplate.update("UPDATE users SET callback = ? WHERE pseudonym = ?",
            new Object[] {true, userPseudonym});
        this.jdbcTemplate.update(
            "UPDATE users SET callback_date = ? WHERE pseudonym = ?", new Object[] {
                new Date(System.currentTimeMillis()), userPseudonym});
        this.jdbcTemplate.update(
            "UPDATE users SET callback_reason = ? WHERE pseudonym= ?", new Object[] {
                callbackReason, userPseudonym});
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#storeIssuerCredentials
     * (javax .security.cert.X509Certificate, javax.security.cert.X509Certificate,
     * java.security.KeyPair)
     */
    @Override
    public void storeIssuerCredentials(final X509Certificate iPRootCertificate,
        final X509Certificate sPRootCertificate, final KeyPair issuerKeyPair)
        throws SecurityEngineException
    {
        this.lobhandler = new DefaultLobHandler();
        // Convert the root certificate to a byte array.
        final byte[] rootCertificateAsBlob = FileAndObjectHandler
            .objectToBlob(iPRootCertificate);
        // Convert the key pair to a byte array.
        final byte[] sPKeyPairAsBlob = FileAndObjectHandler.objectToBlob(issuerKeyPair);
        // Convert the SP certificate to a byte array.
        final byte[] spCertificateAsBlob = FileAndObjectHandler
            .objectToBlob(sPRootCertificate);
        this.jdbcTemplate
            .execute(
                "insert into sp_credentials(ip_root_certificate, sp_keypair, sp_root_certificate) values (?, ?, ?)",
                new AbstractLobCreatingPreparedStatementCallback(this.lobhandler) {
                    @Override
                    protected void setValues(final PreparedStatement ps,
                        final LobCreator lobCreator) throws SQLException
                    {
                        lobCreator.setBlobAsBytes(ps, 1, rootCertificateAsBlob);
                        lobCreator.setBlobAsBytes(ps, 2, sPKeyPairAsBlob);
                        lobCreator.setBlobAsBytes(ps, 3, spCertificateAsBlob);
                    }
                });
    }

    /**
     * Stores a new user to the database.
     * 
     * @param userPseudonym
     * @param authCertificate
     * @param signCertificate
     * @param ciphCertificate
     * @param iPUserCertificate
     * @return
     * @throws SecurityEngineException
     */
    public int storeUser(final String userPseudonym,
        final X509Certificate authCertificate, final X509Certificate signCertificate,
        final X509Certificate ciphCertificate, final X509Certificate iPUserCertificate)
        throws SecurityEngineException
    {
        this.lobhandler = new DefaultLobHandler();
        // Check if this pseudonym already exists inside the database.
        final int createdId = this.getNextUserID();
        if (this.isUserCreated(userPseudonym))
        {
            throw new SecurityEngineException(
                "This pseudonym is already chosen by another user. Please choose another pseudonym.");
        }
        else
        {
            // Convert the certificates to a byte array.
            final byte[] authCertificateAsBlob = FileAndObjectHandler
                .objectToBlob(authCertificate);
            final byte[] signCertificateAsBlob = FileAndObjectHandler
                .objectToBlob(signCertificate);
            final byte[] ciphCertificateAsBlob = FileAndObjectHandler
                .objectToBlob(ciphCertificate);
            final byte[] iPUserCertificateAsBlob = FileAndObjectHandler
                .objectToBlob(iPUserCertificate);
            this.jdbcTemplate
                .execute(
                    "insert into users(id, pseudonym, callback, callback_date, callback_reason, auth_certificate, sign_certificate, ciph_certificate, ip_usercertificate) values (?, ?, ?, ?, ?, ?, ?, ?, ?)",
                    new AbstractLobCreatingPreparedStatementCallback(this.lobhandler) {
                        @Override
                        protected void setValues(final PreparedStatement ps,
                            final LobCreator lobCreator) throws SQLException
                        {
                            ps.setInt(1, createdId);
                            ps.setString(2, userPseudonym);
                            ps.setBoolean(3, false);
                            ps.setDate(4, null);
                            ps.setString(5, null);
                            lobCreator.setBlobAsBytes(ps, 6, authCertificateAsBlob);
                            lobCreator.setBlobAsBytes(ps, 7, signCertificateAsBlob);
                            lobCreator.setBlobAsBytes(ps, 8, ciphCertificateAsBlob);
                            lobCreator.setBlobAsBytes(ps, 9, iPUserCertificateAsBlob);
                        }
                    });
        }
        return createdId;
    }

    /*
     * (non-Javadoc)
     * @see org.iso.mpeg.mxm.engine.securityTE.RepositoryManager#storeUserCertsToCard
     * (java. lang.String, javax.security.cert.X509Certificate[])
     */
    @Override
    public void storeUserCertsToCard(final String PIN,
        final X509Certificate[] userCertificates) throws SecurityEngineException
    {
        // TODO use real card command here
        // this.cryptoModuleManager.exampleCardCommand();
    }

    @Override
    public Response authenticateUser(AuthnRequest authenticateUserRequest)
        throws MXMEngineException
    {
        // TODO Auto-generated method stub
        return null;
    }
}
