/** ***** BEGIN LICENSE BLOCK *****
 * Version: MPL 1.1
 *
 * The contents of this file are subject to the Mozilla  Public License Version
 * 1.1 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 * http://www.mozilla.org/MPL/
 *
 * Software distributed under the License is distributed on an "AS IS" basis,
 * WITHOUT WARRANTY OF ANY KIND, either express or implied. See the License
 * for the specific language governing rights and  limitations under the
 * License.
 *
 * The Original Code is the Chillout Project (http://chillout.dmpf.org)
 *
 * The Initial Developer of the Original Code is Joint Digital Lab (China)
 * Portions created by the Initial Developer are Copyright (C) 2006 - 2008
 * All Rights Reserved.
 *
 * The Initial Developer: Mou Luntian 
 * Contributor(s): Filippo Chiariglione
 *
 * ***** END LICENSE BLOCK ***** */
package org.iso.mpeg.mxm.securityTE.technologyhandler.util;


import java.io.ByteArrayInputStream;
import java.io.ByteArrayOutputStream;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.ObjectInputStream;
import java.io.ObjectOutputStream;
import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;

import org.apache.log4j.Logger;
import org.bouncycastle.util.encoders.Base64;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;


public class SecureInfoStorageTool
{
    // private static SecureInfoStorageEngine SecureInfoStorageEngine = null;
    private final Logger logger    = Logger.getLogger(SecureInfoStorageTool.class);
    private File         storeFile = null;

    /**
     * Constructor. You must specify the location for the infostore when you new a
     * SecureInfoStorageEngine.
     * 
     * @param _infoStoreLocation - filename for the infostore in use, full pathname is
     *            expected.
     * @throws IOException
     */
    public SecureInfoStorageTool(final File storeFile) throws SecurityEngineException
    {
        this.storeFile = storeFile;
        if (storeFile == null)
        {
            throw new SecurityEngineException(
                "A null infoStore file was passed in the constructor.");
        }
        if (!storeFile.exists())
        {
            try
            {
                storeFile.createNewFile();
            }
            catch (final IOException e)
            {
                e.printStackTrace();
                final String message = "The file: " + storeFile.getAbsolutePath()
                    + " cannot be created. The detailed exception is: " + e.getMessage();
                this.logger.error(message);
                throw new SecurityEngineException(message);
            }
        }
        if (!(storeFile.isFile() || storeFile.canRead() || storeFile.canWrite()))
        {
            throw new SecurityEngineException("The infostore pathname given in input:"
                + storeFile.getAbsolutePath() + " is invalid.");
        }
    }

    // /**
    // * get an instance of class SecureInfoStorageEngine
    // * @param _infoStoreLocation - filename for the infostore in use, full pathname
    // recommended
    // * @return
    // */
    // public static SecureInfoStorageEngine getInstance(String storeLocation) {
    // if (SecureInfoStorageEngine == null)
    // SecureInfoStorageEngine = new SecureInfoStorageEngine(storeLocation);
    // return SecureInfoStorageEngine;
    // }
    /**
     * Store secure information in the infostore with the given alias You can implement
     * your own infostore according to different security requirements. Our current
     * implementation of an infostore is a file which contains an array of entries, the
     * actual structure of one record in this file is: aliasLen(2 bytes) + secureInfoAlias
     * + infoLen(4 bytes) + secureInfo.
     * 
     * @param secureInfoAlias - alias for the entry of the secure information
     * @param secureInfo - secure information
     * @throws IOException
     */
    public void addSecureInfo(final String secureInfoAlias, final String secureInfo)
        throws IOException
    {
        int aliasLen, infoLen;
        final FileOutputStream fos = new FileOutputStream(this.storeFile, true);
        aliasLen = secureInfoAlias.length();
        // if(aliasLen > 65535)
        fos.write(aliasLen >> 8);
        fos.write(aliasLen);
        fos.write(secureInfoAlias.getBytes());
        infoLen = secureInfo.length();
        // if(infoLen > 4294967295)
        fos.write(infoLen >> 24);
        fos.write(infoLen >> 16);
        fos.write(infoLen >> 8);
        fos.write(infoLen);
        fos.write(secureInfo.getBytes());
        fos.flush();
        fos.close();
    }

    /**
     * Delete secure information using the given alias
     * 
     * @param secureInfoAlias - alias for the entry of the secure information
     * @return all pieces of secure information with the given Alias
     * @throws IOException
     */
    public boolean deleteSecureInfo(final String secureInfoAlias, final String secureInfo)
        throws IOException
    {
        int aliasLen, infoLen;
        int tmp1, tmp2, tmp3, tmp4, tmp5, tmp6;
        boolean result = false;
        boolean skip;
        byte[] tmpAlias, tmpInfo;
        int[] tmpData;
        String strAlias = null;
        String strInfo = null;
        int outcnt = 0;
        final int size = (int)this.storeFile.length();
        tmpData = new int[size];
        final FileInputStream fis = new FileInputStream(this.storeFile);
        while (true)
        {
            tmp1 = fis.read();
            tmp2 = fis.read();
            aliasLen = (tmp1 << 8) | tmp2;
            if (aliasLen == -1)
            {
                break;
            }
            tmpAlias = new byte[aliasLen];
            fis.read(tmpAlias, 0, aliasLen);
            strAlias = new String(tmpAlias);
            tmp3 = fis.read();
            tmp4 = fis.read();
            tmp5 = fis.read();
            tmp6 = fis.read();
            infoLen = (tmp3 << 24) | (tmp4 << 16) | (tmp5 << 8) | tmp6;
            tmpInfo = new byte[infoLen];
            fis.read(tmpInfo, 0, infoLen);
            strInfo = new String(tmpInfo);
            if (secureInfo == null)
            {
                if (strAlias.compareTo(secureInfoAlias) == 0)
                {
                    skip = true;
                }
                else
                {
                    skip = false;
                }
            }
            else
            {
                if ((strAlias.compareTo(secureInfoAlias) == 0)
                    && (strInfo.compareTo(secureInfo) == 0))
                {
                    skip = true;
                    result = true;
                }
                else
                {
                    skip = false;
                }
            }
            if (!skip)
            {
                tmpData[outcnt++] = tmp1;
                tmpData[outcnt++] = tmp2;
                for (int i = 0; i < aliasLen; i++)
                {
                    tmpData[outcnt++] = tmpAlias[i];
                }
                tmpData[outcnt++] = tmp3;
                tmpData[outcnt++] = tmp4;
                tmpData[outcnt++] = tmp5;
                tmpData[outcnt++] = tmp6;
                for (int i = 0; i < infoLen; i++)
                {
                    tmpData[outcnt++] = tmpInfo[i];
                }
            }
        }
        ;
        fis.close();
        this.storeFile.delete();
        final FileOutputStream fos = new FileOutputStream(this.storeFile, true);
        for (int i = 0; i < outcnt; i++)
        {
            fos.write(tmpData[i]);
        }
        fos.flush();
        fos.close();
        return result;
    }

    /**
     * Retrieve secure information using the given alias
     * 
     * @param secureInfoAlias - alias for the entry of the secure information
     * @return all pieces of secure information with the given Alias
     * @throws IOException
     */
    public List<String> getSecureInfo(final String secureInfoAlias) throws IOException
    {
        int aliasLen, tmpAliasLen, infoLen;
        int tmp1, tmp2, tmp3, tmp4;
        final List<String> records = new ArrayList<String>();
        byte[] tmpAlias, tmpInfo;
        String infoAlias = null;
        String secureInfo = null;
        final FileInputStream fis = new FileInputStream(this.storeFile);
        aliasLen = secureInfoAlias.length();
        tmp1 = fis.read();
        tmp2 = fis.read();
        tmpAliasLen = (tmp1 << 8) | tmp2;
        while (tmpAliasLen != -1)
        {
            if (tmpAliasLen == aliasLen)
            {
                tmpAlias = new byte[aliasLen];
                fis.read(tmpAlias, 0, aliasLen);
                final String strAlias = new String(tmpAlias);
                infoAlias = strAlias;
                tmp1 = fis.read();
                tmp2 = fis.read();
                tmp3 = fis.read();
                tmp4 = fis.read();
                infoLen = (tmp1 << 24) | (tmp2 << 16) | (tmp3 << 8) | tmp4;
                if (infoAlias.compareTo(secureInfoAlias) == 0)
                {
                    tmpInfo = new byte[infoLen];
                    fis.read(tmpInfo, 0, infoLen);
                    final String strInfo = new String(tmpInfo);
                    secureInfo = strInfo;
                    records.add(secureInfo);
                }
                else
                {
                    fis.skip(infoLen);
                }
            }
            else
            {
                fis.skip(tmpAliasLen);
                tmp1 = fis.read();
                tmp2 = fis.read();
                tmp3 = fis.read();
                tmp4 = fis.read();
                infoLen = (tmp1 << 24) | (tmp2 << 16) | (tmp3 << 8) | tmp4;
                fis.skip(infoLen);
            }
            tmp1 = fis.read();
            tmp2 = fis.read();
            tmpAliasLen = (tmp1 << 8) | tmp2;
        }
        fis.close();
        return records;
    }
    
    
   


}
