package org.iso.mpeg.securityTE.test;


import java.io.File;
import java.math.BigInteger;
import java.security.PublicKey;
import java.util.Date;
import java.util.Random;

import javax.crypto.SecretKey;
import javax.xml.bind.JAXBContext;
import javax.xml.bind.Marshaller;

import junit.framework.TestCase;
import morpho.util.Converter;

import org.apache.log4j.Logger;
import org.apache.xml.security.encryption.EncryptedKey;
import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.configuration.MXMConfigurationException;
import org.iso.mpeg.mxm.core.engine.management.MXMEngineException;
import org.iso.mpeg.mxm.engine.securityTE.schema.SecurityEngineContext;
import org.iso.mpeg.mxm.tEngine.utils.SchemaNamespacePrefixMapperImpl;
import org.iso.mpeg.mxm.tEngine.securityTE.SecurityEngine;
import org.iso.mpeg.mxm.tEngine.securityTE.exceptions.SecurityEngineException;
import org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedData;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.CertificateManager;
import org.iso.mpeg.mxm.tEngine.securityTE.technologyhandler.KeyManager;


public class KeyManagerTest extends TestCase
{
    private static final int        cipherKeyLength = 128;
    private static final BigInteger PubExp          = new BigInteger("10001", 16);
    private static final int        rsaKeyLength    = 2048;
    private CertificateManager      certMngr        = null;
    private JAXBContext             jc              = null;
    private KeyManager              keyMngr         = null;
    private final Logger            logger          = Logger
                                                        .getLogger(KeyManagerTest.class);
    private Marshaller              m               = null;
    private SecurityEngine          securityTE      = null;
    private String                  APP_ALIAS       = "1";

    @Override
    protected void setUp() throws Exception
    {
        super.setUp();
        // Initialize JAXB Context.
        this.jc = SecurityEngineContext.getInstance().getJAXBContext();
        this.m = this.jc.createMarshaller();
        this.m.setProperty(Marshaller.JAXB_FORMATTED_OUTPUT, Boolean.TRUE);
        this.m.setProperty("com.sun.xml.bind.namespacePrefixMapper",
            SchemaNamespacePrefixMapperImpl.getInstance());
        // Initialize MXM.
        this.logger.info("\n--- Setup MXMConfiguration ---");
        final File f = new File(this.getClass().getClassLoader()
            .getResource("MXMConfiguration.xml").getFile());
        final MXM mxm = org.iso.mpeg.mxm.core.MXM.getInstance(f);
        mxm.listEnginesAndProperties();
        this.logger.info("\n--- Init SecurityEngine ---");
        this.securityTE = (SecurityEngine)mxm.getDefaultEngine(MXMEngineName.SecurityTE
            .toString());
        this.securityTE.setProperty("conf", "223456");
        this.securityTE.setProperty("auth", "323456");
        this.securityTE.setProperty("sig", "12345678");
        this.securityTE.setProperty("gakp", "12345678");
        this.logger.info("\n--- Done! ---");
        // TODO replace the Mngr with hard coded input values from testSec.xml
        this.certMngr = this.securityTE.getSecurityEngineTechnologyHandler()
            .getCertificateManager();
        this.keyMngr = this.securityTE.getSecurityEngineTechnologyHandler()
            .getKeyManager();
    }

    public void testKeyManager() throws MXMConfigurationException,MXMEngineException
    {
        try
        {
            this.certMngr.generateRSAKeyPair(APP_ALIAS, "conf", 2048);
      
            this.logger.info("\n---------- testKeyManager ----------");
            this.logger.info("\n--- Generating a random plaintext message ---");
            final Date date = new Date();
            final long time = date.getTime();
            final Random r = new Random(time);
            final byte[] plainText = new byte[100];
            r.nextBytes(plainText);
            this.logger.info("\nGenerated Plaintext: " + Converter.toHex(plainText));
            PublicKey pubKey = this.securityTE.getSecurityEngineTechnologyHandler()
                .getAuthenticationManager().getPublicKey(APP_ALIAS, "conf", "");// this.certMngr.generateRSAKeyPair(APP_ALIAS,
                                                                                // "conf",
                                                                                // rsaKeyLength);
            this.logger.info("\n--- generateRandomKey (AES, " + cipherKeyLength
                + "bit) ---");
            final SecretKey secretKey = this.keyMngr.generateRandomKey(null,
                cipherKeyLength);
            this.logger.info("\nRandom Secret Key: "
                + Converter.toHex(secretKey.getEncoded()));
            this.logger.info("\n--- wrap ---");
            // TODO use a certificate (public key) from card for wrapping
            final org.iso.mpeg.mxm.tEngine.securityTE.schemahandler.xenc.EncryptedKey wrappedKey = this.keyMngr
                .wrap(secretKey, null, pubKey);
            this.logger.info("\nWrapped Key: "
                + Converter.toHex(wrappedKey.getCipherData().getCipherValue()));
            this.logger.info("\n--- unwrap ---");
            final SecretKey unwrappedKey = this.keyMngr.unwrap(wrappedKey, APP_ALIAS, "");
            this.logger.info("\nUnwrapped Key: "
                + Converter.toHex(unwrappedKey.getEncoded()));
            this.logger.info("Asserting unwrapped key..");
            assertEquals(Converter.toHex(secretKey.getEncoded()),
                Converter.toHex(unwrappedKey.getEncoded()));
            this.logger.info("\n--- encrypt ---");
            final EncryptedData encryptedMessage = this.keyMngr.encrypt(plainText,
                secretKey, null);
            this.logger.info("\nEncrypted Message: "
                + Converter.toHex(wrappedKey.getCipherData().getCipherValue()));
            this.logger.info("\n--- decrypt ---");
            final byte[] decryptedMessage = this.keyMngr.decrypt(encryptedMessage,
                unwrappedKey, null);
            this.logger.info("\nDecrypted Message: " + Converter.toHex(decryptedMessage));
            assertEquals(Converter.toHex(plainText), Converter.toHex(decryptedMessage));
        }
        catch (final SecurityEngineException e)
        {
            this.logger.error(e.getMessage());
        }
    }
}
