package eu.convergence.middleware.tool.metadata;

import java.io.File;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;

import org.iso.mpeg.mxm.core.MXM;
import org.iso.mpeg.mxm.core.MXMEngineName;
import org.iso.mpeg.mxm.core.MXMException;
import org.iso.mpeg.mxm.tEngine.digitalitemTE.schemahandler.didl_msx.StructuredData;
import org.w3c.dom.Document;
import org.w3c.dom.Node;

import eu.convergence.middleware.core.ConvergenceEngineName;
import eu.convergence.middleware.engine.cdsTE.CdsEngine;
import eu.convergence.middleware.engine.metadataTE.ConvergenceMetadataEngine;
import eu.convergence.middleware.engine.metadataTE.technologyhandler.MetadataKinds;
import eu.convergence.middleware.engine.metadataTE.technologyhandler.RdfMetadataParser;
import eu.convergence.middleware.engine.vdiTE.VdiEngine;

/**
 * The MetadataTool allows the user to create and parse RDF metadata descriptions.
 * The user may create descriptions based on ontologies and the Resource Description
 * Framework producing RDF/XML fragments.
 * In addition, functionalities of CDS TE are exploited in order to provide users with
 * ontology entities and assist the user to build rich semantic descriptions.
 */
public class MetadataTool implements MetadataCreator, MetadataRequester, MetadataWrapper
{
	private CdsEngine cdsEngine;
	private ConvergenceMetadataEngine convMetadataEngine;
	private VdiEngine vdiEngine;
	
	private Node rootNode;
	private RdfMetadataParser rdfMetadataParser;
	
	/**
     * This is the static constructor of the metadata tool that returns a MetadataCreator
     * 
     * @return A MetadataCreator implementation
     */
	public static MetadataCreator newMetadataCreator()
	{
		return new MetadataTool();
	}
	
	/**
     * This is the static constructor of the metadata tool that returns a MetadataRequester
     * 
     * @return A MetadataRequester implementation
     */
	public static MetadataRequester newMetadataRequester()
	{
		return new MetadataTool();
	}
	
	/**
     * This is the static constructor of the metadata tool that returns a MetadataWrapper
     * 
     * @return A MetadataWrapper implementation
     */
	public static MetadataWrapper newMetadataWrapper()
	{
		return new MetadataTool();
	}
	
	private MetadataTool()
	{
		try
		{
			cdsEngine = (CdsEngine) MXM.getInstance().getDefaultEngine(ConvergenceEngineName.CDSTE.toString());
			vdiEngine = (VdiEngine) MXM.getInstance().getDefaultEngine(MXMEngineName.DigitalItemTE.toString());
			convMetadataEngine = (ConvergenceMetadataEngine) MXM.getInstance().getDefaultEngine(ConvergenceEngineName.ConvergenceMetadataTE.toString());
			
			rdfMetadataParser = convMetadataEngine.getConvergenceMetadataTechnologyHandler().getRdfMetadataParser();
		} 
		catch (MXMException e) 
		{
			e.printStackTrace();
		} 
	}
	
	public boolean parseRDF(File rdfFile, String baseURI)
	{
		return rdfMetadataParser.parseRDF(rdfFile, baseURI);
	}
	
	public List<Map<String,List<String>>> requestMetadata(String label, String type, String modelURI)
	{
		return cdsEngine.getEntityExplorer().requestOntologyEntity(label, type, modelURI);
	}
	
//	/**
//     * Adds a field value statement to the description.
//     * User passes a field which should be qualified, and a value object of anyType.
//     * 
//     * @param field The field of the statement
//     * @param value The value of the statement
//     */
//	public void addXMLStatement(String field, Object value)
//	{
//	
//	}
	
	public void addRDFTriple(String subject, String predicate, String object)
	{
		rdfMetadataParser.addRDFTriple(subject, predicate, object);
	}
	
	public void addRDFLiteralTriple(String subject, String predicate, Object value)
	{
		rdfMetadataParser.addRDFLiteralTriple(subject, predicate, value);
	}
	
	public StructuredData generateStructuredData()
	{
		try
		{
			StructuredData structuredData = vdiEngine.getDIEngineSchemaHandler().getStructuredData();
			
			structuredData.setRef(MetadataKinds.RDF.getUri());
			
			rootNode = rdfMetadataParser.writeModel();
			
			DocumentBuilderFactory dbfac = DocumentBuilderFactory.newInstance();
			DocumentBuilder docBuilder = dbfac.newDocumentBuilder();
			Document doc = docBuilder.newDocument();
			doc.adoptNode(rootNode);
			doc.appendChild(rootNode);
			
			structuredData.setAny(doc);
			
			return structuredData;
		}
		catch (Exception ex)
		{
			ex.printStackTrace();
		}
		
		return null;
	}
	
//	/**
//     * Returns a map with all the XML statements of the description.
//     * The keys of fields are qualified names of XML elements.
//     * 
//     * @return A mapping between the XML elements and the value.
//     */
//	public Map<String, Object> getFieldValueMap()
//	{
//		return null;
//	}
	
//	/**
//     * Returns the RDF triples that are stated in the description.
//     * 
//     * @return A mapping between the predicate and the object.
//     */
//	public Map<String, Object> getRDFTriples()
//	{
//		return null;
//	}
	
//	public class Triple
//	{
//		String subject;
//		String predicate;
//		Object object;
//		
//		public Triple(String subject, String predicate, Object object)
//		{
//			this.object = object;
//			this.predicate = predicate;
//			this.subject = subject;
//		}
//	}
	
}
